//
// HDAdditions.h
// -------------------------------------------------------------------------
// Copyright (C) 2007 Andrew Merenbach <http://www.merenbach.com/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------

#import "HDAdditions.h"


NSString *HDMainBundleName() {
	return [[NSBundle mainBundle] objectForInfoDictionaryKey:(NSString *)kCFBundleNameKey];
}

NSString *HDMainBundleIdentifier() {
	return [[NSBundle mainBundle] bundleIdentifier];
}

NSString *HDApplicationSupportDirectory() {
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask, YES);
	NSString *basePath = ([paths count] > 0) ? [paths objectAtIndex:0] : NSTemporaryDirectory();
	NSString *directory = [basePath stringByAppendingPathComponent:HDMainBundleName()];
	
	return directory;
}

NSString *HDPreferencesPath() {
	NSString *filename = [HDMainBundleIdentifier() stringByAppendingPathExtension:@"plist"];

	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES);
	NSString *basePath = ([paths count] > 0) ? [paths objectAtIndex:0] : NSTemporaryDirectory();
	NSString *directory = [basePath stringByAppendingPathComponent:@"Preferences"];
	NSString *file = [directory stringByAppendingPathComponent:filename];
	
	return file;
}



@implementation NSArrayController (HDAdditions)

- (void)removeAllArrangedObjects:(id)sender {
	unsigned int count = [[self arrangedObjects] count];
	if (count > 0) {
		NSIndexSet *indexSet = [[NSIndexSet alloc] initWithIndexesInRange:NSMakeRange(0, count)];
		[self removeObjectsAtArrangedObjectIndexes:indexSet];
		[indexSet release];
	}
}

- (id)objectAtArrangedObjectIndex:(unsigned)idx {
	return [[self arrangedObjects] objectAtIndex:idx];
}

- (NSArray *)valuesForKey:(NSString *)key {
	NSMutableArray *array = [NSMutableArray array];
	
	if (key) {
		NSEnumerator *e = [[self arrangedObjects] objectEnumerator];
		id obj;
		
		while (obj = [e nextObject]) {
			[array addObject:[obj valueForKey:key]];
		}
	}
	
	return array;
}

/*- (void)setValues:(NSArray *)values forKey:(NSString *)key {
	if (key) {
		NSEnumerator *e = [[self arrangedObjects] objectEnumerator];
		id obj;
		
		while (obj = [e nextObject]) {
			
		}
	}
}*/

@end

@implementation NSWorkspace (HDAdditions)

+ (BOOL)deleteFileAtPath:(NSString *)path {
	/*BOOL success = NO;
	
	// Moves 'path' to the trash, a la NSWorkspace.
	if ([[[self class] sharedWorkspace] performFileOperation:NSWorkspaceRecycleOperation
		source:[path stringByDeletingLastPathComponent]
		destination:@""
		files:[NSArray arrayWithObject:[path lastPathComponent]]
		tag:0])
	{
		success = YES;
	}*/
	
	BOOL success = [[NSFileManager defaultManager] removeFileAtPath:path handler:nil];

	return success;
}

@end
