// HDAttributedStringCellFontController.m
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import "HDAttributedStringCellFontController.h"


static NSParagraphStyle *truncatedTailParagraphStyle = nil;
static NSParagraphStyle *truncatedTailParagraphStyleCentered = nil;

static NSDictionary *highlightedBoldFontDictionary = nil;
static NSDictionary *highlightedTinyFontDictionary = nil;
static NSDictionary *highlightedNormalFontDictionary = nil;

static NSDictionary *unhighlightedBoldFontDictionary = nil;
static NSDictionary *unhighlightedTinyFontDictionary = nil;
static NSDictionary *unhighlightedNormalFontDictionary = nil;

static NSDictionary *highlightedBoldFontDictionaryCentered = nil;
static NSDictionary *highlightedTinyFontDictionaryCentered = nil;
static NSDictionary *highlightedNormalFontDictionaryCentered = nil;

static NSDictionary *unhighlightedBoldFontDictionaryCentered = nil;
static NSDictionary *unhighlightedTinyFontDictionaryCentered = nil;
static NSDictionary *unhighlightedNormalFontDictionaryCentered = nil;


@implementation HDAttributedStringCellFontController

+ (NSParagraphStyle *)truncatedTailParagraphStyle {
	if (!truncatedTailParagraphStyle) {
		NSMutableParagraphStyle *paragraphStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
		[paragraphStyle setLineBreakMode:NSLineBreakByTruncatingTail];
		truncatedTailParagraphStyle = [paragraphStyle copy];
		[paragraphStyle release];
	}

	return truncatedTailParagraphStyle;
}

+ (NSParagraphStyle *)truncatedTailParagraphStyleCentered {
	if (!truncatedTailParagraphStyleCentered) {
		NSMutableParagraphStyle *paragraphStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
		[paragraphStyle setLineBreakMode:NSLineBreakByTruncatingTail];
		[paragraphStyle setAlignment:NSCenterTextAlignment];
		truncatedTailParagraphStyleCentered = [paragraphStyle copy];
		[paragraphStyle release];
	}
	return truncatedTailParagraphStyleCentered;
}

+ (NSParagraphStyle *)spacingParagraphStyle:(float)spacing {
	NSMutableParagraphStyle *paragraphStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
	[paragraphStyle setMinimumLineHeight:spacing];
	[paragraphStyle setMaximumLineHeight:spacing];
	
	return [paragraphStyle autorelease];
}

+ (NSDictionary *)highlightedBoldFontDictionary {
	if (!highlightedBoldFontDictionary) {
		highlightedBoldFontDictionary = [[NSDictionary alloc] initWithObjectsAndKeys:
			[NSFont boldSystemFontOfSize:11.0],NSFontAttributeName,
			[NSColor whiteColor],NSForegroundColorAttributeName,
			[[self class] truncatedTailParagraphStyle],NSParagraphStyleAttributeName,
			nil];
	}
	return highlightedBoldFontDictionary;
}

+ (NSDictionary *)highlightedTinyFontDictionary {
	if (!highlightedTinyFontDictionary) {
		highlightedTinyFontDictionary =
			[[NSDictionary alloc] initWithObjectsAndKeys:
				[NSFont systemFontOfSize:10.0],NSFontAttributeName,
				[NSColor whiteColor],NSForegroundColorAttributeName,
				[[self class] truncatedTailParagraphStyle],NSParagraphStyleAttributeName,
				nil];
	}
	return highlightedTinyFontDictionary;
}

+ (NSDictionary *)highlightedNormalFontDictionary {
	if (!highlightedNormalFontDictionary) {
		highlightedNormalFontDictionary =
			[[NSDictionary alloc] initWithObjectsAndKeys:
				[NSFont systemFontOfSize:11.0],NSFontAttributeName,
				[NSColor whiteColor],NSForegroundColorAttributeName,
				[[self class] truncatedTailParagraphStyle],NSParagraphStyleAttributeName,
				nil];
	}
	return highlightedNormalFontDictionary;
}

+ (NSDictionary *)unhighlightedBoldFontDictionary {
	if (!unhighlightedBoldFontDictionary) {
		unhighlightedBoldFontDictionary =
			[[NSDictionary alloc] initWithObjectsAndKeys:
				[NSFont boldSystemFontOfSize:11.0],NSFontAttributeName,
				[[self class] truncatedTailParagraphStyle],NSParagraphStyleAttributeName,
				nil];
	}
	return unhighlightedBoldFontDictionary;
}

+ (NSDictionary *)unhighlightedTinyFontDictionary {
	if (!unhighlightedTinyFontDictionary) {
		unhighlightedTinyFontDictionary =
			[[NSDictionary alloc] initWithObjectsAndKeys:
				[NSFont systemFontOfSize:10.0],NSFontAttributeName,
				[NSColor darkGrayColor],NSForegroundColorAttributeName,
				[[self class] truncatedTailParagraphStyle],NSParagraphStyleAttributeName,
				nil];
	}
	return unhighlightedTinyFontDictionary;
}

+ (NSDictionary *)unhighlightedNormalFontDictionary {
	if (!unhighlightedNormalFontDictionary) {
		unhighlightedNormalFontDictionary =
			[[NSDictionary alloc] initWithObjectsAndKeys:
				[NSFont systemFontOfSize:11.0],NSFontAttributeName,
				[[self class] truncatedTailParagraphStyle],NSParagraphStyleAttributeName,
				nil];
	}
	return unhighlightedNormalFontDictionary;
}


+ (NSDictionary *)highlightedBoldFontDictionaryCentered {
	if (!highlightedBoldFontDictionaryCentered) {
		highlightedBoldFontDictionaryCentered = [[NSDictionary alloc] initWithObjectsAndKeys:
			[NSFont boldSystemFontOfSize:11.0],NSFontAttributeName,
			[NSColor whiteColor],NSForegroundColorAttributeName,
			[[self class] truncatedTailParagraphStyleCentered],NSParagraphStyleAttributeName,
			nil];
	}
	return highlightedBoldFontDictionaryCentered;
}

+ (NSDictionary *)highlightedTinyFontDictionaryCentered {
	if (!highlightedTinyFontDictionaryCentered) {
		highlightedTinyFontDictionaryCentered =
			[[NSDictionary alloc] initWithObjectsAndKeys:
				[NSFont systemFontOfSize:10.0],NSFontAttributeName,
				[NSColor whiteColor],NSForegroundColorAttributeName,
				[[self class] truncatedTailParagraphStyleCentered],NSParagraphStyleAttributeName,
				nil];
	}
	return highlightedTinyFontDictionaryCentered;
}

+ (NSDictionary *)highlightedNormalFontDictionaryCentered {
	if (!highlightedNormalFontDictionaryCentered) {
		highlightedNormalFontDictionaryCentered =
			[[NSDictionary alloc] initWithObjectsAndKeys:
				[NSFont systemFontOfSize:11.0],NSFontAttributeName,
				[NSColor whiteColor],NSForegroundColorAttributeName,
				[[self class] truncatedTailParagraphStyleCentered],NSParagraphStyleAttributeName,
				nil];
	}
	return highlightedNormalFontDictionaryCentered;
}

+ (NSDictionary *)unhighlightedBoldFontDictionaryCentered {
	if (!unhighlightedBoldFontDictionaryCentered) {
		unhighlightedBoldFontDictionaryCentered =
			[[NSDictionary alloc] initWithObjectsAndKeys:
				[NSFont boldSystemFontOfSize:11.0],NSFontAttributeName,
				[[self class] truncatedTailParagraphStyleCentered],NSParagraphStyleAttributeName,
				nil];
	}
	return unhighlightedBoldFontDictionaryCentered;
}

+ (NSDictionary *)unhighlightedTinyFontDictionaryCentered {
	if (!unhighlightedTinyFontDictionaryCentered) {
		unhighlightedTinyFontDictionaryCentered =
			[[NSDictionary alloc] initWithObjectsAndKeys:
				[NSFont systemFontOfSize:10.0],NSFontAttributeName,
				[NSColor darkGrayColor],NSForegroundColorAttributeName,
				[[self class] truncatedTailParagraphStyleCentered],NSParagraphStyleAttributeName,
				nil];
	}
	return unhighlightedTinyFontDictionaryCentered;
}

+ (NSDictionary *)unhighlightedNormalFontDictionaryCentered	{
	if (!unhighlightedNormalFontDictionaryCentered) {
		unhighlightedNormalFontDictionaryCentered =
			[[NSDictionary alloc] initWithObjectsAndKeys:
				[NSFont systemFontOfSize:11.0],NSFontAttributeName,
				[[self class] truncatedTailParagraphStyleCentered],NSParagraphStyleAttributeName,
				nil];
	}
	return unhighlightedNormalFontDictionaryCentered;
}

@end
