//
//  HDDownloadEntry.m
// -------------------------------------------------------------------------
// Copyright (C) 2007 Andrew Merenbach <http://www.merenbach.com/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------



#import "HDDownloadEntry.h"


#import "HDTransferSource.h"	/* [NOTE][MAJOR] only for last three accessors... */

// new
NSString *HDTransferSourceDownloadsPExpandableKey = @"PExpandable";


NSString *HDTransferSourceDownloadsURLKey = @"url";
NSString *HDTransferSourceDownloadsSizeKey = @"size";
NSString *HDTransferSourceDownloadsFileKey = @"file";
NSString *HDTransferSourceDownloadsUserKey = @"user";
NSString *HDTransferSourceDownloadsHashKey = @"hash";
NSString *HDTransferSourceDownloadsStateKey = @"state";
NSString *HDTransferSourceDownloadsStatusKey = @"status";
NSString *HDTransferSourceDownloadsTransmitKey = @"transmit";

NSString *HDTransferSourceDownloadsPSourcesKey = @"PSources";
NSString *HDTransferSourceDownloadsPSourcesDictKey = @"PSourcesDict";

NSString *HDTransferSourceDownloadsPTransferKey = @"PTransfer";
//NSString *HDTransferSourceDownloadsPSizeKey = @"PSize";

NSString *HDTransferSourceDownloadsTotalKey = @"total";

NSString *PZDownloadEntryFileInfoArray = @"PFileUser";
NSString *PZDownloadEntryFileSizeString = @"PSize";

NSString *HDDownloadEntryPIconKey = @"PIcon";

@implementation HDDownloadEntry

- (id)init {
	if (nil != (self = [super init])) {
		[self setEntryDictionary:[NSDictionary dictionary]];
		[self setChildren:[NSArray array]];
	}
	return self;
}

- (void)dealloc {
	[self setEntryDictionary:nil];
	[self setChildren:nil];
	
	[super dealloc];
}

- (NSMutableDictionary *)entryDictionary {
	return [[AM_entryDictionary retain] autorelease];
}

- (void)setEntryDictionary:(NSDictionary *)aDict {
	if (aDict != AM_entryDictionary) {
		[AM_entryDictionary release];
		AM_entryDictionary = [aDict mutableCopy];
	}
}

- (NSMutableArray *)children {
	return [[AM_children retain] autorelease];
}

- (void)setChildren:(NSArray *)anArray {
	if (anArray != AM_children) {
		[AM_children release];
		AM_children = [anArray mutableCopy];
	}
}


@end

@implementation HDDownloadEntry (KeyAccessorMethods)

- (NSArray *)fileInfoArray {
	return [[self entryDictionary] objectForKey:PZDownloadEntryFileInfoArray];
}

- (void)setFileInfoArray:(NSArray *)aString {
	[[self entryDictionary] setObject:aString forKey:PZDownloadEntryFileInfoArray];
}

- (NSString *)fileSizeString {
	return [[self entryDictionary] objectForKey:PZDownloadEntryFileSizeString];
}

- (void)setFileSizeString:(NSString *)aString {
	[[self entryDictionary] setObject:aString forKey:PZDownloadEntryFileSizeString];
}

/*- (NSString *)filenameString {
	return [[AM_filenameString retain] autorelease];
}

- (void)setFilenameString:(NSString *)aString {
	if (aString != AM_filenameString) {
		[AM_filenameString release];
		AM_filenameString = [aString copy];
	}
}*/

- (NSNumber *)PExpandable {
	return [[self entryDictionary] objectForKey:HDTransferSourceDownloadsPExpandableKey];
}

- (void)setPExpandable:(NSNumber *)aNumber {
	[[self entryDictionary] setObject:aNumber forKey:HDTransferSourceDownloadsPExpandableKey];
}


- (NSString *)url {
	return [[self entryDictionary] objectForKey:HDTransferSourceDownloadsURLKey];
}

- (void)setUrl:(NSString *)aString {
	[[self entryDictionary] setObject:aString forKey:HDTransferSourceDownloadsURLKey];
}

- (NSString *)size {
	return [[self entryDictionary] objectForKey:HDTransferSourceDownloadsSizeKey];
}

- (void)setSize:(NSString *)aString {
	[[self entryDictionary] setObject:aString forKey:HDTransferSourceDownloadsSizeKey];
}

- (NSString *)file {
	return [[self entryDictionary] objectForKey:HDTransferSourceDownloadsFileKey];
}

- (void)setFile:(NSString *)aString {
	[[self entryDictionary] setObject:aString forKey:HDTransferSourceDownloadsFileKey];
}

- (NSString *)user {
	return [[self entryDictionary] objectForKey:HDTransferSourceDownloadsUserKey];
}

- (void)setUser:(NSString *)aString {
	NSLog(@"Wow!  We're actually invoking -[HDDownloadEntry setUser:]!");
	[[self entryDictionary] setObject:aString forKey:HDTransferSourceDownloadsUserKey];
}

- (NSString *)hash {
	return [[self entryDictionary] objectForKey:HDTransferSourceDownloadsHashKey];
}

- (void)setHash:(NSString *)aString {
	[[self entryDictionary] setObject:aString forKey:HDTransferSourceDownloadsHashKey];
}


- (NSString *)state {
	return [[self entryDictionary] objectForKey:HDTransferSourceDownloadsStateKey];
}

- (void)setState:(NSString *)aString {
	[[self entryDictionary] setObject:aString forKey:HDTransferSourceDownloadsStateKey];
}



- (NSString *)status {
	return [[self entryDictionary] objectForKey:HDTransferSourceDownloadsStatusKey];
}

- (void)setStatus:(NSString *)aString {
	[[self entryDictionary] setObject:aString forKey:HDTransferSourceDownloadsStatusKey];
}

- (NSString *)transmit {
	return [[self entryDictionary] objectForKey:HDTransferSourceDownloadsTransmitKey];
}

- (void)setTransmit:(NSString *)aString {
	[[self entryDictionary] setObject:aString forKey:HDTransferSourceDownloadsTransmitKey];
}

- (NSArray *)PSources {
	return [[self entryDictionary] objectForKey:HDTransferSourceDownloadsPSourcesKey];
}

- (void)setPSources:(NSArray *)anArray {
	NSLog(@"-[HDDownloadEntry setPSources:] -- we should probably make a mutable copy here...");
	[[self entryDictionary] setObject:anArray forKey:HDTransferSourceDownloadsPSourcesKey];
}

- (NSDictionary *)PSourcesDict {
	return [[self entryDictionary] objectForKey:HDTransferSourceDownloadsPSourcesDictKey];
}

- (void)setPSourcesDict:(NSDictionary *)aDictionary {
	NSLog(@"-[HDDownloadEntry setPSourcesDict:] -- we should probably make a mutable copy here...");
	[[self entryDictionary] setObject:aDictionary forKey:HDTransferSourceDownloadsPSourcesDictKey];
}

- (NSArray *)PTransfer {
	return [[self entryDictionary] objectForKey:HDTransferSourceDownloadsPTransferKey];
}

- (void)setPTransfer:(NSArray *)anArray {
	[[self entryDictionary] setObject:anArray forKey:HDTransferSourceDownloadsPTransferKey];
}

- (NSString *)total {
	return [[self entryDictionary] objectForKey:HDTransferSourceDownloadsTotalKey];
}

- (void)setTotal:(NSString *)aString {
	[[self entryDictionary] setObject:aString forKey:HDTransferSourceDownloadsTotalKey];
}

#pragma mark -

- (NSArray *)PFileUser {
	return [[self entryDictionary] objectForKey:PZDownloadEntryFileInfoArray];
}

- (void)setPFileUser:(NSArray *)anArray {
	[[self entryDictionary] setObject:anArray forKey:PZDownloadEntryFileInfoArray];
}

- (NSString *)PSize {
	return [[self entryDictionary] objectForKey:PZDownloadEntryFileSizeString];
}

- (void)setPSize:(NSString *)aString {
	[[self entryDictionary] setObject:aString forKey:PZDownloadEntryFileSizeString];
}


- (NSImage *)PIcon {
	return [[self entryDictionary] objectForKey:HDDownloadEntryPIconKey];
}

- (void)setPIcon:(NSImage *)anImage {
	[[self entryDictionary] setObject:anImage forKey:HDDownloadEntryPIconKey];
}

#pragma mark -

- (NSNumber *)PTicket {
	return [[self entryDictionary] objectForKey:HDTransferSourcePTicketKey];
}

- (void)setPTicket:(NSNumber *)aNumber {
	[[self entryDictionary] setObject:aNumber forKey:HDTransferSourcePTicketKey];
}

- (NSNumber *)PStatus {
	return [[self entryDictionary] objectForKey:HDTransferSourcePStatusKey];
}

- (void)setPStatus:(NSNumber *)aNumber {
	[[self entryDictionary] setObject:aNumber forKey:HDTransferSourcePStatusKey];
}

- (NSArray *)PProgress {
	return [[self entryDictionary] objectForKey:HDTransferSourcePProgressKey];
}

- (void)setPProgress:(NSArray *)anArray {
	[[self entryDictionary] setObject:anArray forKey:HDTransferSourcePProgressKey];
}

@end
