// HDMediaTypeDetector.m
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import "HDMediaTypeDetector.h"

static NSSet *extensionsForMusicType = nil;
static NSSet *extensionsForVideoType = nil;
static NSSet *extensionsForImageType = nil;
static NSSet *extensionsForDocumentType = nil;


@implementation HDMediaTypeDetector

+ (NSSet *)extensionsForMusicType {
	if (!extensionsForMusicType) {
		extensionsForMusicType = [[NSSet alloc] initWithObjects: @"mp3", @"ogg", @"wma", @"aiff", @"aif", @"au", @"mod", @"wav", @"mid", @"m4a", nil];
	}
	return extensionsForMusicType;
}

+ (NSSet *)extensionsForVideoType {
	if (!extensionsForVideoType) {
		extensionsForVideoType = [[NSSet alloc] initWithObjects: @"mp4", @"avi", @"mov", @"mpg", @"mpeg", @"asf", @"rm", @"ram", @"divx", @"wmv", nil];
	}
	return extensionsForVideoType;
}

+ (NSSet *)extensionsForImageType {
	if (!extensionsForImageType) {
		extensionsForImageType = [[NSSet alloc] initWithObjects: @"gif", @"jpg", @"jpe", @"jpeg", @"png", @"tif", @"tiff", @"bmp", nil];
	}
	return extensionsForImageType;
}

+ (NSSet *)extensionsForDocumentType {
	if (!extensionsForDocumentType) {
		extensionsForDocumentType = [[NSSet alloc] initWithObjects: @"doc", @"pdf", @"rtf", @"rtfd", @"txt", @"htm", @"html", nil];
	}
	return extensionsForDocumentType;
}

+ (PMediaType)mediaTypeForExtension:(NSString *)pathExtension {
	PMediaType mediaType;
    
	Class class = [self class];
	
    if ([[class extensionsForMusicType] member:pathExtension]) mediaType = PMediaTypeMusic;
    else if ([[class extensionsForVideoType] member:pathExtension]) mediaType = PMediaTypeVideo;
    else if ([[class extensionsForImageType] member:pathExtension]) mediaType = PMediaTypeImage;
    else if ([[class extensionsForDocumentType] member:pathExtension]) mediaType = PMediaTypeDocument;
    else mediaType = PMediaTypeFile;
	
	return mediaType;
}

+ (BOOL)mediaTypeIsMusic:(NSString *)pathExtension {
    return ([[self class] mediaTypeForExtension:pathExtension] == PMediaTypeMusic);
}

/*+ (BOOL)mediaTypeIsImage:(NSString *)pathExtension {
    return ([[self class] mediaTypeForExtension:pathExtension] == PMediaTypeImage);
}

+ (BOOL)mediaTypeIsVideo:(NSString *)pathExtension {
    return ([[self class] mediaTypeForExtension:pathExtension] == PMediaTypeVideo);
}

+ (BOOL)mediaTypeIsDocument:(NSString *)pathExtension {
    return ([[self class] mediaTypeForExtension:pathExtension] == PMediaTypeDocument);
}*/


@end