//
// HDNetworkEntry.m
// -------------------------------------------------------------------------
// Copyright (C) 2007 Andrew Merenbach <http://www.merenbach.com/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------

#import "HDNetworkEntry.h"
#import "HDAttributedStringCellFontController.h"

NSString *PZNetworkEntryIconKey = @"iconImage";
NSString *PZNetworkEntrySummaryKey = @"summary";

NSString *PZNetworkEntryProtocolNameKey = @"protocolName";
NSString *PZNetworkEntryNetworkPopulationKey = @"networkPopulation";

const float PNetworkTableEntryInset = 4.0;


@implementation HDNetworkEntry

- (id)init {
	if (nil != (self = [super init])) {
		[self setEntryDictionary:[NSDictionary dictionary]];
		//[self setCellHighlighted:NO];
	}
	return self;
}

- (id)initWithEntryDictionary:(NSDictionary *)aDictionary {
	if (nil != (self = [super init])) {
		[self setEntryDictionary:aDictionary];
		//[self setCellHighlighted:NO];
	}
	return self;
}

- (void)dealloc {
	[self setEntryDictionary:nil];
	[super dealloc];
}

/*- (BOOL)isCellHighlighted {
	return AM_cellHighlighted;
}

- (void)setCellHighlighted:(BOOL)flag {
	[self willChangeValueForKey:@"summary"];
	AM_cellHighlighted = flag;
	[self didChangeValueForKey:@"summary"];
}

- (NSAttributedString *)summary {
	return [self summaryAttributedStringWithHighlight:[self isCellHighlighted]];
}*/

- (NSAttributedString *)summaryAttributedStringWithHighlight:(BOOL)highlighted {
	NSMutableAttributedString *attributedString = [[NSMutableAttributedString alloc] initWithString:@""];

	NSDictionary *boldFont;
	NSDictionary *tinyFont;
	
	if (highlighted) {
		boldFont = [HDAttributedStringCellFontController highlightedBoldFontDictionary];
		tinyFont = [HDAttributedStringCellFontController highlightedTinyFontDictionary];
	} else {
		boldFont = [HDAttributedStringCellFontController unhighlightedBoldFontDictionary];
		tinyFont = [HDAttributedStringCellFontController unhighlightedTinyFontDictionary];
	}
	
	NSAttributedString *spacer = [[NSAttributedString alloc] initWithString:@"\n" attributes:[NSDictionary dictionaryWithObject:[HDAttributedStringCellFontController spacingParagraphStyle:PNetworkTableEntryInset] forKey:NSParagraphStyleAttributeName]];
	NSAttributedString *line1 = [[NSAttributedString alloc] initWithString:[self protocolName]  attributes:boldFont];
	NSAttributedString *newline = [[NSAttributedString alloc] initWithString:@"\n"];
	NSAttributedString *line2 = [[NSAttributedString alloc] initWithString:[self networkPopulation] attributes:tinyFont];
	
	[attributedString appendAttributedString:spacer];
	[attributedString appendAttributedString:line1];
	[attributedString appendAttributedString:newline];
	[attributedString appendAttributedString:line2];

	[spacer release];
	[line1 release];
	[newline release];
	[line2 release];
	
	return [attributedString autorelease];
}

- (NSMutableDictionary *)entryDictionary {
	return [[AM_entryDictionary retain] autorelease];
}

- (void)setEntryDictionary:(NSDictionary *)aDictionary {
	if (aDictionary != AM_entryDictionary) {
		[AM_entryDictionary release];
		AM_entryDictionary = [aDictionary mutableCopy];
	}
}

@end

@implementation HDNetworkEntry (KeyAccessors)

- (NSImage *)iconImage {
	return [[self entryDictionary] objectForKey:PZNetworkEntryIconKey];
}

- (void)setIconImage:(NSImage *)anImage {
	[[self entryDictionary] setObject:anImage forKey:PZNetworkEntryIconKey];
}

- (NSArray *)protocolArray {
	return [[self entryDictionary] objectForKey:PZNetworkEntrySummaryKey];
}

- (void)setProtocolArray:(NSArray *)anArray {
	[[self entryDictionary] setObject:anArray forKey:PZNetworkEntrySummaryKey];
}

- (NSString *)protocolName {
	return [[self entryDictionary] objectForKey:PZNetworkEntryProtocolNameKey];
}

- (void)setProtocolName:(NSString *)aString {
	[[self entryDictionary] setObject:aString forKey:PZNetworkEntryProtocolNameKey];
}

- (NSString *)networkPopulation {
	return [[self entryDictionary] objectForKey:PZNetworkEntryNetworkPopulationKey];
}

- (void)setNetworkPopulation:(NSString *)aString {
	[[self entryDictionary] setObject:aString forKey:PZNetworkEntryNetworkPopulationKey];
}

@end
