//
//  HDSearchListEntry.m
// -------------------------------------------------------------------------
// Copyright (C) 2007 Andrew Merenbach <http://www.merenbach.com/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------


#import "HDSearchListEntry.h"
#import "HDAttributedStringCellFontController.h"

const float PSearchListEntryInset = 1.0;


NSString *PZSearchListEntryDictionaryKey = @"entryDictionary";
NSString *PZSearchListEntryEntryDataSourceKey = @"entryDataSource";

NSString *PZSearchListEntryTicketStringKey = @"ticketString";
NSString *PZSearchListEntryResultCountKey = @"resultCount";
NSString *PZSearchListEntryQueryStringKey = @"queryString";
NSString *PZSearchListEntryInfoStringKey = @"infoString";

NSString *PZSearchListEntrySummaryKey = @"summary";


@implementation HDSearchListEntry

- (id)init {
	if (nil != (self = [super init])) {
		[self setEntryDictionary:[NSDictionary dictionary]];
		
		[self setTicketString:@""];
		[self setQueryString:@""];
		[self setResultCount:@""];
		[self setInfoString:@""];
		
		[self setEntryDataSource:nil];
	}
	return self;
}

- (void)dealloc {
	[self setEntryDictionary:nil];
	[self setEntryDataSource:nil];
	
	[super dealloc];
}

- (NSMutableDictionary *)entryDictionary {
	return [[AM_entryDictionary retain] autorelease];
}

- (void)setEntryDictionary:(NSDictionary *)aDictionary {
	if (aDictionary != AM_entryDictionary) {
		[AM_entryDictionary release];
		AM_entryDictionary = [aDictionary mutableCopy];
	}
}

- (NSAttributedString *)summaryAttributedStringWithHighlight:(BOOL)highlighted {
	NSMutableAttributedString *attributedString = [[NSMutableAttributedString alloc] initWithString:@""];

	NSDictionary *boldFont;
	NSDictionary *tinyFont;
	
	if (highlighted) {
		boldFont = [HDAttributedStringCellFontController highlightedBoldFontDictionary];
		tinyFont = [HDAttributedStringCellFontController highlightedTinyFontDictionary];
	} else {
		boldFont = [HDAttributedStringCellFontController unhighlightedBoldFontDictionary];
		tinyFont = [HDAttributedStringCellFontController unhighlightedTinyFontDictionary];
	}
	
	NSAttributedString *spacer = [[NSAttributedString alloc] initWithString:@"\n" attributes:[NSDictionary dictionaryWithObject:[HDAttributedStringCellFontController spacingParagraphStyle:PSearchListEntryInset] forKey:NSParagraphStyleAttributeName]];
	NSAttributedString *line1 = [[NSAttributedString alloc] initWithString:[self infoString] attributes:boldFont];
	NSAttributedString *newline = [[NSAttributedString alloc] initWithString:@"\n"];
	NSAttributedString *line2 = [[NSAttributedString alloc] initWithString:[self resultCount] attributes:tinyFont];

	[attributedString appendAttributedString:spacer];
	[attributedString appendAttributedString:line1];
	[attributedString appendAttributedString:newline];
	[attributedString appendAttributedString:line2];

	[spacer release];
	[line1 release];
	[newline release];
	[line2 release];

	return [attributedString autorelease];
}

- (id)entryDataSource {
	return [[AM_entryDataSource retain] autorelease];
}

- (void)setEntryDataSource:(id)aDataSource {
	if (aDataSource != AM_entryDataSource) {
		[AM_entryDataSource release];
		AM_entryDataSource = [aDataSource retain];
	}
}

@end


@implementation HDSearchListEntry (KeyAccessorMethods)

- (NSString *)ticketString {
	return [[self entryDictionary] objectForKey:PZSearchListEntryTicketStringKey];
}

- (void)setTicketString:(NSString *)aString {
	[[self entryDictionary] setObject:aString forKey:PZSearchListEntryTicketStringKey];
}

- (NSString *)queryString {
	return [[self entryDictionary] objectForKey:PZSearchListEntryQueryStringKey];
}

- (void)setQueryString:(NSString *)aString {
	[[self entryDictionary] setObject:aString forKey:PZSearchListEntryQueryStringKey];
}

- (NSString *)resultCount {	// [TODO] make NSNumber? in resultCount we have an "x of y" string, so could we use a value transformer?
	return [[self entryDictionary] objectForKey:PZSearchListEntryResultCountKey];
}

- (void)setResultCount:(NSString *)aString {
	//[self willChangeValueForKey:PZSearchListEntrySummaryKey];
	[[self entryDictionary] setObject:aString forKey:PZSearchListEntryResultCountKey];
	//[self didChangeValueForKey:PZSearchListEntrySummaryKey];
}

- (NSString *)infoString {
	return [[self entryDictionary] objectForKey:PZSearchListEntryInfoStringKey];
}

- (void)setInfoString:(NSString *)aString {
	//[self willChangeValueForKey:PZSearchListEntrySummaryKey];
	[[self entryDictionary] setObject:aString forKey:PZSearchListEntryInfoStringKey];
	//[self didChangeValueForKey:PZSearchListEntrySummaryKey];
}

@end
