//
//  HDValueTransformers.m
// -------------------------------------------------------------------------
// Copyright (C) 2007 Andrew Merenbach <http://www.merenbach.com/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2003 Poisoned Project <http://www.gottsilla.net/>
// ---------------------------------------------------------------------------


#import "HDValueTransformers.h"
#import "HDAdditions.h"


@implementation PZSearchFilterStateValueTransformer
+ (Class)transformedValueClass { return [NSString class]; }
+ (BOOL)allowsReverseTransformation { return NO; }
- (id)transformedValue:(id)value {
	NSString *returnValue = NSLocalizedString(@"no search selected", @"no search selected"); 
	if (value) {
		BOOL state = [value boolValue];
		returnValue = (state ? NSLocalizedString(@"on", @"on") : NSLocalizedString(@"off", @"off"));
	}
	
	return returnValue;
}
@end

/*@implementation PZSearchImageValueTransformer
- (id)init {
	if (nil != (self = [super init])) {
		
	}
	return self;
}

+ (Class)transformedValueClass { return [NSAttributedString class]; }
+ (BOOL)allowsReverseTransformation { return NO; }
- (id)transformedValue:(id)value {
	NSAttributedString *attributedString = nil;
	if (value) {
		BOOL isActive = [[value valueForKey:@"entryDataSource"] isActive];
		image = (isActive ? [[]])
	}
	return attributedString;
}
@end*/

@implementation HDUploadPreferencesIconValueTransformer
+ (Class)transformedValueClass { return [NSImage class]; }
+ (BOOL)allowsReverseTransformation { return NO; }
- (id)transformedValue:(id)value {
	id returnValue = nil;
	if (value) {
		returnValue = [[NSWorkspace sharedWorkspace] iconForFile:[value stringByExpandingTildeInPath]];
	}
    return returnValue;
}
@end

@implementation PZProtocolImageValueTransformer
+ (Class)transformedValueClass { return [NSImage class]; }
+ (BOOL)allowsReverseTransformation { return NO; }
- (id)transformedValue:(id)value {
    return (value != nil) ? [NSImage imageNamed:value] : nil;
}
@end

/*@implementation PZNetworksTableTitleDescriptionValueTransformer

- (id)init {
	if (nil != (self = [super init])) {
		[self prepareFonts];
	}
	return self;
}

- (void)dealloc {
	[highlightedBoldFont release];
	[highlightedNormalFont release];
	[highlightedTinyFont release];

	[unhighlightedBoldFont release];
	[unhighlightedNormalFont release];
	[unhighlightedTinyFont release];

	highlightedBoldFont = nil;
	highlightedNormalFont = nil;
	highlightedTinyFont = nil;

	unhighlightedBoldFont = nil;
	unhighlightedNormalFont = nil;
	unhighlightedTinyFont = nil;
	
	[super dealloc];
}

- (void)prepareFonts {
	NSMutableParagraphStyle *paragraphStyle = [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
	[paragraphStyle setLineBreakMode:NSLineBreakByTruncatingTail];
	
	highlightedBoldFont =
		[[NSDictionary alloc] initWithObjectsAndKeys:
			[NSFont boldSystemFontOfSize:11.0], NSFontAttributeName,
			[NSColor whiteColor], NSForegroundColorAttributeName,
			paragraphStyle, NSParagraphStyleAttributeName,
		nil];
	highlightedNormalFont =
		[[NSDictionary alloc] initWithObjectsAndKeys:
			[NSFont systemFontOfSize:11.0],NSFontAttributeName,
			[NSColor whiteColor], NSForegroundColorAttributeName,
			paragraphStyle, NSParagraphStyleAttributeName,
		nil];
	highlightedTinyFont =
		[[NSDictionary alloc] initWithObjectsAndKeys:
			[NSFont systemFontOfSize:10.0], NSFontAttributeName,
			[NSColor whiteColor], NSForegroundColorAttributeName,
			paragraphStyle, NSParagraphStyleAttributeName,
		nil];

	unhighlightedBoldFont =
		[[NSDictionary alloc] initWithObjectsAndKeys:
			[NSFont boldSystemFontOfSize:11.0], NSFontAttributeName,
			paragraphStyle, NSParagraphStyleAttributeName,
		nil];
	unhighlightedNormalFont =
		[[NSDictionary alloc] initWithObjectsAndKeys:
			[NSFont systemFontOfSize:11.0], NSFontAttributeName,
			paragraphStyle, NSParagraphStyleAttributeName,
		nil];
	unhighlightedTinyFont =
		[[NSDictionary alloc] initWithObjectsAndKeys:
			[NSFont systemFontOfSize:10.0], NSFontAttributeName,
			//[NSColor darkGrayColor], NSForegroundColorAttributeName,
			paragraphStyle, NSParagraphStyleAttributeName,
		nil];
	
	[paragraphStyle release];
}

+ (Class)transformedValueClass { return [NSAttributedString class]; }
+ (BOOL)allowsReverseTransformation { return NO; }
- (id)transformedValue:(id)value {
	id returnValue = nil;
	if (value) {
		NSMutableAttributedString *attributedString = [[[NSMutableAttributedString alloc] initWithString:@""] autorelease];
	
		if ([value respondsToSelector:@selector(count)]) {
			NSDictionary *boldFont;
			NSDictionary *normalFont;
			NSDictionary *tinyFont;
			
			//if ([self isHighlighted] && ([self highlightColorWithFrame:cellFrame inView:controlView] != [NSColor secondarySelectedControlColor])) {
				//boldFont = highlightedBoldFont;
				//normalFont = highlightedNormalFont;
				//tinyFont = highlightedTinyFont;
			//} else {
				boldFont = unhighlightedBoldFont;
				normalFont = unhighlightedNormalFont;
				tinyFont = unhighlightedTinyFont;
			//}

			if ([[value objectAtIndex:0] boolValue]) {//[value respondsToSelector:@selector(count)]) {
				NSAttributedString *line1 = [[NSAttributedString alloc] initWithString:[value objectAtIndex:1] attributes:boldFont];
				NSAttributedString *newline = [[NSAttributedString alloc] initWithString:@"\n"];
				NSAttributedString *line2 = [[NSAttributedString alloc] initWithString:[value objectAtIndex:2] attributes:tinyFont];

				[attributedString appendAttributedString:line1];
				[attributedString appendAttributedString:newline];
				[attributedString appendAttributedString:line2];
				
				[line1 release];
				[newline release];
				[line2 release];
			} else {
				NSAttributedString *line1 = [[NSAttributedString alloc] initWithString:[value objectAtIndex:1] attributes:normalFont];
				[attributedString appendAttributedString:line1];
				[line1 release];
			}
		}
		returnValue = [[attributedString copy] autorelease];
	}
	return returnValue;
}
@end*/

/*@implementation PZSearchTableTitleDescriptionValueTransformer

- (id)init {
	if (nil != (self = [super init])) {
		[self prepareFonts];
	}
	return self;
}

- (void)dealloc {
	[highlightedBoldFont release];
	[highlightedTinyFont release];

	[unhighlightedBoldFont release];
	[unhighlightedTinyFont release];

	highlightedBoldFont = nil;
	highlightedTinyFont = nil;

	unhighlightedBoldFont = nil;
	unhighlightedTinyFont = nil;
	
	[super dealloc];
}

- (void)prepareFonts {
	NSMutableParagraphStyle *paragraphStyle= [[NSParagraphStyle defaultParagraphStyle] mutableCopy];
	[paragraphStyle setLineBreakMode:NSLineBreakByTruncatingTail];
	
	highlightedBoldFont =
		[[NSDictionary alloc] initWithObjectsAndKeys:
			[NSFont boldSystemFontOfSize:11.0],NSFontAttributeName,
			[NSColor whiteColor],NSForegroundColorAttributeName,
			paragraphStyle,NSParagraphStyleAttributeName,
		nil];
	highlightedTinyFont =
		[[NSDictionary alloc] initWithObjectsAndKeys:
			[NSFont systemFontOfSize:10.0],NSFontAttributeName,
			[NSColor whiteColor],NSForegroundColorAttributeName,
			paragraphStyle,NSParagraphStyleAttributeName,
		nil];
	
	unhighlightedBoldFont =
			[[NSDictionary alloc] initWithObjectsAndKeys:
				[NSFont boldSystemFontOfSize:11.0],NSFontAttributeName,
				paragraphStyle,NSParagraphStyleAttributeName,
			nil];
	unhighlightedTinyFont =
			[[NSDictionary alloc] initWithObjectsAndKeys:
				[NSFont systemFontOfSize:10.0],NSFontAttributeName,
				//[NSColor darkGrayColor],NSForegroundColorAttributeName,
				paragraphStyle,NSParagraphStyleAttributeName,
			nil];
			
	[paragraphStyle release];
}

+ (Class)transformedValueClass { return [NSAttributedString class]; }
+ (BOOL)allowsReverseTransformation { return NO; }
- (id)transformedValue:(id)value {
	id returnValue = nil;
	if (value) {
		NSMutableAttributedString *attributedString = [[[NSMutableAttributedString alloc] initWithString:@""] autorelease];
	
		if ([value respondsToSelector:@selector(count)]) {
			NSDictionary *boldFont;
			NSDictionary *tinyFont;
			
			//if ([self isHighlighted] && ([self highlightColorWithFrame:cellFrame inView:controlView] != [NSColor secondarySelectedControlColor])) {
				//boldFont = highlightedBoldFont;
				//normalFont = highlightedNormalFont;
				//tinyFont = highlightedTinyFont;
			//} else {
				boldFont = unhighlightedBoldFont;
				tinyFont = unhighlightedTinyFont;
			//}

			NSAttributedString *line1 = [[NSAttributedString alloc] initWithString:[value objectAtIndex:0] attributes:boldFont];
			NSAttributedString *newline = [[NSAttributedString alloc] initWithString:@"\n"];
			NSString *line2base = [[NSString alloc] initWithFormat:@"(%@)", [value objectAtIndex:1]];
				NSAttributedString *line2 = [[NSAttributedString alloc] initWithString:line2base attributes:tinyFont];
			[line2base release];
			
			[attributedString appendAttributedString:line1];
			[attributedString appendAttributedString:newline];
			[attributedString appendAttributedString:line2];
			
			[line1 release];
			[newline release];
			[line2 release];
		}
		returnValue = [[attributedString copy] autorelease];
	}
	return returnValue;
}
@end*/

@implementation PZSizeValueTransformer
+ (Class)transformedValueClass { return [NSString class]; }
+ (BOOL)allowsReverseTransformation { return NO; }
- (id)transformedValue:(id)value {
	NSString *returnValue = @"";
	if (value) {
		long long size;
		[[NSScanner scannerWithString:value] scanLongLong:&size];
		if (size < HD_KILOBYTE)
			returnValue = [NSString stringWithFormat:@"%i B",(int)size];
		else if (size < HD_MEGABYTE)
			returnValue = [NSString stringWithFormat:@"%i kB",(int)floor(size/(float)HD_KILOBYTE)];
		else if (size < HD_GIGABYTE)
			returnValue = [NSString stringWithFormat:@"%.2f MB",(size/(float)HD_MEGABYTE)];
		else
			returnValue = [NSString stringWithFormat:@"%.2f GB",(size/(float)HD_GIGABYTE)];
	}
	
	return returnValue;
}
@end

@implementation PZBitrateValueTransformer
+ (Class)transformedValueClass { return [NSString class]; }
+ (BOOL)allowsReverseTransformation { return NO; }
- (id)transformedValue:(id)value {
	NSString *returnValue = @"";
	if (value) {
		int bitrate = [value intValue];
		if (bitrate<10000)
			returnValue = [NSString stringWithFormat:@"%i kbps",bitrate];
		else if (bitrate<999999)
			returnValue = [NSString stringWithFormat:@"%i kbps",bitrate/1000];
		else
			returnValue = @"";
	}
	
	return returnValue;
}
@end


@implementation PZDurationValueTransformer
+ (Class)transformedValueClass { return [NSString class]; }
+ (BOOL)allowsReverseTransformation { return NO; }
- (id)transformedValue:(id)value {
	NSString *returnValue = @"";
	if (value) {
		// is it ok to just parse for an integer??? -rizzi
		int s = [value intValue];
		if (s<60)
			returnValue = [NSString stringWithFormat:@"0:%02d",s];
		else if (s<3600)
			returnValue = [NSString stringWithFormat:@"%i:%02d",s/60,s%60];
		else {
			int hrs = s/3600;
			s -= hrs*3600;
			returnValue = [NSString stringWithFormat:@"%i:%02d:%02d",hrs,s/60,s%60];
		}
	}
	
	return returnValue;
}

@end
