//
// HDAppleScriptSongPlayer.m
// -------------------------------------------------------------------------
// Copyright (C) 2007 Andrew Merenbach <http://www.merenbach.com/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Note that the scripts employed by this class are
//	Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
// ---------------------------------------------------------------------------

#import "HDAppleScriptSongPlayer.h"
#import "HDAdditions.h"

NSString *PZAppleScriptSongPlayerOSAScriptPath = @"/usr/bin/osascript";
NSString *PZAppleScriptSongPlayerScriptFilename = @"play-song";
NSString *PZAppleScriptSongPlayerScriptFileExtension = @"applescript";


@implementation HDAppleScriptSongPlayer

- (id)init {
	if (nil != (self = [super init])) {
		[self setFilePath:nil];
		[self setShellWrapper:nil];
	}
	return self;
}

- (id)initWithPath:(NSString *)path {
	if (nil != (self = [super init])) {
		[self setFilePath:path];
		[self setShellWrapper:nil];
	}
	return self;
}

- (void)dealloc {
	[self setFilePath:nil];
	[self setShellWrapper:nil];
	
	[super dealloc];
}

- (void)playSong:(BOOL)flag;
{
	NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
	
	NSString *path = [self filePath];

	NSString *playStateString;
	if (flag) {
		playStateString = @"true";
	} else {
		playStateString = @"false";
	}

	NSString *playlistStateString;
	if ([userDefaults boolForKey:@"HDIntegrationAddSongToPlaylist"]) {
		playlistStateString = @"true";
	} else {
		playlistStateString = @"false";
	}

	NSString *playlistName = [userDefaults stringForKey:@"HDIntegrationImportPlaylistName"];
	NSString *scriptPath = [[NSBundle mainBundle] pathForResource:PZAppleScriptSongPlayerScriptFilename ofType:PZAppleScriptSongPlayerScriptFileExtension];

	if (path) {
		NSArray *args = nil;
		
		args = [NSArray arrayWithObjects:
			PZAppleScriptSongPlayerOSAScriptPath,
			scriptPath,
			playStateString,
			playlistStateString,
			playlistName,
			nil];

		AMShellWrapper *shellWrapper = [[AMShellWrapper alloc] initWithController:self
			inputPipe:nil
			outputPipe:nil
			errorPipe:nil
			workingDirectory:@"."
			environment:nil
			arguments:args];
		
		[self setShellWrapper:shellWrapper];
		[shellWrapper release];

		@try {
			if (shellWrapper) {
				[shellWrapper setOutputStringEncoding:NSUTF8StringEncoding];
				[shellWrapper startProcess];
			}
		}
		@catch (NSException *e) {
			NSLog(@"Caught %@: %@", [e name], [e reason]);
			//[self processLaunchException:e];
		}
	}
}

- (NSString *)filePath {
	return [[AM_filePath retain] autorelease];
}

- (void)setFilePath:(NSString *)aString {
	if (aString != AM_filePath) {
		[AM_filePath release];
		AM_filePath = [aString copy];
	}
}

- (AMShellWrapper *)shellWrapper {
	return [[AM_shellWrapper retain] autorelease];
}

- (void)setShellWrapper:(AMShellWrapper *)aShellWrapper {
	if (aShellWrapper != AM_shellWrapper) {
		[AM_shellWrapper release];
		AM_shellWrapper = [aShellWrapper retain];
	}
}

#pragma mark -
#pragma mark AMShellWrapperController protocol

// implement this protocol to control your AMShellWrapper object:

- (void)appendOutput:(NSString *)output {
	// do nothing
}

// output from stdout

- (void)appendError:(NSString *)error {
	NSLog(@"An error occurred: %@", error);
}

// output from stderr

- (void)processStarted:(id)sender {
	// do nothing
}
// This method is a callback which your controller can use to do other initialization
// when a process is launched.

- (void)processFinished:(id)sender withTerminationStatus:(int)resultCode {
	if ([[NSUserDefaults standardUserDefaults] boolForKey:@"HDIntegrationDeleteAfterImport"]) {
		[NSWorkspace deleteFileAtPath:[self filePath]];
	}
	[self setFilePath:nil];
	[self setShellWrapper:nil];
}
// This method is a callback which your controller can use to do other cleanup
// when a process is halted.

// AMShellWrapper posts a AMShellWrapperProcessFinishedNotification when a process finished.
// The userInfo of the notification contains the corresponding NSTask ((NSTask *), key @"task")
// and the result code ((NSNumber *), key @"resultCode")
// ! notification removed since it prevented the task from getting deallocated

//#pragma mark -

/*- (void)processLaunchException:(NSException *)exception
{
	NSLog(@"An exception occured");
}*/

@end
