//
// HDAspectControllerDaemon.m
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import "HDAspectControllerDaemon.h"
#import "HDDaemonCommander.h"
#import "HDAdditions.h"
#import "HDMainController.h"
#import "FileHandle.h"
#import "HDAdditions.h"
#import "HDValueTransformers.h"

#import "PIconShop.h"

#import "HDNetworkEntry.h"

#import "HDAlertLocalizedStringKeys.h"
#import "HDProcessLister.h"
#import "HDMainToolbarCategory.h"

const int PDaemonControllerMaxCrashes = 5;

const NSTimeInterval HDDaemonRunningTestTimeInterval = 2.0;

@implementation HDAspectControllerDaemon

+ (void)initialize {
	NSValueTransformer *vt;
	
	vt = [[PZProtocolImageValueTransformer alloc] init];
	[NSValueTransformer setValueTransformer:vt forName:@"ProtocolImageValueTransformer"];
	[vt release];
	
	/*vt = [[PZNetworksTableTitleDescriptionValueTransformer alloc] init];
	[NSValueTransformer setValueTransformer:vt forName:@"NetworksTableTitleDescriptionValueTransformer"];
	[vt release];*/
}

- (id)init {
	if (nil != (self = [super init])) {
		// prepare rows for various states
		{
			[self am_setDaemonRunningCache:NO];
			daemonRunningTimer = [[NSTimer scheduledTimerWithTimeInterval:HDDaemonRunningTestTimeInterval
					target:self
					selector:@selector(recacheDaemonRunning:)
					userInfo:nil
					repeats:YES]
				retain];
		
			NSDictionary *rowDict;
			
			rowDict = [[NSDictionary alloc] initWithObjectsAndKeys:
				@"poisoned.tiff", PZNetworkEntryIconKey,
				NSLocalizedString(@"Not connected.", @"Not connected."), PZNetworkEntryProtocolNameKey,
				@"", PZNetworkEntryNetworkPopulationKey,
				nil];
			
			entryForNotConnected = [[HDNetworkEntry alloc] initWithEntryDictionary:rowDict];
			[rowDict release];
			
			
			rowDict = [[NSDictionary alloc] initWithObjectsAndKeys:
				@"Stop", PZNetworkEntryIconKey,
				NSLocalizedString(@"Stopping giFT", @"Stopping giFT"), PZNetworkEntryProtocolNameKey,
				NSLocalizedString(@"This may take a while...", @"This may take a while..."), PZNetworkEntryNetworkPopulationKey,
				nil];
				
			entryForStoppingGiFT = [[HDNetworkEntry alloc] initWithEntryDictionary:rowDict];
			[rowDict release];
						
			rowDict = [[NSDictionary alloc] initWithObjectsAndKeys:
				@"Power", PZNetworkEntryIconKey,
				NSLocalizedString(@"Starting giFT", @"Starting giFT"), PZNetworkEntryProtocolNameKey,
				@"", PZNetworkEntryNetworkPopulationKey,
				nil];
				
			entryForStartingGiFT = [[HDNetworkEntry alloc] initWithEntryDictionary:rowDict];
			[rowDict release];
			
			rowDict = [[NSDictionary alloc] initWithObjectsAndKeys:
				@"Red Ball", PZNetworkEntryIconKey,
				NSLocalizedString(@"Disconnecting from the daemon...", @"Disconnecting from the daemon..."), PZNetworkEntryProtocolNameKey,
				@"", PZNetworkEntryNetworkPopulationKey,
				nil];
			
			entryForDisconnectingFromDaemon = [[HDNetworkEntry alloc] initWithEntryDictionary:rowDict];
			[rowDict release];
					
			rowDict = [[NSDictionary alloc] initWithObjectsAndKeys:
				@"Green Ball", PZNetworkEntryIconKey,
				NSLocalizedString(@"Connecting to the daemon...", @"Connecting to the daemon..."), PZNetworkEntryProtocolNameKey,
				@"", PZNetworkEntryNetworkPopulationKey,
				nil];
				
			entryForConnectingToDaemon = [[HDNetworkEntry alloc] initWithEntryDictionary:rowDict];
			[rowDict release];
		}
		
		[self setAttachString:nil];
		
		if (![NSBundle loadNibNamed:@"AspectControllerDaemon" owner:self]) {
            [self release];
            self = nil;
        }
	}
	return self;
}

- (void)dealloc
{
	// [AM] begin new stuff
	{
		[daemonRunningTimer invalidate];	// will also release
		//[daemonRunningTimer release];
		daemonRunningTimer = nil;
	
		[entryForNotConnected release];
		entryForNotConnected = nil;
		
		[entryForStoppingGiFT release];
		entryForStoppingGiFT = nil;
		
		[entryForStartingGiFT release];
		entryForStartingGiFT = nil;
		
		[entryForDisconnectingFromDaemon release];
		entryForDisconnectingFromDaemon = nil;
	}
		// things loaded from the nib
		[protocolArrayController release];
		protocolArrayController = nil;
				
		[connectionSheet release];
		connectionSheet = nil;

		/*[remoteAddress release];
		remoteAddress = nil;

		[remotePort release];
		remotePort = nil;*/

		//[protocolTableView release];
		//protocolTableView = nil;
		
	// [AM] end new stuff



	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[icon_shop release];
	[self setAttachString:nil];
	[task release];
	
	[super dealloc];
}

- (void)awakeFromNib
{
	// first check if all needed files are in ~/Library/Application Support/Poisoned
	// if not copy them
	[self checkConfFiles];
	
	userDefaults = [NSUserDefaults standardUserDefaults];
	

	
	// for the protocol icons
	icon_shop = [[PIconShop alloc] init];
	
	// setup the table
	//[protocolTable setDelegate:self];
	//[protocolTable setDrawsGrid:YES];
	//[protocolTable setRowHeight:34];
	/*id networkCell = [[PNetworkTableTitleDescriptionCell alloc] init];
	[[protocolTableView tableColumnWithIdentifier:PZNetworkEntrySummaryKey] setDataCell:networkCell];
	[networkCell release];*/
	/*[[protocolTable tableColumnWithIdentifier:PZNetworkEntryIconKey] setDataCell:
		[[[NSImageCell alloc] init] autorelease]
	];*/

	// not connected at startup
	[protocolArrayController removeAllArrangedObjects:nil];
	[protocolArrayController addObject:entryForNotConnected];
	
	// getting ready for launching/connecting
	task = [[NSTask alloc] init];
	AM_synchronizing = NO;
	AM_startingDaemon = NO;
	AM_connectOnce=NO;
	
	// counting crashes in a row
	
	crashes=0;
	
	// NOTE: i moved checking of HDDaemonAutoConnect to setCommander:andController
	// because the commander is needed before it can attempt a connect - jjt
	// NOTE2: alsoe moved checking of HDDaemonAutoLaunch, because i wasn't sure if it
	// could happen that we try to connect to the launched daemon when the
	// commander isn't set yet
}

- (void)registerWithCommander
{
	[super registerWithCommander];

	[[HDDaemonCommander sharedInstance] registerController:self forCommands:
		[NSArray arrayWithObjects:
			@"ATTACH",
			@"STATS",
			@"SHARE",
			nil]
	];
	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(connectionDidFail:) name:@"DaemonConnectionTimedOut" object:nil];	// object:[HDDaemonCommander sharedInstance]

	if ([userDefaults boolForKey:@"HDDaemonAutoLaunch"]) {
		[self launchDaemon:self];
	}
	// now that we have the commander, we can attempt an auto connect
	else if ([userDefaults boolForKey:@"HDDaemonAutoConnect"]) {
		[self connectToLocalDaemon:nil];
	}
}

// this method gets invoked every 3 secs when we're connected to the daemon
- (void)periodicalUpdate
{
	[[HDDaemonCommander sharedInstance] executeCommand:@"STATS"];
}

// this is called by a notification when a connection with the daemon is made
- (void)didConnect:(NSNotification *)aNotification
{
	AM_connectOnce = NO;
	AM_startingDaemon = NO;
	
	[[NSApp delegate] validateToolbar];
	
	// set all gui stuff	
	id appDelegate = [[NSApplication sharedApplication] delegate];
	[[appDelegate statusImageView] setImage:[NSImage imageNamed:@"online.tiff"]];
	[[appDelegate statusImageView] setNeedsDisplay:YES];
	[[appDelegate localConnectMenuItem] setAction:nil];
	[[appDelegate remoteConnectMenuItem] setAction:nil];
	
	// attach the client
	AM_protosSend = NO; // see STATS:
	id theCommander = [HDDaemonCommander sharedInstance];
	// we want non-localized versions, so we do *not* use -[NSBundle objectForInfoDictionaryKey:]...
	NSString *bundleName = [[[NSBundle mainBundle] infoDictionary] objectForKey:(NSString *)kCFBundleNameKey];
	NSString *bundleVersion = [[[NSBundle mainBundle] infoDictionary] objectForKey:(NSString *)kCFBundleVersionKey];
	[theCommander executeCommand:[NSString stringWithFormat:@"ATTACH client(%@) version(%@)", bundleName, bundleVersion]];
	[theCommander executeCommand:@"STATS"];
	
	// setup periodical update (STATS)
	NSMethodSignature *sig = [[self class] instanceMethodSignatureForSelector:@selector(periodicalUpdate)];
	NSInvocation *invoc = [NSInvocation invocationWithMethodSignature:sig];
	[invoc setTarget:self];
	[invoc setSelector:@selector(periodicalUpdate)];
	(void)[NSTimer scheduledTimerWithTimeInterval:1.0 invocation:invoc repeats:NO];	// [AM] what's this for?
	(void)[NSTimer scheduledTimerWithTimeInterval:2.0 invocation:invoc repeats:NO];	// [AM] what's this for?
	timer = [NSTimer scheduledTimerWithTimeInterval:3.0 invocation:invoc repeats:YES];
}

// this is called by a notification when the aonnection timed out
// this method also gets invoked in self, if something goes wrong while connecting
- (void)connectionDidFail:(NSNotification *)aNotification
{
	AM_connectOnce=NO;
	AM_startingDaemon = NO;
	
	// if we just launched giFT, it's possible that it isn't ready for a connection yet
	// => no error panel, instead we just try connecting again
	if ([task isRunning]) {
		//[self connect:nil];
		NSAlert *alert = [NSAlert alertWithMessageText:HDLocalizedAlert(HDAspectControllerDaemonConnectionFailedKey)
			defaultButton:HDLocalizedOKButton
			alternateButton:nil
			otherButton:nil
			informativeTextWithFormat:HDLocalizedAlert(HDAspectControllerDaemonPleaseWaitKey)];

		[alert beginSheetModalForWindow:[NSApp mainWindow]
			modalDelegate:nil
			didEndSelector:NULL
			contextInfo:NULL];
	}
	else {
		NSAlert *alert = [NSAlert alertWithMessageText:HDLocalizedAlert(HDAspectControllerDaemonConnectionFailedKey)
			defaultButton:HDLocalizedOKButton
			alternateButton:nil
			otherButton:nil
			informativeTextWithFormat:HDLocalizedAlert(HDAspectControllerDaemonCheckRunningKey)];

		[alert beginSheetModalForWindow:[NSApp mainWindow]
			modalDelegate:nil
			didEndSelector:NULL
			contextInfo:NULL];
			
		// just to make sure the right thing is displayed
		// need to implement disabling of toolbar buttons while (dis)connecting/launching/stopping - sr
		[protocolArrayController removeAllArrangedObjects:nil];
		[protocolArrayController addObject:entryForNotConnected];
	}
}

// this is called by a notification when the connection closed
- (void)didDisconnect:(NSNotification *)aNotification
{
	[super didDisconnect:aNotification];

	// some setup for the next connection
	AM_synchronizing=NO;

	// don't send STATS anymore
	[timer invalidate];	// will also release
	//[timer release];
	timer = nil;

	BOOL shouldVisiblyDisconnect = YES;

	// first check if giFT crashed]
	if (!AM_connectOnce) {
		if ([userDefaults boolForKey:@"HDDaemonRelaunchAfterCrash"] && !AM_userDisconnected && !AM_remoteDaemon && ![task isRunning]) {
		
			// HDAspectControllerSearch is an observer for this notificatoin
			[[NSNotificationCenter defaultCenter] postNotificationName:@"PConnectionClosedNotification" object:self
				userInfo:[NSDictionary dictionaryWithObjectsAndKeys:@"dummy",@"crash",nil]];
			
			// if more then 6 crashes in a row then we have a problem
			
			NSLog(@"giftd crash #%i",crashes);
			
			if (crashes++>PDaemonControllerMaxCrashes) {
				[self giftCrashNotification];
				crashes=0;
			}
			[self launchDaemon:nil];	// giFT crashed....
			shouldVisiblyDisconnect = NO;
		}
		else if ([task isRunning] && AM_startingDaemon) {
			(void)[self instantiateConnectionToLocalDaemon];
			shouldVisiblyDisconnect = NO;
		}
	}

	if (shouldVisiblyDisconnect) {
		if (!AM_userDisconnected) {
			[[NSNotificationCenter defaultCenter] postNotificationName:@"DaemonConnectionTimedOut" object:[HDDaemonCommander sharedInstance]];
		}
		
		// HDAspectControllerSearch is an observer for this notificatoin
		[[NSNotificationCenter defaultCenter] postNotificationName:@"PConnectionClosedNotification" object:self
			userInfo:[NSDictionary dictionaryWithObjectsAndKeys:@"dummy",@"userDisconnected",nil]];

		AM_startingDaemon = NO;
		AM_connectOnce=NO;

		// update the gui
		[protocolArrayController removeAllArrangedObjects:nil];
		[protocolArrayController addObject:entryForNotConnected];
		
		id appDelegate = [[NSApplication sharedApplication] delegate];
		[[appDelegate localConnectMenuItem] setAction:@selector(connectToLocalDaemon:)];
		[[appDelegate remoteConnectMenuItem] setAction:@selector(connectToRemoteDaemon:)];
		[[appDelegate statusImageView] setImage:[NSImage imageNamed:@"offline.tiff"]];
		[[appDelegate statsTextField] setStringValue:NSLocalizedString(@"Not Connected", @"Not Connected")];
		[[appDelegate statusImageView] setNeedsDisplay:YES];
	}
}

/*- (void)UpShutdown
{
	AM_userDisconnected=YES;
	AM_startingDaemon=NO;
	[commander executeCommand:@"QUIT"];
	//if ([task isRunning]) [task terminate];
	[protocolArrayController removeAllArrangedObjects:nil];
	[protocolArrayController addObject:entryForStoppingGiFT];
}*/

- (IBAction)launchDaemon:(id)sender
{
	id theCommander = [HDDaemonCommander sharedInstance];
	if ([theCommander isConnected]) {
		AM_userDisconnected=YES;
		AM_startingDaemon=NO;
		[theCommander executeCommand:@"QUIT"];
		//if ([task isRunning]) [task terminate];
		[protocolArrayController removeAllArrangedObjects:nil];
		[protocolArrayController addObject:entryForStoppingGiFT];
	}
	else {
		AM_userDisconnected=NO;
		//AM_startingDaemon=NO;
		//if (![self instantiateConnectionToLocalDaemon]) {
		[self startDaemon];
	   // }
		[self instantiateConnectionToLocalDaemon];
	}
	[[NSApp delegate] validateToolbar];
}

- (void)startDaemon
{
	[protocolArrayController removeAllArrangedObjects:nil];
	[protocolArrayController addObject:entryForStartingGiFT];

	// if giFT is already running, just try to connect
	AM_startingDaemon = NO;
	if ([task isRunning]) {
		(void)[self instantiateConnectionToLocalDaemon];
	}
	else {
		AM_startingDaemon = YES;

		NSString *launchPath;
		
		[task autorelease];
		task=nil;
		task = [[NSTask alloc] init];
		
		if ([userDefaults boolForKey:@"HDDaemonUseCustom"]) {
			launchPath = [userDefaults stringForKey:@"HDDaemonGiFTPath"];
		}
		else
		{
			launchPath = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:@"giFT/bin/giftd"];
			NSMutableDictionary *envDict = [NSMutableDictionary dictionaryWithDictionary:[[NSProcessInfo processInfo] environment]];
			
			// NOTE: now that we are using the build script, which sets the install_name for libraries relative to the binary
			// we no longer need to use DYLD_LIBRARY_PATH - jjt
			// i set the current working directory and use a relative path for DYLD_LIBRARY_PATH because DYLD_LIBRARY_PATH
			// is a colon separated list of paths, but OS X allows colons in paths. - jjt
			
			//[task setCurrentDirectoryPath:[[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:@"giFT/"]];
			//[envDict setObject: @"lib/" forKey:@"DYLD_LIBRARY_PATH"];
			
			// data dir should be Resources/giFT/share
			[envDict setObject: [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:@"giFT/share"] forKey: @"GIFT_DATA_DIR"];

			NSString *localGiFTPath = HDApplicationSupportDirectory();

			[envDict setObject: [localGiFTPath stringByExpandingTildeInPath] forKey: @"GIFT_LOCAL_DIR"];
			[envDict setObject: [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:@"giFT/lib/giFT"] forKey: @"GIFT_PLUGIN_DIR"];
			
			[task setEnvironment:envDict];
		}
		[task setLaunchPath:launchPath];
		@try {
			[task launch];
			AM_remoteDaemon = NO;
		}
		@catch (NSException *e) {
			NSAlert *alert = [NSAlert alertWithMessageText:HDLocalizedAlert(HDDaemonControllerCouldNotStartMessageTextKey)
				defaultButton:HDLocalizedOKButton
				alternateButton:nil
				otherButton:nil
				informativeTextWithFormat:HDLocalizedAlert(HDDaemonControllerMakeSurePathExistsTextKey),[userDefaults stringForKey:@"HDDaemonGiFTPath"]];
			
			[alert setAlertStyle:NSCriticalAlertStyle];
			
			[alert beginSheetModalForWindow:[[NSApp delegate] mainWindow]
				modalDelegate:nil
				didEndSelector:NULL
				contextInfo:NULL];
		}
	}
}

- (void)giftCrashNotification
{
	NSAlert *alert = [NSAlert alertWithMessageText:HDLocalizedAlert(HDDaemonControllerCrashAlertMessageTextKey)
		defaultButton:HDLocalizedYesButton
		alternateButton:HDLocalizedNoButton
		otherButton:nil
		informativeTextWithFormat:HDLocalizedAlert(HDDaemonControllerCrashAlertInformativeTextKey)];
	
	[alert setAlertStyle:NSCriticalAlertStyle];	
	
	int button = [alert runModal];
	
	if (button==NSOKButton) {
		[self am_resetProgram];
	}
}

- (IBAction)connectToLocalDaemon:(id)sender
{
	id theCommander = [HDDaemonCommander sharedInstance];
	if ([theCommander isConnected]) {
		[protocolArrayController removeAllArrangedObjects:nil];
		[protocolArrayController addObject:entryForDisconnectingFromDaemon];
		
		AM_userDisconnected=YES;
		[theCommander closeConnection];
		
	} else {
		[protocolArrayController removeAllArrangedObjects:nil];
		[protocolArrayController addObject:entryForConnectingToDaemon];

		AM_userDisconnected=NO;
		AM_connectOnce=YES;
		if (![self instantiateConnectionToLocalDaemon]) {
			// there is no notification from HDDaemonCommander
			// invoking connectionFailed makes sure there will be an error panel
			[[NSNotificationCenter defaultCenter] postNotificationName:@"DaemonConnectionTimedOut" object:[HDDaemonCommander sharedInstance]];
		}
	}
}

- (BOOL)instantiateConnectionToLocalDaemon
{
	BOOL connectionInstantiated = NO;

	AM_remoteDaemon=NO;
	
	NSString *address;
	int port;
	HDPreferencesConfigurationEditorUI *ui_conf = [HDPreferencesConfigurationEditorUI singleton];
	if (!ui_conf)		{
		NSAlert *alert = [NSAlert alertWithMessageText:HDLocalizedAlert(HDAspectControllerDaemonConnectionFailedKey)
			defaultButton:HDLocalizedOKButton
			alternateButton:nil
			otherButton:nil
			informativeTextWithFormat:HDLocalizedAlert(HDAspectControllerDaemonConfigFilesMissingKey)];

		//[alert setAlertStyle:NSCriticalAlertStyle];	// [TODO] should this be critical?

		[alert runModal];

		// returning yes here, otherwise there would be two error panels
		connectionInstantiated = YES;
	}
	else {
		[ui_conf read];
		port = [[ui_conf optionValueForKey:@"port"] intValue];
		address = [ui_conf optionValueForKey:@"host"];

		connectionInstantiated = [[HDDaemonCommander sharedInstance] connect:address withPort:port];
	}
	
	return connectionInstantiated;
}

- (IBAction)connectToRemoteDaemon:(id)sender
{
	id sharedApp = [NSApplication sharedApplication];
	id appDelegate = [sharedApp delegate];
	[[appDelegate localConnectMenuItem] setAction:nil];
	[[appDelegate remoteConnectMenuItem] setAction:nil];

	NSString *address;
	if (address=[userDefaults stringForKey:@"PRemoteAddress"]) {
		[remoteAddressField setStringValue:address];
		[remotePortField setIntValue:[userDefaults integerForKey:@"PRemotePort"]];
	}
	[sharedApp beginSheet:connectionSheet
		modalForWindow:[sharedApp mainWindow]
		modalDelegate:self
		didEndSelector:@selector(remoteConnectionSheetDidEnd:returnCode:contextInfo:)
		contextInfo:NULL];
}

- (IBAction)closeRemoteConnectionSheet:(id)sender
{
	id sharedApp = [NSApplication sharedApplication];
	[sharedApp endSheet:[[sharedApp mainWindow] attachedSheet] returnCode:[sender tag]];
	[connectionSheet orderOut:self];
}

- (void)remoteConnectionSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo {
	id appDelegate = [[NSApplication sharedApplication] delegate];
	[[appDelegate localConnectMenuItem] setAction:@selector(connectToLocalDaemon:)];
	[[appDelegate remoteConnectMenuItem] setAction:@selector(connectToRemoteDaemon:)];

	if (returnCode == NSOKButton) {
		[self initiateConnectionToRemoteDaemon];
	}
}

- (void)initiateConnectionToRemoteDaemon {
	[protocolArrayController removeAllArrangedObjects:nil];
	[protocolArrayController addObject:entryForConnectingToDaemon];
	
	AM_remoteDaemon=YES;
	AM_userDisconnected=NO;
	//[self cancelSheet:nil];
	
	NSString *address = [remoteAddressField stringValue];
	int port = [remotePortField intValue];

	AM_connectOnce=YES;
	if (![[HDDaemonCommander sharedInstance] connect:address withPort:port]) {
		[[NSNotificationCenter defaultCenter] postNotificationName:@"DaemonConnectionTimedOut" object:[HDDaemonCommander sharedInstance]];
	}
	else {
		[userDefaults setObject:address forKey:@"PRemoteAddress"];
		if (port>0) [userDefaults setInteger:port forKey:@"PRemotePort"];
		else [userDefaults removeObjectForKey:@"PRemotePort"];
	}
}

- (void)askAboutReset {
	NSAlert *alert = [NSAlert alertWithMessageText:HDLocalizedAlert(PDaemonControllerWouldYouLikeToResetKey)
		defaultButton:HDLocalizedYesButton
		alternateButton:HDLocalizedCancelButton
		otherButton:nil
		informativeTextWithFormat:HDLocalizedAlert(PDaemonControllerThisWillTrashDataKey)];
	
	[alert setAlertStyle:NSCriticalAlertStyle];
	
	int returnCode = [alert runModal];
	
	if (returnCode == NSOKButton) {
		[self finishResettingProgram];
	}
	
	/*[alert beginSheetModalForWindow:[NSApp mainWindow]
			modalDelegate:self
			didEndSelector:@selector(resetProgramAlertSheetDid:returnCode:contextInfo:)
			contextInfo:NULL];*/
}

//- (void)detectedGiFTRunningAlertSheetDidEnd:(NSAlert *)alert returnCode:(int)returnCode contextInfo:(void *)contextInfo
- (void)tryToShutdownGiFTForReset {
	//int returnCode = [[NSApp delegate] tryToShutdownGiFT];
	(void)[[NSApp delegate] stopDaemon];
	
	if ([HDProcessLister isProcessRunning:HDDaemonName]) {
		NSAlert *alert = [NSAlert alertWithMessageText:HDLocalizedAlert(PDaemonControllerSorryString1Key)
			defaultButton:HDLocalizedOKButton
			alternateButton:nil
			otherButton:nil
			informativeTextWithFormat:HDLocalizedAlert(PDaemonControllerGiFTFailedShutdownKey)];
		[alert setAlertStyle:NSCriticalAlertStyle];
		
		(void)[alert runModal];
		
	} else {
		[self askAboutReset];
	}
}

//- (void)resetProgramAlertSheetDidEnd:(NSAlert *)alert returnCode:(int)returnCode contextInfo:(void *)contextInfo
- (void)finishResettingProgram {

NSLog(@"finishing reset...");
	//[[alert window] orderOut:self];
	HDPreferencesConfigurationEditorGiFT *gift_conf = [HDPreferencesConfigurationEditorGiFT singleton];
	[gift_conf read];

	NSString *programName = HDMainBundleName();

	if ([NSWorkspace deleteFileAtPath:[gift_conf optionValueForKey:@"incoming"]]) {
		NSLog(@"Moving directory \"%@\" to Trash...", [gift_conf optionValueForKey:@"incoming"]);
	}
	else {
		
		NSAlert *alert = [NSAlert alertWithMessageText:HDLocalizedAlert(PDaemonControllerSorryString2Key)
			defaultButton:HDLocalizedOKButton
			alternateButton:nil
			otherButton:nil
			informativeTextWithFormat:HDLocalizedAlert(PDaemonControllerIncomingPermissionsErrorStringKey), programName, programName];
		
		[alert setAlertStyle:NSCriticalAlertStyle];
		
		(void)[alert runModal];
	}
	
	NSString *applicationSupportDirectory = HDApplicationSupportDirectory();
		
	if ([NSWorkspace deleteFileAtPath:applicationSupportDirectory]) {
		NSLog(@"Moving directory \"%@\" to Trash...", applicationSupportDirectory);
	}
	else {
		NSAlert *alert = [NSAlert alertWithMessageText:HDLocalizedAlert(PDaemonControllerSorryString3Key)
			defaultButton:HDLocalizedOKButton
			alternateButton:nil
			otherButton:nil
			informativeTextWithFormat:HDLocalizedAlert(PDaemonControllerAppSuppPermissionsErrorStringKey), programName, programName, programName];

		[alert setAlertStyle:NSCriticalAlertStyle];

		(void)[alert runModal];
	}

	NSString *programPrefs = HDPreferencesPath();
	
	if ([NSWorkspace deleteFileAtPath:programPrefs]) {
		NSLog(@"Moving file \"%@\" to Trash...", programPrefs);
	}
	else {
		NSAlert *alert = [NSAlert alertWithMessageText:HDLocalizedAlert(PDaemonControllerSorryString4Key)
			defaultButton:HDLocalizedOKButton
			alternateButton:nil
			otherButton:nil
			informativeTextWithFormat:HDLocalizedAlert(PDaemonControllerPrefsPermissionsErrorStringKey), programName, programPrefs];
		
		[alert setAlertStyle:NSCriticalAlertStyle];

		(void)[alert runModal];
	}
	
	[NSApp terminate:self];
}

- (BOOL)validateToolbarItem:(NSToolbarItem *)theItem
{
	BOOL shouldBeEnabled = YES;

	NSString *identifier = [theItem itemIdentifier];
	BOOL connected = [[HDDaemonCommander sharedInstance] isConnected];
	
	if ([identifier isEqualToString:@"DaemonConnection"]) {
		//shouldBeEnabled = (!connected ? YES : NO);
		if (connected) {
			[theItem setImage:[NSImage imageNamed:@"Red Ball"]];
			[theItem setLabel:NSLocalizedString(@"Disconnect", @"Disconnect")];
		}
		else {
			[theItem setImage:[NSImage imageNamed:@"Green Ball"]];
			[theItem setLabel:NSLocalizedString(@"Connect", @"Connect")];
		}
	}
	else if ([identifier isEqualToString:@"DaemonProcess"]) {
		//shouldBeEnabled = ![self am_isDaemonRunningCache];
		if ([self am_isDaemonRunningCache]) {
			[theItem setImage:[NSImage imageNamed:@"Stop"]];
			[theItem setLabel:NSLocalizedString(@"Stop Daemon", @"Stop Daemon")];
		}
		else {
			[theItem setImage:[NSImage imageNamed:@"Power"]];
			[theItem setLabel:NSLocalizedString(@"Start Daemon", @"Start Daemon")];
		}
	}
	
	return shouldBeEnabled;
}

- (void)checkConfFiles
{
	BOOL setRandomOpenFTPorts=NO;
	BOOL setRandomGnutellaPort=NO;
	if (![self checkForItemAtPath:@""]) {	// -> ~/Library/Application Support/Poisoned
		setRandomOpenFTPorts=YES;
		setRandomGnutellaPort=YES;
	}
	if (![self checkForItemAtPath:@"/giftd.conf"]) {
		// giftd.conf wasn't there already, so it's possible that the user still has an old gift.conf
		// if so, we have to save the old prefs from gift.conf into giftd.conf
		HDPreferencesConfigurationEditorGiFT *gift_conf = [HDPreferencesConfigurationEditorGiFT singleton];
		[gift_conf restoreOldPrefs];
	}
	if (![self checkForItemAtPath:@"/OpenFT"]) setRandomOpenFTPorts=YES;
	if (![self checkForItemAtPath:@"/OpenFT/OpenFT.conf"]) setRandomOpenFTPorts=YES;
		
	if (![self checkForItemAtPath:@"/Gnutella"]) setRandomGnutellaPort=YES;
	if (![self checkForItemAtPath:@"/Gnutella/Gnutella.conf"]) setRandomGnutellaPort=YES;
	[self checkForItemAtPath:@"/FastTrack"];
	[self checkForItemAtPath:@"/FastTrack/FastTrack.conf"];
	[self checkForItemAtPath:@"/ui"];
	[self checkForItemAtPath:@"/ui/ui.conf"];

	if (setRandomOpenFTPorts) {
		// OpenFT.conf wasn't there already, this means "port" and "http_port" still have the default values
		// we replace them with random values
		// we should take these two values into the prefs so the user can change these
		HDPreferencesConfigurationEditorOpenFT *openft_conf = [HDPreferencesConfigurationEditorOpenFT singleton];
		[openft_conf setRandomValues];
	}
	if (setRandomGnutellaPort) {
		// same as for OpenFT.conf, here it's just port
		HDPreferencesConfigurationEditorGnutella *gnutella_conf = [HDPreferencesConfigurationEditorGnutella singleton];
		[gnutella_conf setRandomValues];
	}

}

// returns YES if the file/folder was already there, NO otherwise
- (BOOL)checkForItemAtPath:(NSString *)path
{
	BOOL itemExists = YES;

	NSFileManager *manager = 	[NSFileManager defaultManager];
	
	NSString *gift = HDApplicationSupportDirectory();

	NSString *shared = 		[[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:@"giFT/share"];

	if (![manager fileExistsAtPath:[gift stringByAppendingString:path]]) {
		NSLog(@"%@ not found: copying...",[gift stringByAppendingString:path]);
		[manager 
			copyPath:[shared stringByAppendingString:path] 
			toPath:[gift stringByAppendingString:path] 
			handler:nil];
		itemExists = NO;
		
		[[NSNotificationCenter defaultCenter] postNotificationName:@"PUpdateFromConfFiles" object:nil];	// [AM] my addition
	}
	
	return itemExists;
}


- (NSString *)attachString {
	@synchronized(self) {
		return [[AM_attachString retain] autorelease];
	}
	return nil;
}

- (void)setAttachString:(NSString *)aString {
	@synchronized(self) {
		if (aString != AM_attachString) {
			[AM_attachString release];
			AM_attachString = [aString copy];
		}
	}
}

/*- (void)gui_update:(BOOL)activeView
{

}*/

#pragma mark -
#pragma mark Table View Delegate Methods

// delegate method; we don't want the protocol list to be interactive
- (BOOL)tableView:(NSTableView *)aTableView shouldSelectRow:(int)rowIndex
{
	return NO;
}

// another delegate method
- (void)tableView:(NSTableView *)aTableView willDisplayCell:(id)aCell 
	forTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
	NSString *identifier = [aTableColumn identifier];
	if ([identifier isEqualToString:PZNetworkEntrySummaryKey]) {
		BOOL shouldHighlight = ([[aTableView selectedRowIndexes] containsIndex:rowIndex] && [[NSApp mainWindow] firstResponder] == aTableView);
		id network = [protocolArrayController objectAtArrangedObjectIndex:rowIndex];
		NSAttributedString *attributedString = [network summaryAttributedStringWithHighlight:shouldHighlight];
		[aCell setAttributedStringValue:attributedString];
	}
}

#pragma mark -
#pragma mark Data Methods

- (oneway void)ATTACH:(in NSArray *)data
{
	if (nil != data) {
		NSDictionary *server = [data objectAtIndex:2];
		NSString *serverString = [server objectForKey:@"server"];
		NSString *versionString = [server objectForKey:@"version"];

		NSString *newAttachString;
		if (serverString && versionString) {
			newAttachString = [[serverString stringByAppendingString:@" "] stringByAppendingString:versionString];
		}
		else {
			newAttachString = @"";
		}
		[self setAttachString:newAttachString];
	}
}

- (oneway void)STATS:(in NSArray *)data
{
	if (nil != data) {
		NSMutableArray *tmpprotos = [[NSMutableArray alloc] init];
		[protocolArrayController removeAllArrangedObjects:nil];
		NSArray *protos = [data objectAtIndex:3];
		NSNumberFormatter *formatter=[[NSNumberFormatter alloc] init];
		[formatter setFormatterBehavior:NSNumberFormatterBehavior10_4];
		[formatter setFormat:@"#,###"];
		double totalSize=0.0;
		long totalUsers=0;
		long totalFiles=0;
		//int i, count=[protos count];
		
		NSString *shares = @"";
		//for (i=0;i<count;i++)
			//tmp = [protos objectAtIndex:i];
		NSEnumerator *e = [protos objectEnumerator];
		NSArray *tmp;

		while (tmp = [e nextObject]) {
			if ([[tmp objectAtIndex:2] objectForKey:@"users"]) {
				[tmpprotos addObject:[tmp objectAtIndex:0]]; // for the notification...
				totalSize += [[[tmp objectAtIndex:2] objectForKey:@"size"] doubleValue];
				totalUsers += (long)[[[tmp objectAtIndex:2] objectForKey:@"users"] intValue];
				totalFiles += (long)[[[tmp objectAtIndex:2] objectForKey:@"files"] intValue];
				
				if ([[[tmp objectAtIndex:2] objectForKey:@"users"] intValue]>0 && 
					[[[tmp objectAtIndex:2] objectForKey:@"users"] intValue]<100)
				{					
					NSDictionary *dict = [[NSDictionary alloc] initWithObjectsAndKeys:
						[icon_shop largeIconNameForProtocol:[tmp objectAtIndex:0]], PZNetworkEntryIconKey,
						[tmp objectAtIndex:0], PZNetworkEntryProtocolNameKey,
						NSLocalizedString(@"Connected (impossible to estimate size)", @"Connected (impossible to estimate size)"), PZNetworkEntryNetworkPopulationKey,
						nil];
					
					HDNetworkEntry *entry = [[HDNetworkEntry alloc] initWithEntryDictionary:dict];
					[protocolArrayController addObject:entry];
					[entry release];
					
					[dict release];
					
				}
				else {
					if ([[[tmp objectAtIndex:2] objectForKey:@"users"] intValue]==0) {	// == nil?!?!
						NSDictionary *dict = [[NSDictionary alloc] initWithObjectsAndKeys:
							[icon_shop largeIconNameForProtocol:[tmp objectAtIndex:0]], PZNetworkEntryIconKey,
							[tmp objectAtIndex:0], PZNetworkEntryProtocolNameKey,	// protocol name
							NSLocalizedString(@"Not Connected", @"Not Connected"), PZNetworkEntryNetworkPopulationKey,
							nil];
						
						HDNetworkEntry *entry = [[HDNetworkEntry alloc] initWithEntryDictionary:dict];
						[protocolArrayController addObject:entry];
						[entry release];
						
						[dict release];
						
					}
					else {
						NSDictionary *dict = [[NSDictionary alloc] initWithObjectsAndKeys:
							[icon_shop largeIconNameForProtocol:[tmp objectAtIndex:0]],PZNetworkEntryIconKey,
							[tmp objectAtIndex:0], PZNetworkEntryProtocolNameKey,	// protocol name
							[NSString stringWithFormat:@"%@ Users - %@ Files - %@ GB - (estimated)",
								[formatter stringForObjectValue:
									[NSNumber numberWithLong:[[[tmp objectAtIndex:2] objectForKey:@"users"] intValue]]],
								[formatter stringForObjectValue:
									[NSNumber numberWithLong:[[[tmp objectAtIndex:2] objectForKey:@"files"] intValue]]],
								[formatter stringForObjectValue:
									[NSNumber numberWithDouble:[[[tmp objectAtIndex:2] objectForKey:@"size"] doubleValue]]]
							], PZNetworkEntryNetworkPopulationKey,
							nil];
						
						HDNetworkEntry *entry = [[HDNetworkEntry alloc] initWithEntryDictionary:dict];
						[protocolArrayController addObject:entry];
						[entry release];
						
						[dict release];
					}
				}

			}
			else {
				int files = [[[tmp objectAtIndex:2] objectForKey:@"files"] intValue];
				NSString *f = (files != 1) ? @"Files" : @"File";
				if (AM_synchronizing) shares = @"synchronizing... (this may take a while)";
				else shares = [[NSString stringWithFormat:@"%i %@ - ",files,f] stringByAppendingString:
								[NSString stringWithFormat:@"%@ GB",[[tmp objectAtIndex:2] objectForKey:@"size"]]];
								
				NSDictionary *dict = [[NSDictionary alloc] initWithObjectsAndKeys:
					@"public.icns", PZNetworkEntryIconKey,
					@"Sharing", PZNetworkEntryProtocolNameKey,	// protocol name
					shares, PZNetworkEntryNetworkPopulationKey,
					nil];

				HDNetworkEntry *entry = [[HDNetworkEntry alloc] initWithEntryDictionary:dict];
				[protocolArrayController insertObject:entry atArrangedObjectIndex:0];
				[entry release];
				
				[dict release];
			}
		}

		NSString *total = [NSString stringWithFormat:@"%@ Users - %@ Files - %@ GB - (estimated)",
			[formatter stringForObjectValue:[NSNumber numberWithDouble:totalUsers]],
			[formatter stringForObjectValue:[NSNumber numberWithLong:totalFiles]],
			[formatter stringForObjectValue:[NSNumber numberWithLong:totalSize]]];
		
		NSDictionary *dict;
		HDNetworkEntry *entry;
		
		dict = [NSDictionary dictionaryWithObjectsAndKeys:
			@"OpenFT32.icns", PZNetworkEntryIconKey,
			[self attachString], PZNetworkEntryProtocolNameKey,	// giFT & version
			total, PZNetworkEntryNetworkPopulationKey,	// total stats for all networks
			nil];
		entry = [[HDNetworkEntry alloc] initWithEntryDictionary:dict];
		[protocolArrayController insertObject:entry atArrangedObjectIndex:1];
		[entry release];
			
		dict = [NSDictionary dictionaryWithObjectsAndKeys:
			@"", PZNetworkEntryIconKey,
			@"", PZNetworkEntryProtocolNameKey,
			@"", PZNetworkEntryNetworkPopulationKey,
			nil];
		entry = [[HDNetworkEntry alloc] initWithEntryDictionary:dict];
		[protocolArrayController insertObject:entry atArrangedObjectIndex:1];
		[entry release];
		
		[[[[NSApplication sharedApplication] delegate] statsTextField] setStringValue:total];	// [TODO] this should be more MVC-oriented
		
		if (!AM_protosSend) {
			// send notification with available protocols, but only if we didn't do so yet
			// HDSearchFilterController needs this for the protol filter
			[[NSNotificationCenter defaultCenter] postNotificationName:@"PStatsProtocolsAvailable" object:self
				userInfo:[NSDictionary dictionaryWithObjectsAndKeys:tmpprotos,@"protos",nil]];
			AM_protosSend=YES;
		}
		[tmpprotos release];
		[formatter release];
	}
}

- (oneway void)SHARE:(in NSArray *)data
{
	if (data) {
		NSDictionary *item = [data objectAtIndex:2];
		if (item) {
			NSString *action = [item objectForKey:@"action"];
			NSString *status = [item objectForKey:@"status"];
			if (action && status && [action isEqualToString:@"sync"]) {
				if ([status isEqualToString:@"Done"]) {
					AM_synchronizing=NO;
					/* giFT finished synchronizing the shared files
					 * so we need new STATS */
					[[HDDaemonCommander sharedInstance] executeCommand:@"STATS"];
				}
				else {
					AM_synchronizing=YES;
				}
			}
		}
	}
}

@end

#pragma mark -

@implementation HDAspectControllerDaemon (Internal)

- (void)am_resetProgram
{
	// giftd shutdown detection will be added here as well.
	if ([HDProcessLister isProcessRunning:HDDaemonName])
	{
		NSAlert *alert = [NSAlert alertWithMessageText:HDLocalizedAlert(PDaemonControllerDetectedGiFTRunningMTKey)
			defaultButton:HDLocalizedYesButton
			alternateButton:HDLocalizedNoButton
			otherButton:nil
			informativeTextWithFormat:HDLocalizedAlert(PDaemonControllerResetGiFTMustBeShutDownKey)];
		
		[alert setAlertStyle:NSCriticalAlertStyle];
		
		int returnCode = [alert runModal];
		if (returnCode == NSOKButton) {
			[self tryToShutdownGiFTForReset];
		}
		
		/*[alert beginSheetModalForWindow:[NSApp mainWindow]
			modalDelegate:self
			didEndSelector:@selector(detectedGiFTRunningAlertSheetDidEnd:)
			contextInfo:NULL];*/
	} else {
		[self askAboutReset];
	}
}

- (BOOL)am_isDaemonRunningCache {
	return AM_daemonIsRunning;
}

- (void)am_setDaemonRunningCache:(BOOL)flag {
	AM_daemonIsRunning = flag;
}

- (void)recacheDaemonRunning:(NSTimer *)aTimer {
	[self am_setDaemonRunningCache:[HDProcessLister isProcessRunning:HDDaemonName]];
	[[NSApp delegate] validateToolbar];
}

@end
