//
// HDAspectControllerDownloads.m
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import "HDAspectControllerDownloads.h"
#import "HDDaemonCommander.h"
#import "HDTransferSource.h"
#import "HDPreferencesConfigurationEditorGiFT.h"
#import "PDownloadTitleDescriptionCell.h"
#import "PSizeCell.h"
#import "PDownloadProgressCell.h"
#import "HDMediaTypeDetector.h"
#import "PSearchResultsOutlineView.h"

#import "HDTransferSourceDownloads.h"

#import "HDAlertLocalizedStringKeys.h"

#import "HDDownloadEntry.h"
#import "HDMainToolbarCategory.h"


@implementation HDAspectControllerDownloads

- (id)init {
	if (nil != (self = [super init])) {
		if (![NSBundle loadNibNamed:@"AspectControllerDownloads" owner:self]) {
            [self release];
            self = nil;
        }
	}
	return self;
}

- (void)dealloc
{
    [table release];			// [AM] added on 2007-08-19
    [contextualMenu release];	// [AM] added on 2007-08-19
	
	table = nil;			// [AM] added on 2007-08-19
	contextualMenu = nil;	// [AM] added on 2007-08-19

	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[hashes release];
	[dataSource release];
	[super dealloc];
}

- (void)awakeFromNib
{
	//NSArray *cols;
	//int i, count;
	
	hashes = [[NSMutableSet alloc] init];

	dataSource = [[HDTransferSourceDownloads alloc] initWithTable:table andTreeController:downloadsTreeController];
	[table setMenuDelegate:self];
	[table setDataSource:dataSource];
	[table setDelegate:dataSource];
	//[table setDrawsGrid:YES];
	//[table setAllowsColumnSelection:NO];
	//[table setTarget:self];
	//[table setDelegate:self];
	
	//cols = [table tableColumns];
	//count=[cols count];
	/*for (i=0;i<count;i++) [[[cols objectAtIndex:i] dataCell] setDrawsBackground:NO];*/
	[table setOutlineTableColumn:[[table tableColumns] objectAtIndex:1]];
	[table setAutoresizesOutlineColumn:NO];
	
	//[table setSmallRowHeight:16];
	//[table setRowHeight:34];
	[[table tableColumnWithIdentifier:@"PFileUser"] setDataCell:
		[[[PDownloadTitleDescriptionCell alloc] init] autorelease]
	];
	[[table tableColumnWithIdentifier:@"PTransfer"] setDataCell:
		[[[PSizeCell alloc] init] autorelease]
	];
	[[table tableColumnWithIdentifier:@"PSize"] setDataCell:
		[[[PSizeCell alloc] init] autorelease]
	];
	[[table tableColumnWithIdentifier:@"PProgress"] setDataCell:
		[[[PDownloadProgressCell alloc] init] autorelease]
	];
	[[table tableColumnWithIdentifier:@"PIcon"] setDataCell:
		[[[NSImageCell alloc] init] autorelease]
	];
}

- (void)registerWithCommander {
	[super registerWithCommander];

	[[HDDaemonCommander sharedInstance] registerController:self forCommands:
		[NSArray arrayWithObjects:@"ADDDOWNLOAD",@"CHGDOWNLOAD",@"DELDOWNLOAD",nil]
	];

	// datasource needs the commander for cancelling downloads etc...
	//[dataSource setCommander:aCommander];
	// datasource needs the downloading hashses set
	// when the user cancels a download with the del key
	[dataSource setHashes:hashes];
}

- (void)didDisconnect:(NSNotification *)aNotification
{
	[super didDisconnect:aNotification];

	[hashes removeAllObjects];
	[dataSource disconnected];
}

/*- (void)gui_update:(BOOL)activeView
{
	//if (activeView) [table reloadData];
}*/

- (IBAction)cancelDownload:(id)sender
{
	if ([[NSUserDefaults standardUserDefaults] boolForKey:@"HDDownloadsConfirmCancellation"]) {
		NSAlert *alert = [NSAlert alertWithMessageText:HDLocalizedAlert(PDownloadControllerCancelSelectedDownloadsMessageTextKey)
			defaultButton:HDLocalizedYesButton
			alternateButton:HDLocalizedNoButton
			otherButton:nil
			informativeTextWithFormat:HDLocalizedAlert(PDownloadControllerCancelSelectedDownloadsInformativeTextKey)];

		[alert beginSheetModalForWindow:[[NSApplication sharedApplication] mainWindow]
			modalDelegate:self
			didEndSelector:@selector(cancelSheetDidEnd:returnCode:contextInfo:)
			contextInfo:NULL];
	} else {
		[self doDelete];
	}
}

- (void)cancelSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{    
	if(returnCode == NSAlertDefaultReturn) {
		[self doDelete];
	}
}

- (void)doDelete
{
	NSEnumerator *enumerator = [table selectedRowEnumerator];
	NSNumber *num;
	NSMutableDictionary *item;
	while (num=[enumerator nextObject]) {
		item = [table itemAtRow:[num intValue]];
		if ([item valueForKey:HDTransferSourceDownloadsHashKey]) [hashes removeObject:[item valueForKey:HDTransferSourceDownloadsHashKey]];
		[dataSource cancel:[HDDaemonCommander sharedInstance]];
	}
}

- (IBAction)pauseDownload:(id)sender
{
	[dataSource pause:[HDDaemonCommander sharedInstance]];
}

- (IBAction)resumeDownload:(id)sender
{
	[dataSource resume:[HDDaemonCommander sharedInstance]];
}

- (IBAction)delsource:(id)sender
{
	[dataSource delsource:[HDDaemonCommander sharedInstance]];
}

- (NSString *)hostForBrowsing;
{
	return [[[table itemAtRow:[table selectedRow]] valueForKey:@"PFileUser"] objectAtIndex:1];
}

- (IBAction)browseDownloadHost:(id)sender
{
	[[NSNotificationCenter defaultCenter] postNotificationName:@"PDownBrowseHost" object:self userInfo:nil];
}

- (IBAction)cleanUpDownloads:(id)sender
{
	[dataSource cleanUp:[HDDaemonCommander sharedInstance]];
}

- (IBAction)reveal:(id)sender
{
	// first we have to look for a parent item to get the filename and status from
	int row = [table selectedRow];
	while (row!=-1 && ![[[table itemAtRow:row] valueForKey:@"PExpandable"] boolValue]) row--;
	int status = [[[table itemAtRow:row] valueForKey:@"PStatus"] intValue];
		NSLog(@"%i",status);
	if (status == HDTransferStatusCompleted)
	{
		HDPreferencesConfigurationEditorGiFT *gift_conf = [HDPreferencesConfigurationEditorGiFT singleton];
		[gift_conf read];
		NSString *path = [gift_conf optionValueForKey:@"completed"];
		if (path)
		{
			NSLog(@"reveal...");
			NSString *fileName = [[[table itemAtRow:row] valueForKey:@"PFileUser"] objectAtIndex:1];
			// selectFile:... only accepts full path's
			path = [[path stringByAppendingPathComponent:fileName] stringByExpandingTildeInPath];
						//if ([[NSFileManager defaultManager] fileExistsAtPath: path]) {
			NSLog(path);
			[[NSWorkspace sharedWorkspace] selectFile: path inFileViewerRootedAtPath: @""];
		}
	}
	else
	{
		HDPreferencesConfigurationEditorGiFT *gift_conf = [HDPreferencesConfigurationEditorGiFT singleton];
		[gift_conf read];
		NSString *path = [gift_conf optionValueForKey:@"incoming"];
		if (path)
		{
					NSLog(@"reveal...");
			//NSString *fileName = [[[table itemAtRow:row] objectForKey:@"PFileUser"] objectAtIndex:1];
			// selectFile:... only accepts full path's
			path = [[path stringByAppendingPathComponent:path] stringByExpandingTildeInPath];
						//if ([[NSFileManager defaultManager] fileExistsAtPath: path]) {
						//NSLog(path);
						//[[NSWorkspace sharedWorkspace] selectFile: path inFileViewerRootedAtPath: path];
			[[NSWorkspace sharedWorkspace] openTempFile:path];
		}
	}
}

- (IBAction)expandAll:(id)sender
{
	[dataSource expandAll];
}

- (IBAction)collapseAll:(id)sender
{
	[dataSource collapseAll];
}

- (int)downloadSpeed
{
	return [dataSource transferSpeed];
}

- (NSSet *)hashes
{
	return hashes;
}

- (BOOL)validateToolbarItem:(NSToolbarItem *)theItem
{
	BOOL isValidated = YES;

	NSString *ident = [theItem itemIdentifier];
	if ([ident isEqualToString:@"downCancel"]) {
		isValidated = [dataSource validateCancel];
	}
	else if ([ident isEqualToString:@"downPause"]) {
		isValidated = [dataSource validatePause];
	}
	else if ([ident isEqualToString:@"downResume"]) {
		isValidated = [dataSource validateResume];
	}
	else if ([ident isEqualToString:@"downDelSource"]) {
		isValidated = [dataSource validateDelSource];
	}
	else if ([ident isEqualToString:@"browseDownloadHost"]) {
		if ([table numberOfSelectedRows]==1 && ![[[table itemAtRow:[table selectedRow]] valueForKey:@"PExpandable"] boolValue]) {
			isValidated = YES;
		}
		else {
			isValidated = NO;
		}
	}
	else if([ident isEqualToString:@"downPreview"]) {
		isValidated = [dataSource validatePreview];
	}
	
	return isValidated;
}

- (NSMenu *)tableView:(NSOutlineView *)_table menuForTableColumn:(NSTableColumn *)_column row:(int)_row
{
	[[[NSApplication sharedApplication] mainWindow] makeFirstResponder:_table];
	if (![_table isRowSelected:_row]) [_table selectRow:_row byExtendingSelection:NO];
	return contextualMenu;
}

- (BOOL)validateMenuItem:(NSMenuItem *)menuItem
{
	BOOL isValidated = YES;
	SEL action = [menuItem action];

	if (action == @selector(findMoreSources:)) {
		isValidated = [dataSource validateFindMoreSources];
	}
	else if (action == @selector(cancelDownload:)) {
		isValidated = [dataSource validateCancel];
	}
	else if (action == @selector(pauseDownload:)) {
		isValidated = [dataSource validatePause];
	}
	else if (action == @selector(resumeDownload:)) {
		isValidated = [dataSource validateResume];
	}
	else if (action == @selector(delsource:)) {
		isValidated = [dataSource validateDelSource];
	}
	else if (action == @selector(openFile:)){
		if ([table numberOfSelectedRows] > 0)
		{
			int row = [table selectedRow];
			while (row!=-1 && ![[[table itemAtRow:row] valueForKey:@"PExpandable"] boolValue]) row--;
			int status = [[[table itemAtRow:row] valueForKey:@"PStatus"] intValue];
			if (status == HDTransferStatusCompleted)
				isValidated = YES;
		}
		else {
			isValidated = NO;
		}
	}
	else if (action == @selector(previewFile:)){
		if ([table numberOfSelectedRows] > 0)
		{
			int row = [table selectedRow];
			while (row!=-1 && ![[[table itemAtRow:row] valueForKey:@"PExpandable"] boolValue]) row--;
			int status = [[[table itemAtRow:row] valueForKey:@"PStatus"] intValue];
			if (status == HDTransferStatusCompleted)
				isValidated = NO;
		}
		else {
			return YES;
		}
	}
	else if (action == @selector(browseDownloadHost:)) {
		if ([table numberOfSelectedRows]==1 && ![[[table itemAtRow:[table selectedRow]] valueForKey:@"PExpandable"] boolValue]) {
			isValidated = YES;
		}
		else {
			isValidated = NO;
		}
	}
	else if (action == @selector(revealFile:))
	{
		// first we have to look for a parent item to get the status from
		if ([table numberOfSelectedRows] > 0)
		{
			int row = [table selectedRow];
			while (row!=-1 && ![[[table itemAtRow:row] valueForKey:@"PExpandable"] boolValue]) row--;
			int status = [[[table itemAtRow:row] valueForKey:@"PStatus"] intValue];
			if (status == HDTransferStatusCompleted)
				isValidated = YES;
		}
		else {
			isValidated = NO;
		}
	}
	
	return isValidated;
}

- (IBAction)findMoreSources:(id)sender;
{
	NSEnumerator *enumerator = [table selectedRowEnumerator];
	NSNumber *num;
	NSMutableDictionary *item;
	NSString *hash;
	while (num=[enumerator nextObject]) {
		item = [table itemAtRow:[num intValue]];
		if (hash=[item valueForKey:HDTransferSourceDownloadsHashKey]) {
			[[NSNotificationCenter defaultCenter] postNotificationName:@"PFindMoreSources" object:self
			userInfo:[NSDictionary dictionaryWithObjectsAndKeys:hash,HDTransferSourceDownloadsHashKey,nil]];
		}
	}
}

- (void)openFile:(id)sender
{
	int row = [table selectedRow];
	while (row!=-1 && ![[[table itemAtRow:row] valueForKey:@"PExpandable"] boolValue]) {
		row--;
	}

	int status = [[[table itemAtRow:row] valueForKey:@"PStatus"] intValue];
	if (status == HDTransferStatusCompleted)
	{
		HDPreferencesConfigurationEditorGiFT *gift_conf = [HDPreferencesConfigurationEditorGiFT singleton];
		[gift_conf read];
		NSString *fileName = [[[table itemAtRow:row] valueForKey:@"PFileUser"] objectAtIndex:1];
		NSString *path = [gift_conf optionValueForKey:@"completed"];
		if(path)
		{
			path = [[path stringByAppendingPathComponent:fileName] stringByExpandingTildeInPath];
			[[NSWorkspace sharedWorkspace] openFile:path];
		}
	}
}

- (void)previewFile:(id)sender
{
	int row = [table selectedRow];
	while (row!=-1 && ![[[table itemAtRow:row] valueForKey:@"PExpandable"] boolValue]) row--;

		HDPreferencesConfigurationEditorGiFT *gift_conf = [HDPreferencesConfigurationEditorGiFT singleton];
		[gift_conf read];
		NSString *fileName = [[[table itemAtRow:row] valueForKey:@"PFileUser"] objectAtIndex:1];
		NSString *path = [gift_conf optionValueForKey:@"incoming"];
		if (path) {
			[self previewFileWithName:fileName];
		}
}

/*- (void)tableView:(NSTableView *)aTableView willDisplayCell:(id)aCell 
	forTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
	NSString *identifier = [aTableColumn identifier];

	if ([identifier isEqualToString:@"PFileUser"]) {		
		BOOL shouldHighlight = [[aTableView selectedRowIndexes] containsIndex:rowIndex];
		NSAttributedString *attributedString = [aCell generatedAttributedStringWithHighlight:shouldHighlight];
		[aCell setAttributedStringValue:attributedString];
	}
}*/

#pragma mark -
#pragma mark Download Commands

- (oneway void)ADDDOWNLOAD:(in NSArray *)data 
{
	if (data) {
		NSString *hash = [[data objectAtIndex:2] objectForKey:HDTransferSourceDownloadsHashKey];
		[[HDDaemonCommander sharedInstance] removeTicket:[data objectAtIndex:1]];
		[dataSource ADDDOWNLOAD:data];

	   if (hash) { 
			[hashes addObject:[[hash copy] autorelease]];
			[[NSNotificationCenter defaultCenter] postNotificationName:@"PTransferSetChanged" object:self];
		}
		
		[[NSApp delegate] validateToolbar];
	}
}

- (oneway void)CHGDOWNLOAD:(in NSArray *)data
{
	if (data) {
		[dataSource CHGDOWNLOAD:data];
		[[NSApp delegate] validateToolbar];
	}
}

- (oneway void)DELDOWNLOAD:(in NSArray *)data
{
	if (data) {
		NSString *hash = [dataSource hashForTicket:[data objectAtIndex:1]];
		[[HDDaemonCommander sharedInstance] freeTicket:[data objectAtIndex:1]];
		[dataSource DELDOWNLOAD:data];
		
		if (hash) {
			[hashes removeObject:hash];
			[[NSNotificationCenter defaultCenter] postNotificationName:@"PTransferSetChanged" object:self];
		}
		[[NSApp delegate] validateToolbar];
	}
}

- (void)ADDSOURCE:(NSArray *)data
{
	if (data) {
		[dataSource ADDSOURCE:data];
	}
}

- (void)DELSOURCE:(NSArray *)data
{
	if (data) {
		[dataSource DELSOURCE:data];
	}
}


@end

#pragma mark -

@implementation HDAspectControllerDownloads (Internal)

- (void)previewFileWithName:(NSString *)filename
{
	HDPreferencesConfigurationEditorGiFT *gift_conf = [HDPreferencesConfigurationEditorGiFT singleton];
	[gift_conf read];
	NSString *path = [gift_conf optionValueForKey:@"incoming"];
	 
	NSString* file;
	NSString* tmpFile = @"";
	NSDirectoryEnumerator* dirEnum;
	dirEnum = [[NSFileManager defaultManager] enumeratorAtPath:[path stringByExpandingTildeInPath]];
	while(file = [dirEnum nextObject])
	{
		if([file length] >= 21)
		tmpFile = [file substringFromIndex: 21];
		if([tmpFile isEqualToString:filename])
		{
			path = [[path stringByAppendingPathComponent:file] stringByExpandingTildeInPath];
			if ([HDMediaTypeDetector mediaTypeIsMusic:[[file pathExtension] lowercaseString]])
				[[NSWorkspace sharedWorkspace] openFile:path withApplication:@"QuickTime Player.app"];
			else
				[[NSWorkspace sharedWorkspace] openFile:path];
			break;
		}
		
	}
}

@end
