//
// HDAspectControllerSearch.h
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import <Cocoa/Cocoa.h>
#import "HDAspectController.h"

#import "HDSearchCommands.h"


@class AMTableView;
@class PSearchResultsOutlineView;
@class HDSearchResultSource;
@class HDSearchFilterController;
@class AMSortingArrayController;


@interface HDAspectControllerSearch : HDAspectController <HDSearchCommands>
{
	IBOutlet AMSortingArrayController *searchListArrayController;
	IBOutlet NSTreeController *searchResultsTreeController;

	IBOutlet NSView *drawerView;

    IBOutlet NSSearchField *searchField;	// PTextField
    IBOutlet NSPopUpButton *searchRealmPopUpButton;
        
    IBOutlet PSearchResultsOutlineView *resultsOutlineView;
    IBOutlet AMTableView *searchListTableView;
    
    IBOutlet NSMenu *contextualMenu;
    IBOutlet NSMenu *tableMenu;

    
    //table colums
    IBOutlet NSTableColumn *tc_fileicon;
    NSTableColumn *tc_file;
    IBOutlet NSTableColumn *tc_networkicon;
    IBOutlet NSTableColumn *tc_user;
    IBOutlet NSTableColumn *tc_size;
    IBOutlet NSTableColumn *tc_bitrate;
    IBOutlet NSTableColumn *tc_artist;
    IBOutlet NSTableColumn *tc_album;
    IBOutlet NSTableColumn *tc_availability;
    IBOutlet NSTableColumn *tc_length;
                
    NSMutableDictionary *datasources;
    NSMutableArray *searches;
    HDSearchResultSource *current_src;
    
    NSUserDefaults *userDefaults;
    
    //NSImage *stopSearchImage;
    //NSImage *resumeSearchImage;
    
    IBOutlet HDSearchFilterController *filter;
    NSSet *AM_downloadingHashes;
    
    //NSMutableArray *recentSearches;

    NSMutableArray *AM_newItems;
    NSTimer *AM_refreshTimer;
}

- (void)saveSearches;
- (void)didConnect:(NSNotification *)aNotification;
- (void)didDisconnect:(NSNotification *)aNotification;

- (void)reloadResultsOutlineViewData:(NSNotification *)aNotification;

- (IBAction)toggleTableColumnVisibility:(id)sender;
- (NSMenu *)tableView:(NSOutlineView *)_table menuForTableColumn:(NSTableColumn *)_column row:(int)_row;

- (void)loadResultTable:(id)sender;
- (void)filterDidDeactivate:(NSNotification *)aNotification;
- (void)reloadSearchTable;

- (void)refreshTimerFired:(NSTimer *)timer;

- (void)deleteEvent:(id)sender;

- (void)search:(NSString *)_query info:(NSString *)_info ticket:(NSString *)_ticket hidden:(BOOL)_hidden;
- (IBAction)search:(id)sender;
//- (void)searchRecent:(id)sender;
- (IBAction)browse:(id)sender;
- (void)browseHost:(NSString *)user;
- (void)findMoreSources:(NSNotification *)notification;

- (void)toggleSelectedSearchStatus:(id)sender;
- (void)toggleSearches:(id)sender;
- (IBAction)download:(id)sender;

- (void)gui_update:(BOOL)activeView;

//- (void)clearRecentSearches;
//- (void)addRecentSearch:(NSString *)search withRealm:(NSString *)realm;

// FOR TESTING
- (void)buyAtiTMS:(id)sender;
- (BOOL)validateBuyAtiTMS;

@end

// [AM] added 2007-08-19; cover methods for inter-nib access
@interface HDAspectControllerSearch (UICoverMethods)
- (NSSearchField *)toolbarSearchField;
- (NSPopUpButton *)toolbarSearchRealmPopUpButton;
- (NSView *)drawerView;

@end

@interface HDAspectControllerSearch (Accessors)

- (NSTimer *)refreshTimer;
- (void)setRefreshTimer:(NSTimer *)aTimer;

- (NSMutableArray *)newItems;
- (void)setNewItems:(NSArray *)anArray;

- (NSSet *)downloadingHashes;
- (void)setDownloadingHashes:(NSSet *)aSet;

@end