//
// HDAspectControllerSearch.m
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import "HDAspectControllerSearch.h"
#import "HDDaemonCommander.h"

#import "HDMainController.h"
#import "HDSearchResultsCornerView.h"
#import "HDAdditions.h"
#import "HDValueTransformers.h"
#import "HDMediaTypeDetector.h"

#import "PSearchResultsOutlineView.h"
#import "HDSearchListEntry.h"

#import "HDSearchResultSource.h"
#import "HDSearchFilterController.h"

#import "AMSortingArrayController.h"

#import "AMTableView.h"


@implementation HDAspectControllerSearch

- (id)init {
	if (nil != (self = [super init])) {
		[self setNewItems:[NSArray array]];
		[self setRefreshTimer:nil];	// table refresh timer
		[self setDownloadingHashes:nil];
		
		//stopSearchImage = [[NSImage imageNamed:@"stop_search.tiff"] retain];
		//resumeSearchImage = [[NSImage imageNamed:@"re_search.tiff"] retain];
		
		if (![NSBundle loadNibNamed:@"AspectControllerSearch" owner:self]) {
			[self release];
			self = nil;
		}
	}
	return self;
}

- (void)dealloc
{
	if ([[self refreshTimer] isValid]) {
		[[self refreshTimer] invalidate];
	}
	[self setRefreshTimer:nil];
	[self setNewItems:nil];

	// [AM] begin new stuff
	[self setDownloadingHashes:nil];
	
	[searchListArrayController release];
	searchListArrayController = nil;
	
	[searchField release];
	[searchRealmPopUpButton release];
	
	[resultsOutlineView release];
	[searchListTableView release];
	
	[contextualMenu release];
	[tableMenu release];
	
	//table colums
	[tc_fileicon release];
	[tc_file release];
	[tc_networkicon release];
	[tc_user release];
	[tc_size release];
	[tc_bitrate release];
	[tc_artist release];
	[tc_album release];
	[tc_availability release];
	[tc_length release];
	
	[filter release];
	
	searchField = nil;
	searchRealmPopUpButton = nil;
	
	resultsOutlineView = nil;
	searchListTableView = nil;
	
	contextualMenu = nil;
	tableMenu = nil;
	
	//table colums
	tc_fileicon = nil;
	tc_file = nil;
	tc_networkicon = nil;
	tc_user = nil;
	tc_size = nil;
	tc_bitrate = nil;
	tc_artist = nil;
	tc_album = nil;
	tc_availability = nil;
	tc_length = nil;
	
	filter = nil;

	// [AM] end new stuff

	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[[NSNotificationCenter defaultCenter] removeObserver:resultsOutlineView];


	[tc_fileicon release];
	[tc_file release];
	[tc_networkicon release];
	[tc_user release];
	[tc_size release];
	[tc_bitrate release];
	[tc_artist release];
	[tc_album release];
	[tc_availability release];
	[tc_length release];
	
	
	//[stopSearchImage release];
	//[resumeSearchImage release];
	
	//stopSearchImage = nil;
	//resumeSearchImage = nil;
	
	
	//[recentSearches release];
	
	[searches release];
	//[datasources release];
	[super dealloc];
}

- (void)awakeFromNib
{
	[searchField retain];	// [AM] needed to prevent mysterious crash
	[searchRealmPopUpButton retain];	// [AM] needed to prevent mysterious crash
	
	[searchListTableView setTarget:self];
	[searchListTableView setDoubleAction:@selector(toggleSelectedSearchStatus:)];

	[searchListTableView setDeleteKeyRemoves:YES];
	[searchListTableView setSpaceBarSendsDoubleAction:YES];
	[searchListTableView setReturnKeySendsDoubleAction:YES];
	
	[searchListArrayController setAllowsRowCopy:NO];

	userDefaults = [NSUserDefaults standardUserDefaults];
	
	NSArray *recentSearches = [userDefaults objectForKey:@"PRecentSearches"];
	[searchField setRecentSearches:recentSearches];
		
	// setting up the search text field -------------------------------
	//[searchField setImage:[NSImage imageNamed:@"searchField.tiff"]];
	[searchField setTarget:self];
	[searchField setDelegate:self];
	[searchField setAction:@selector(search:)];
	// ----------------------------------------------------------------
	
	// sources --------------------------------------------------------
	//searches = [[NSMutableArray alloc] init];
	//datasources = [[NSMutableDictionary alloc] init];
	current_src = nil;
	// ----------------------------------------------------------------
		
	[filter setSearchDataSource:nil];
	
	// setting up the tables ------------------------------------------
	//[searchListTableView setRowHeight:30];
	//[resultsOutlineView setAutosaveName:@"PSearchTableAutoSave"];
	//[resultsOutlineView setAutosaveTableColumns:YES];
	[resultsOutlineView setMenuDelegate:self];
	//[resultsOutlineView setRowHeight:16];
	//[resultsOutlineView setDrawsGrid:YES];
	[resultsOutlineView setTarget:self];
	[resultsOutlineView setDelegate:self];
	[resultsOutlineView setDoubleAction:@selector(download:)];
	//[searchListTableView setDataSource:self];
	//[searchListTableView setDelegate:self];
	//[searchListTableView setTarget:self];
	//[searchListTableView setAction:@selector(loadResultTable:)];
	
	/*NSArray *cols = [resultsOutlineView tableColumns];
	int i, count=[cols count];
	for (i=0;i<count;i++) [[[cols objectAtIndex:i] dataCell] setDrawsBackground:NO];*/
	[resultsOutlineView setOutlineTableColumn:[resultsOutlineView tableColumnWithIdentifier:@"file"]];
	[resultsOutlineView setAutoresizesOutlineColumn:NO];
	
	[[resultsOutlineView tableColumnWithIdentifier:@"icon"] setDataCell:[[[NSImageCell alloc] init] autorelease]];
	[[resultsOutlineView tableColumnWithIdentifier:@"PProtoIcon"] setDataCell:[[[NSImageCell alloc] init] autorelease]];
	[[resultsOutlineView tableColumnWithIdentifier:@"PAvailability"] setDataCell:[[[NSImageCell alloc] init] autorelease]];
	
	/*NSButtonCell *button = [[NSButtonCell alloc] init];
	[button setButtonType:NSMomentaryChangeButton];
	[button setBordered:NO];
	[button setTarget:self];
	[button setAction:@selector(stop:)];
	[[searchListTableView tableColumnWithIdentifier:@"button"] setDataCell:button];
	[button release];*/

	
	/*id searchCell = [[PSearchListCell alloc] init];
	[[searchListTableView tableColumnWithIdentifier:PZSearchListEntryInfoStringKey] setDataCell:searchCell];
	[searchCell release];*/

	// Install Column Config Table Menu
	//[resultsOutlineView setTableMenu:tableMenu];
	{
		NSPopUpButton *cornerView = [[NSPopUpButton alloc] initWithFrame:NSZeroRect];
		NSPopUpButtonCell *cornerCell = [[HDSearchResultsCornerView alloc] initTextCell:@"" pullsDown:YES];
		[cornerCell setPullsDown:YES];
		[cornerCell setBordered:NO];
		[cornerCell setMenu:tableMenu];

		[cornerView setCell:cornerCell];
		[resultsOutlineView setCornerView:cornerView];
		[cornerCell release];
		[cornerView release];
	}
	 
	// ----------------------------------------------------------------
	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(reloadResultsOutlineViewData:) name:@"PTransferSetChanged" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(filterDidDeactivate:) name:@"PFilterDeactivated" object:filter];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(findMoreSources:) name:@"PFindMoreSources" object:nil];

	[tc_fileicon retain];
	[tc_file retain];
	[tc_networkicon retain];
	[tc_user retain];
	[tc_size retain];
	[tc_bitrate retain];
	[tc_artist retain];
	[tc_album retain];
	[tc_availability retain];
	[tc_length retain];
}

- (void)reloadResultsOutlineViewData:(NSNotification *)aNotification {
	[resultsOutlineView reloadData];
}

// contextual menu ----------------------------------------------------------
- (IBAction)toggleTableColumnVisibility:(id)sender
{
	NSString *title = [sender title];
	NSTableColumn *col = nil;
	if ([title isEqualToString:@"File Icon"])		col = tc_fileicon;
	else if ([title isEqualToString:@"File"])		col = tc_file;
	else if ([title isEqualToString:@"Network Icon"])	col = tc_networkicon;
	else if ([title isEqualToString:@"User"])		col = tc_user;
	else if ([title isEqualToString:@"Size"])		col = tc_size;
	else if ([title isEqualToString:@"Bitrate"])	col = tc_bitrate;
	else if ([title isEqualToString:@"Artist"]) 	col = tc_artist;
	else if ([title isEqualToString:@"Album"])		col = tc_album;
	else if ([title isEqualToString:@"Availability"])   col = tc_availability;
	else if ([title isEqualToString:@"Duration"])         col = tc_length;

	if (col) {
		if ([sender state]==NSOnState)
			[resultsOutlineView removeTableColumn:col];
		else {
			[resultsOutlineView addTableColumn:col];
			
			// if col wasn't already there when awakeFromNib was called
			// it will draw a white background, so we tell it to not do this
			// -rizzi
			//[[col dataCell] setDrawsBackground:NO];	// [AM] disabled
		}
		
		// the column configuration only gets saved after columns have been resized
		// -> dummy resize
		// -rizzi
		[[resultsOutlineView tableColumnWithIdentifier:@"file"] setWidth:[[resultsOutlineView tableColumnWithIdentifier:@"file"] width]+1.0];
		[[resultsOutlineView tableColumnWithIdentifier:@"file"] setWidth:[[resultsOutlineView tableColumnWithIdentifier:@"file"] width]-1.0];
	}
}

// [AM][TODO] auggh!!!! what a nightmare for localization... we should be using tags!!!
- (BOOL)validateMenuItem:(NSMenuItem *)menuItem
{
	NSString *title = [menuItem title];
	if ([title isEqualToString:@"File Icon"]) {
		if ([resultsOutlineView tableColumnWithIdentifier:@"icon"]) [menuItem setState:NSOnState];
		else [menuItem setState:NSOffState];
	}
	else if ([title isEqualToString:@"File"]) {
		[menuItem setState:NSOnState];
		return NO;
	}
	else if ([title isEqualToString:@"Network Icon"]) {
		if ([resultsOutlineView tableColumnWithIdentifier:@"PProtoIcon"]) [menuItem setState:NSOnState];
		else [menuItem setState:NSOffState];
	}
	else if ([title isEqualToString:@"User"]) {
		if ([resultsOutlineView tableColumnWithIdentifier:@"user"]) [menuItem setState:NSOnState];
		else [menuItem setState:NSOffState];
	}
	else if ([title isEqualToString:@"Size"]) {
		if ([resultsOutlineView tableColumnWithIdentifier:@"calcsize"]) [menuItem setState:NSOnState];
		else [menuItem setState:NSOffState];
	}
	else if ([title isEqualToString:@"Bitrate"]) {
		if ([resultsOutlineView tableColumnWithIdentifier:@"bitrate"]) [menuItem setState:NSOnState];
		else [menuItem setState:NSOffState];
	}
	else if ([title isEqualToString:@"Artist"]) {
		if ([resultsOutlineView tableColumnWithIdentifier:@"artist"]) [menuItem setState:NSOnState];
		else [menuItem setState:NSOffState];
	}
	else if ([title isEqualToString:@"Album"]) {
		if ([resultsOutlineView tableColumnWithIdentifier:@"album"]) [menuItem setState:NSOnState];
		else [menuItem setState:NSOffState];
	}
	else if ([title isEqualToString:@"Availability"]) {
		if ([resultsOutlineView tableColumnWithIdentifier:@"PAvailability"]) [menuItem setState:NSOnState];
		else [menuItem setState:NSOffState];
	}
	else if ([title isEqualToString:@"Duration"]) {
		if ([resultsOutlineView tableColumnWithIdentifier:@"formattedDuration"]) [menuItem setState:NSOnState];
		else [menuItem setState:NSOffState];
	}
	else if ([title isEqualToString:@"Download Selection(s)"]){
		if ([resultsOutlineView numberOfSelectedRows]<=0) return NO;
		else
			return YES;
	}
	else if ([title isEqualToString:@"Pause/Resume"]){
		if ([searchListTableView numberOfSelectedRows]<=0) return NO;
		else
			return YES;
	}
	else if ([title isEqualToString:@"Delete Search"]){
		if ([searchListTableView numberOfSelectedRows]<=0) return NO;
		else
			return YES;
	}
	else if ([title isEqualToString:@"Browse Host..."]){
		if ([resultsOutlineView numberOfSelectedRows]==1 && ![[[resultsOutlineView itemAtRow:[resultsOutlineView selectedRow]] objectForKey:@"PExpandable"] boolValue]) return YES;
		else return NO;
	}
	return YES;
}

- (NSMenu *)tableView:(NSOutlineView *)_table menuForTableColumn:(NSTableColumn *)_column row:(int)_row
{
	[[[NSApplication sharedApplication] mainWindow] makeFirstResponder:_table];
	if (![_table isRowSelected:_row]) [_table selectRow:_row byExtendingSelection:NO];
	return contextualMenu;
}
// ----------------------------------------------------------------------------

- (void)registerWithCommander
{
	[super registerWithCommander];

	[[HDDaemonCommander sharedInstance] registerController:self forCommands:[NSArray arrayWithObject:@"ITEM"]];
	// notification from HDAspectControllerDaemon -- tells us that user disconnected or that giftd crashed
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didDisconnect:) name:@"PConnectionClosedNotification" object:nil];
}

// respond to PoisonToCore Notification
- (void)didConnect:(NSNotification *)aNotification
{
	// if searches is not empty => giftd crashed
	// this means searches are already saved (including results)
	if ([[searchListArrayController arrangedObjects] count] == 0) {
	
		NSArray *oldSearches = [userDefaults arrayForKey:@"PSavedSearches"];
		
		NSString *ticket;
		NSMutableDictionary *search;
		HDSearchResultSource *new;
		
		NSEnumerator *e = [oldSearches objectEnumerator];
		id oldSearch;
		
		while (oldSearch = [e nextObject]) {
			search	= [oldSearch mutableCopy];
			
			// we need a new ticket, the old one was freed when we disconnected
			ticket = [[[[HDDaemonCommander sharedInstance] retrieveTicket] copy] autorelease];
			
			// create the result source for this search
			new 	= [[HDSearchResultSource alloc] initWithHashes:[self downloadingHashes] andTreeController:searchResultsTreeController andTable:resultsOutlineView hidden:NO];
			[new setActive:NO];
			
			[search setObject:ticket forKey:PZSearchListEntryTicketStringKey];
			[search setObject:@"0" forKey:PZSearchListEntryResultCountKey];
			
			// replace the old ticket in the query with the new one
			NSMutableArray *tmp = [[[search objectForKey:PZSearchListEntryQueryStringKey] componentsSeparatedByString:@")"] mutableCopy];
			[tmp replaceObjectAtIndex:0 withObject:[NSString stringWithFormat:@"SEARCH(%@",ticket]];
			[search setObject:[tmp componentsJoinedByString:@")"] forKey:PZSearchListEntryQueryStringKey];
			[tmp release];
			
			//[datasources setObject:new forKey:ticket];
			//[searches addObject:search];
			
			id newSearch = [[HDSearchListEntry alloc] init];
			[newSearch setEntryDictionary:search];
			[newSearch setEntryDataSource:new];
			[searchListArrayController addObject:newSearch];
			[newSearch release];
			
			[new release];
			[search release];
		}

		(void)[searchListArrayController setSelectedObjects:nil];
		[searchListTableView reloadData];
	}
}

- (void)didDisconnect:(NSNotification *)aNotification
{
	[super didDisconnect:aNotification];

	if ([[aNotification name] isEqualToString:@"PConnectionClosedNotification"]) {	// [AM] a bloody kludge for now; this is the only aspect controller that is not supposed to react to the DaemonConnectionClosed notification...
		id crash = [[aNotification userInfo] objectForKey:@"crash"];

		if (crash) { // giftd crashed -> set all search inactive
			NSEnumerator *e = [[searchListArrayController arrangedObjects] reverseObjectEnumerator];
			id search;
			while (search = [e nextObject]) {
				//[[datasources objectForKey:[search ticketString]] setActive:NO];
				[[search entryDataSource] setActive:NO];
			}
			[searchListTableView reloadData];
		}
		else { // -> remove all searches
			[self saveSearches];
			if (current_src) [current_src cleanUpTableHeaders];
			current_src = nil;
			NSEnumerator *e = [[searchListArrayController arrangedObjects] reverseObjectEnumerator];
			id search;
			id theCommander = [HDDaemonCommander sharedInstance];
			while (search = [e nextObject]) {
				[theCommander freeTicket:[search ticketString]];
			}
			//[searches removeAllObjects];
			{
				NSEnumerator *e = [[searchListArrayController arrangedObjects] objectEnumerator];
				id theSearch;
				
				while (theSearch = [e nextObject]) {
					[theSearch setEntryDataSource:nil];
				}
			}
			
			[searchListArrayController removeAllArrangedObjects:self];	// [AM] my addition
			//[datasources removeAllObjects];
			[searchListTableView reloadData];
			[resultsOutlineView setDataSource:self];
			[resultsOutlineView setDelegate:self];
			[resultsOutlineView reloadData];
			[filter setSearchDataSource:nil];
		}
	
		[filter disconnected];
	}
}

- (void)saveSearches
{
	//[userDefaults setObject:searches forKey:@"PSavedSearches"];
	[userDefaults setObject:[searchListArrayController valuesForKey:PZSearchListEntryDictionaryKey] forKey:@"PSavedSearches"];
	[userDefaults setObject:[searchField recentSearches] forKey:@"PRecentSearches"];
}

// process ITEM from the daemon
// --------------------------------------------------------------------------------------------------
- (oneway void)ITEM:(in NSArray *)data
{
	if (data) {
		NSString *ticket = [data objectAtIndex:1];			// get the command id
		if (ticket) {
		
			// [TODO] this is horribly inefficient, isn't it?
			id theSearch = nil;
			{
				NSEnumerator *e = [[searchListArrayController arrangedObjects] objectEnumerator];
				id search;
				
				while (search = [e nextObject]) {
					if ([[search ticketString] isEqualToString:ticket]) {
						theSearch = search;
						break;
					}
				}
			}
		
			id theCommander = [HDDaemonCommander sharedInstance];
			//BOOL hidden = [[datasources objectForKey:ticket] isHidden];
			BOOL hidden = [[theSearch entryDataSource] isHidden];

			if ([[data objectAtIndex:2] count]==0) {
				//[[datasources objectForKey:ticket] setActive:NO];
				[[theSearch entryDataSource] setActive:NO];
				
				// find more sources has finished, we can delete this search now
				if (hidden) {
					[theCommander freeTicket:ticket];
					[theSearch setDataSource:nil];
					//[datasources removeObjectForKey:ticket];
				}
				[searchListTableView reloadData];
				
			} else {
				NSDictionary *tmp = [data objectAtIndex:2];
				NSString *hash = [tmp objectForKey:HDSearchResultSourceHashKey];

				if (hash && [[self downloadingHashes] containsObject:hash]) {
					NSString *query = [NSString stringWithFormat:@"ADDSOURCE user(%@) hash(%@) size(%@) url(%@) save(%@)",
						[theCommander prepare:[tmp objectForKey:@"user"]],
						[theCommander prepare:hash],
						[tmp objectForKey:@"size"],
						[theCommander prepare:[tmp objectForKey:@"url"]],
						[theCommander prepare:[tmp objectForKey:@"file"]]
						];
					[theCommander executeCommand:query];
				}

				if (!hidden) {	// we're looking for more sources...
					//HDSearchResultSource *source = [datasources objectForKey:ticket];
					HDSearchResultSource *source = [theSearch entryDataSource];
					if (source)
					{
						// delay the refresh for 1 second to give a chance for more results to come in
						// this way we add items in chunks and the table gets refreshed less - jjt
						[[self newItems] addObject:data];
						if (![[self refreshTimer] isValid])
						{
							// the timer is not valid, so we need to schedule it
							[self setRefreshTimer:[NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(refreshTimerFired:) userInfo:NULL repeats:NO]];
						}
					}
				}
			}
		}
	}
}
// --------------------------------------------------------------------------------------------------

- (void)refreshTimerFired:(NSTimer *)timer
{
	NSEnumerator *iter = [[self newItems] objectEnumerator];
	NSArray *data;
	while (data = [iter nextObject])
	{
		HDSearchResultSource *source = nil;
		{
			NSEnumerator *e = [[searchListArrayController arrangedObjects] objectEnumerator];
			id search;
			
			while (search = [e nextObject]) {
				if ([[search ticketString] isEqualToString:[data objectAtIndex:1]]) {
					source = [search entryDataSource];
					break;
				}
			}
		}
	
		//HDSearchResultSource *source = [datasources objectForKey:[data objectAtIndex:1]];
		[source addItem:data];
	}
	[[self newItems] removeAllObjects];
	[self gui_update:([((HDMainController *)[NSApp delegate]) currentView] == 1)];
	// this is a single shot timer, so after this fire, it isn't scheduled anymore
	[self setRefreshTimer:nil];
}

- (void)deleteEvent:(id)sender
{
	NSArray *selected = [[searchListTableView selectedRowEnumerator] allObjects];
	NSEnumerator *e = [selected reverseObjectEnumerator];
	NSString *ticket;
	while (ticket = [e nextObject]) {
		id theCommander = [HDDaemonCommander sharedInstance];
		int row = [ticket intValue];
		id search = [searchListArrayController objectAtArrangedObjectIndex:row];
		ticket = [search ticketString];
		[theCommander executeCommand:[NSString stringWithFormat:@"SEARCH(%@) action(cancel)",ticket]];
		[theCommander freeTicket:ticket];
		[search setEntryDataSource:nil];
		//[datasources removeObjectForKey:ticket];
		//[searches removeObjectAtIndex:row];
		[searchListArrayController removeObjectAtArrangedObjectIndex:row];
	}
	
	[searchListTableView reloadData];
	
	// use tableViewSelectionDidChange instead of loadResultTable
	// possible fix for crash when deleting searches (???)
	// -rizzi
	//[self loadResultTable];
	[self tableViewSelectionDidChange:nil];
}

- (void)gui_update:(BOOL)activeView
{
	[self reloadSearchTable];
	if (activeView) [resultsOutlineView reloadData];
}

/*- (void)clearRecentSearches
{
	[recentSearches removeAllObjects];
}*/

/*- (void)addRecentSearch:(NSString *)search withRealm:(NSString *)realm
{
	unsigned int max = 15;
	// 0 -> menu item title (just needed for the menu item)
	// 1 -> search term
	// 2 -> realm
	NSMutableArray *item = [NSMutableArray array];
	if (realm) [item addObject:[NSString stringWithFormat:@"  %@ (%@)",search,realm]];
	else [item addObject:[NSString stringWithFormat:@"  %@",search]];
	[item addObject:search];
	if (realm) [item addObject:realm];
	
	// if this search is already stored in the list, remove it
	// and insert it again at the top of the list
	if ([recentSearches containsObject:item]) [recentSearches removeObject:item];
	[recentSearches insertObject:item atIndex:0];
	
	// check if the list is already bigger than allowed
	if ([recentSearches count]>max) [recentSearches removeObjectAtIndex:max];
}*/

- (void)filterDidDeactivate:(NSNotification *)aNotification {
	[self reloadSearchTable];
}

- (void)reloadSearchTable {
	NSEnumerator *e = [[searchListArrayController arrangedObjects] objectEnumerator];
	id search;
	while (search = [e nextObject]) {
		//NSString *ticket = [search ticketString];
		//NSString *resultCount  = [[datasources objectForKey:ticket] resultCount];
		NSString *resultCount = [(HDSearchResultSource *)[search entryDataSource] resultCount];
		[search setResultCount:resultCount];	//[search setValue:resultCount forKey:PZSearchListEntryResultCountKey];
	}
	[searchListTableView reloadData];
}

- (void)findMoreSources:(NSNotification *)notification
{
	NSString *hash = [[notification userInfo] objectForKey:HDSearchResultSourceHashKey];
	NSString *ticket = [[[[HDDaemonCommander sharedInstance] retrieveTicket] copy] autorelease];

	NSString *query = [NSString stringWithFormat:@"LOCATE(%@) query(%@)",ticket,hash];
	[self search:query info:@"" ticket:ticket hidden:YES];
}

- (void)search:(NSString *)_query info:(NSString *)_info ticket:(NSString *)_ticket hidden:(BOOL)_hidden
{
	[[HDDaemonCommander sharedInstance] executeCommand:_query];
	
	HDSearchResultSource *_new = [[HDSearchResultSource alloc] initWithHashes:[self downloadingHashes] andTreeController:searchResultsTreeController andTable:resultsOutlineView hidden:_hidden];
	//[datasources setObject:_new forKey:_ticket];
	if (!_hidden) {
		// insert the new search at the top of the list
		NSDictionary *search = [NSMutableDictionary dictionaryWithObjectsAndKeys:
			_ticket,PZSearchListEntryTicketStringKey,
			_query,PZSearchListEntryQueryStringKey,
			_info,PZSearchListEntryInfoStringKey,
			@"0",PZSearchListEntryResultCountKey,
			nil];
		//[searches insertObject:search atIndex:0];
		
		id newSearch = [[HDSearchListEntry alloc] init];
		[newSearch setEntryDictionary:search];
		[newSearch setEntryDataSource:_new];
		[searchListArrayController insertObject:newSearch atArrangedObjectIndex:0];
		[newSearch release];

		// select the new search at the top of the list
		[searchListTableView reloadData];
		[searchListTableView selectRow:0 byExtendingSelection:NO];
		[self loadResultTable:self];
	}
	[_new release];
}

- (IBAction)search:(id)sender
{
	if ([[HDDaemonCommander sharedInstance] isConnected]) {
		id stringValue = [sender stringValue];
		if(stringValue && ![stringValue isEqualToString:@""]) {
			NSLog(@"Search called...");
			
			NSString *ticket = [[[[HDDaemonCommander sharedInstance] retrieveTicket] copy] autorelease];
			NSString *query;
			if ([[searchRealmPopUpButton title] isEqualToString:@"Everything"]) {
				query = [NSString stringWithFormat:@"SEARCH(%@) query(%@)",
					ticket,
					[[HDDaemonCommander sharedInstance] prepare:[searchField stringValue]]
					];
				// add to the list of recent searches
				//[self addRecentSearch:[sender stringValue] withRealm:nil];
			}
			else { 
				query = [NSString stringWithFormat:@"SEARCH(%@) query(%@) realm(%@)",
					ticket,
					[[HDDaemonCommander sharedInstance] prepare:[searchField stringValue]],
					[[searchRealmPopUpButton title] lowercaseString]
					];
				// add to the list of recent searches
				//[self addRecentSearch:[sender stringValue] withRealm:[searchRealmPopUpButton title]];
			}
			[self search:query info:[searchField stringValue] ticket:ticket hidden:NO];
			[searchField selectText:self];
		}
	}
}

/*- (void)searchRecent:(id)sender
{
	NSLog(@"searchRecent called...");
	NSArray *rep_obj = [[sender representedObject] retain];
	NSString *ticket = [[[commander retrieveTicket] copy] autorelease];
	NSString *term = [rep_obj objectAtIndex:1];
	NSString *query;
	if ([rep_obj count]==3) {
		query = [NSString stringWithFormat:@"SEARCH(%@) query(%@) realm(%@)",
					ticket,
					[commander prepare:term],
					[rep_obj objectAtIndex:2]
				];
	}
	else {
		query = [NSString stringWithFormat:@"SEARCH(%@) query(%@)",
					ticket,
					[commander prepare:term]
				];
	}

	// start the search
	[self search:query info:term ticket:ticket hidden:NO];
	// set the search field
	//[searchField setStringValue:term];
	//[searchField selectText:self];
	//[searchField setStringValue:@""];

	// finally move this search to the top of the recent searches list
	[recentSearches removeObject:rep_obj];
	[recentSearches insertObject:rep_obj atIndex:0];
	[rep_obj release];
}*/

- (IBAction)browse:(id)sender
{
	NSString *user = [[resultsOutlineView itemAtRow:[resultsOutlineView selectedRow]] objectForKey:@"user"];
	[self browseHost:user];
}

- (void)browseHost:(NSString *)user
{
	NSString *ticket = [[[[HDDaemonCommander sharedInstance] retrieveTicket] copy] autorelease];
	NSString *query = [NSString stringWithFormat:@"BROWSE(%@) query(%@)",ticket,user];
	[self search:query info:[NSString stringWithFormat:@"browse %@",user] ticket:ticket hidden:NO];
}

- (IBAction)download:(id)sender
{
	if ([resultsOutlineView numberOfSelectedRows]==0) return;
	if ([[NSUserDefaults standardUserDefaults] boolForKey:@"HDDownloadsSwitchView"]) {
		[(HDMainController *)[NSApp delegate] performSelector:@selector(switchToDownloads:)];
	}
	NSArray *sources = [current_src selectedItems];
	NSString *hash;
	NSEnumerator *e = [sources objectEnumerator];
	NSDictionary *tmp;
	NSString *query;
	while (tmp = [e nextObject]) {
		id theCommander = [HDDaemonCommander sharedInstance];
		hash = [tmp objectForKey:HDSearchResultSourceHashKey];
		if (hash) {
			hash = [theCommander prepare:hash];
			query = [NSString stringWithFormat:@"ADDSOURCE user(%@) hash(%@) size(%@) url(%@) save(%@)",
				[theCommander prepare:[tmp objectForKey:@"user"]],
				[theCommander prepare:hash],
				[tmp objectForKey:@"size"],
				[theCommander prepare:[tmp objectForKey:@"url"]],
				[theCommander prepare:[tmp objectForKey:@"file"]]
				];
		}
		else {
			query = [NSString stringWithFormat:@"ADDSOURCE user(%@) size(%@) url(%@) save(%@)",
				[theCommander prepare:[tmp objectForKey:@"user"]],
				[tmp objectForKey:@"size"],
				[theCommander prepare:[tmp objectForKey:@"url"]],
				[theCommander prepare:[tmp objectForKey:@"file"]]
				];
		}
		//NSLog(@"%@",query);
		[theCommander executeCommand:query];
	}
}

- (void)toggleSelectedSearchStatus:(id)sender {
	NSEnumerator *e = [searchListTableView selectedRowEnumerator];
	id row;
	
	while (row = [e nextObject]) {
		id obj = [searchListArrayController objectAtArrangedObjectIndex:[row intValue]];
		id entryDataSource = [obj entryDataSource];
		[entryDataSource setActive:![entryDataSource isActive]];
	}
	//int row = [searchListTableView selectedRowEnumerator];
	//if (row!=-1) {
	//	id entryDataSource = [[searchListArrayController objectAtArrangedObjectIndex:row] entryDataSource];
	//	[entryDataSource setActive:![entryDataSource isActive]];
	//}
	[self toggleSearches:sender];
}

- (void)toggleSearches:(id)sender
{
	// by now, the -(is)Active state of the data source will have *already* been changed, so we've reversed the start/stop search tests below
	//NSEnumerator *e = [[searchListArrayController selectedObjects] objectEnumerator];
	NSEnumerator *e = [searchListTableView selectedRowEnumerator];
	id row;
	
	while (row = [e nextObject]) {
		id search = [searchListArrayController objectAtArrangedObjectIndex:[row intValue]];
		NSString *ticket = [search ticketString];
		//if ([[datasources objectForKey:ticket] isActive])		// STOP SEARCH
		if (![[search entryDataSource] isActive]) {		// STOP SEARCH
			[[HDDaemonCommander sharedInstance] executeCommand:[NSString stringWithFormat:@"SEARCH(%@) action(cancel)",ticket]];
			//[[datasources objectForKey:ticket] setActive:NO];
			//[[search entryDataSource] setActive:NO];
		}
		else {							// RE-SEARCH
			[[HDDaemonCommander sharedInstance] executeCommand:[search queryString]];
			//[[datasources objectForKey:ticket] setActive:YES];
			//[[search entryDataSource] setActive:YES];
		}
		[self reloadSearchTable];
	}
}

- (BOOL)validateToolbarItem:(NSToolbarItem *)theItem
{
	NSString *ident = [theItem itemIdentifier];
	if ([ident isEqualToString:@"searchDownload"])
		return ([resultsOutlineView numberOfSelectedRows]>0);
	else if ([ident isEqualToString:@"searchBrowse"]) {
		return (([resultsOutlineView numberOfSelectedRows]==1) && (![[resultsOutlineView itemAtRow:[resultsOutlineView selectedRow]] objectForKey:@"expandable"]));
	}
	else if ([ident isEqualToString:@"searchStop"]) {
		if ([searchListTableView numberOfSelectedRows]!=1) return NO;
		int row = [searchListTableView selectedRow];
		if (row!=-1) {
			id dataSource = [[searchListArrayController objectAtArrangedObjectIndex:row] entryDataSource];
			if ([dataSource isActive]) {
				[theItem setLabel:@"Stop Search"];
				[theItem setPaletteLabel:@"Stop Search"];
				[theItem setToolTip:@"Stop the currently viewed search"];
				[theItem setImage:[NSImage imageNamed:@"Stop"]];
			} else {
				[theItem setLabel:@"Start Search"];
				[theItem setPaletteLabel:@"Start Search"];
				[theItem setToolTip:@"Start the currently viewed search"];
				[theItem setImage:[NSImage imageNamed:@"Spotlight"]];
			}
			
			return YES;
		}
		//if (row!=-1 && [[datasources objectForKey:[[searchListArrayController objectAtArrangedObjectIndex:row] ticketString]] isActive]) return YES;
		else return NO;
	}
	else if ([ident isEqualToString:@"iTMS"]) return [self validateBuyAtiTMS];
	else return YES;
	
}

- (void)buyAtiTMS:(id)sender
{
	if ([resultsOutlineView numberOfSelectedRows]!=1) return;
	NSString* artistTerm=@""; 		// empty string instead of (null)
	NSString* titleTerm=@""; 
	NSURL* url;
	NSRange beginRange, endRange;	
	NSDictionary *item = [resultsOutlineView itemAtRow:[resultsOutlineView selectedRow]];	// information on selected item

		// check for no meta tags
		if (![item objectForKey:@"artist"] && ![item objectForKey:@"title"]) 
		{
		
			// most mp3 files downloaded will have a - before the name of the song at the end of a file name
			// some have "02 <name of song>.mp3" due to itunes, the following 3 lines will work that all out.
			// all followed by a .mp3 extension which is worked out at the endRange
			
			beginRange = [[item objectForKey:@"file"] rangeOfString:@"-" options:NSBackwardsSearch];
			if ( beginRange.length==0 && 
					[[[item objectForKey:@"file"] substringWithRange: NSMakeRange(0, 2)] intValue] ) 
					beginRange = [[item objectForKey:@"file"] rangeOfString:@" "];

			endRange = [[item objectForKey:@"file"] rangeOfString:@"." options:NSBackwardsSearch];
			
			// parse the string for name of song and replace spaces for web sending. Now this is not perfect and some oddly named
			// files will be parsed improperly but there is no control over how people name files so no parser is perfect. This 
			// will get 90% of all files which do not have meta-tags or 99.9% of all files in total.
			
			titleTerm = (NSString*)CFURLCreateStringByAddingPercentEscapes(0,(CFStringRef)[
						[item objectForKey:@"file"] substringWithRange:
						NSMakeRange(beginRange.location+1,   		// eat the char we searched for, spaces ignored by itunes
						(endRange.location-(beginRange.location+1)))
						],0,0,kCFStringEncodingISOLatin1);
			url = [NSURL URLWithString:[NSString stringWithFormat:
				@"itms://phobos.apple.com/WebObjects/MZSearch.woa/wa/advancedSearchResults?songTerm=%@", 
				titleTerm]];
		} 
		else 
		{
		
			// parsing meta tags spaces and adding them to the url string
			
			if ([item objectForKey:@"artist"])
				artistTerm = (NSString*)CFURLCreateStringByAddingPercentEscapes(0,(CFStringRef)
					[item objectForKey:@"artist"],0,0,kCFStringEncodingISOLatin1);
			if ([item objectForKey:@"title"])
				titleTerm = (NSString*)CFURLCreateStringByAddingPercentEscapes(0,(CFStringRef)
					[item objectForKey:@"title"],0,0,kCFStringEncodingISOLatin1);
	
			url = [NSURL URLWithString:[NSString stringWithFormat:
				@"itms://phobos.apple.com/WebObjects/MZSearch.woa/wa/advancedSearchResults?songTerm=%@&artistTerm=%@",
				titleTerm, artistTerm]];
		}
		
		// send url to system
		[[NSWorkspace sharedWorkspace] openURL:url];

}

- (BOOL)validateBuyAtiTMS
{
	BOOL validated = NO;

	if ([resultsOutlineView numberOfSelectedRows]==1) {
		if ([HDMediaTypeDetector mediaTypeIsMusic:[[[[resultsOutlineView itemAtRow:[resultsOutlineView selectedRow]] objectForKey:@"file"] pathExtension] lowercaseString]])
			validated = YES;
	}
	
	return validated;
}

#pragma mark -
#pragma mark Table Views and Outline Views

/*- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
	return [searches count];
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{

	NSDictionary *item = [searches objectAtIndex:rowIndex];
	return [NSArray arrayWithObjects:
				[item objectForKey:PZSearchListEntryInfoStringKey],
				[item objectForKey:PZSearchListEntryResultCountKey],
				nil];
}*/

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
	// first deselect table column header
	NSTableColumn *selected = [resultsOutlineView highlightedTableColumn];
	if (selected) {
		[resultsOutlineView setIndicatorImage:nil inTableColumn:selected];
		[resultsOutlineView setHighlightedTableColumn:nil];
	}
	
	int row = [searchListTableView selectedRow];
	if (row!=-1 && [searchListTableView numberOfSelectedRows]==1) {
		current_src = [[searchListArrayController objectAtArrangedObjectIndex:row] entryDataSource];
		//current_src = [datasources objectForKey:[[searchListArrayController objectAtArrangedObjectIndex:row] ticketString]];
		[resultsOutlineView setDataSource:current_src];
		[resultsOutlineView setDelegate:current_src];
		[filter setSearchDataSource:current_src];
		[current_src setTableHeaders];
	}
	else {
		[resultsOutlineView setDataSource:self];
		[resultsOutlineView setDelegate:self];
		[filter setSearchDataSource:nil];	// [TODO][CRASH] this line causes a crash because:
		//2007-09-02 13:16:47.168 Hedera[5495] An instance 0x3a1000 of class HDSearchResultSource is being deallocated while key value observers are still registered with it.  Break on _NSKVODeallocateLog to start debugging.
		
		current_src = nil;
	}
	[resultsOutlineView reloadData];
	[searchListTableView reloadData];
}

- (void)loadResultTable:(id)sender
{
	if ([searchListTableView numberOfSelectedRows]==1) {
		[(HDMainController *)[NSApp delegate] performSelector:@selector(switchToSearch:) withObject:sender];
	}
	[self tableViewSelectionDidChange:nil];
}

- (void)tableView:(NSTableView *)aTableView willDisplayCell:(id)aCell 
	forTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
	NSString *identifier = [aTableColumn identifier];
	if ([identifier isEqualToString:PZSearchListEntrySummaryKey]) {
		BOOL shouldHighlight = ([[aTableView selectedRowIndexes] containsIndex:rowIndex] && [[NSApp mainWindow] firstResponder] == aTableView);
		id search = [searchListArrayController objectAtArrangedObjectIndex:rowIndex];
		NSAttributedString *attributedString = [search summaryAttributedStringWithHighlight:shouldHighlight];
		[aCell setAttributedStringValue:attributedString];
	}
}
 
// dummy datasource for result table... (we use special data source objects instead...)
- (id)outlineView:(NSOutlineView *)outlineView child:(int)index ofItem:(id)item
{
	return nil;
}

- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item
{
	return NO;
}

- (int)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item
{
	return 0;
}

- (id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item 
{
	return nil;
}

- (BOOL)outlineView:(NSOutlineView *)outlineView shouldSelectTableColumn:(NSTableColumn *)tableColumn
{
	return NO;
}

// search field delegate
/*- (NSMenu *)willPopUpMenuForTextField:(PTextField *)textfield
{   
	NSMenu *menu = [[[NSMenu alloc] init] autorelease];
	[menu setMenuRepresentation: [[[NSMenuView alloc]
	initWithFrame: NSZeroRect] autorelease]];
	[[menu menuRepresentation] setFont: [NSFont menuFontOfSize:
	[NSFont smallSystemFontSize]]];
	[menu sizeToFit];
	// end set small menu
	NSMenuItem *mitem;
	int i, count = [recentSearches count];

	if (count==0) {
		[menu addItemWithTitle:@"No recent searches" action:nil keyEquivalent:@""];
		return menu;
	}
	
	[menu addItemWithTitle:@"Recent Searches" action:nil keyEquivalent:@""];
	
	for (i=0;i<count;i++) {
		mitem = [[NSMenuItem alloc] 
			initWithTitle:[[recentSearches objectAtIndex:i] objectAtIndex:0] 
					action:@selector(searchRecent:) 
			keyEquivalent:@""];
		[mitem setRepresentedObject:[recentSearches objectAtIndex:i]];
		[mitem setTarget:self];
		[menu addItem:mitem];
		[mitem release];
	}
	
	[menu addItem:[NSMenuItem separatorItem]];
	
	mitem = [[NSMenuItem alloc] initWithTitle:@"Clear Entries" action:@selector(clearRecentSearches) keyEquivalent:@""];
	[mitem setTarget:self];
	[menu addItem:mitem];
	[mitem release];
	
	
	return menu;
}*/

@end

#pragma mark -

@implementation HDAspectControllerSearch (UICoverMethods)

- (NSSearchField *)toolbarSearchField {
	return [[searchField retain] autorelease];
}

- (NSPopUpButton *)toolbarSearchRealmPopUpButton {
	return [[searchRealmPopUpButton retain] autorelease];
}

- (NSView *)drawerView {
	return [[drawerView retain] autorelease];
}

@end


@implementation HDAspectControllerSearch (Accessors)

- (NSTimer *)refreshTimer {
	return [[AM_refreshTimer retain] autorelease];
}

- (void)setRefreshTimer:(NSTimer *)aTimer {
	if (aTimer != AM_refreshTimer) {
		[AM_refreshTimer release];
		AM_refreshTimer = [aTimer retain];
	}
}

- (NSMutableArray *)newItems {
	return [[AM_newItems retain] autorelease];
}

- (void)setNewItems:(NSArray *)anArray {
	if (anArray != AM_newItems) {
		[AM_newItems release];
		AM_newItems = [anArray mutableCopy];
	}
}

- (NSSet *)downloadingHashes {
	return [[AM_downloadingHashes retain] autorelease];
}

- (void)setDownloadingHashes:(NSSet *)aSet
{
	if (aSet != AM_downloadingHashes) {
		[AM_downloadingHashes release];
		AM_downloadingHashes = [aSet retain];
	}
}

@end
