//
// HDAspectControllerUploads.m
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import "HDAspectControllerUploads.h"
#import "HDDaemonCommander.h"
#import "PUploadTitleDescriptionCell.h"
#import "PSizeCell.h"
#import "PUploadProgressCell.h"
#import "PUploadsTableView.h"

#import "HDTransferSourceUploads.h"
#import "HDMainToolbarCategory.h"


@implementation HDAspectControllerUploads

- (id)init {
	if (nil != (self = [super init])) {
		if (![NSBundle loadNibNamed:@"AspectControllerUploads" owner:self]) {
            [self release];
            self = nil;
        }
	}
	return self;
}

- (void)dealloc
{
	// [AM] begin new stuff
		[table release];    
		[contextualMenu release];

		table = nil;    
		contextualMenu = nil;
	// [AM] end new stuff


	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[dataSource release];
	[super dealloc];
}

- (void)awakeFromNib
{
	dataSource = [[HDTransferSourceUploads alloc] initWithTable:table];
	[table setMenuDelegate:self];
	[table setDataSource:dataSource];
	[table setDelegate:dataSource];
	
	//[table setDrawsGrid:YES];
	/*NSArray *cols = [table tableColumns];
	int i, count=[cols count];
	for (i=0;i<count;i++) [[[cols objectAtIndex:i] dataCell] setDrawsBackground:NO];*/

	//[table setRowHeight:34];
	[[table tableColumnWithIdentifier:@"PFileUser"] setDataCell:
		[[[PUploadTitleDescriptionCell alloc] init] autorelease]
	];
	[[table tableColumnWithIdentifier:@"PTransfer"] setDataCell:
		[[[PSizeCell alloc] init] autorelease]
	];
	[[table tableColumnWithIdentifier:@"PSize"] setDataCell:
		[[[PSizeCell alloc] init] autorelease]
	];
	[[table tableColumnWithIdentifier:@"PProgress"] setDataCell:
		[[[PUploadProgressCell alloc] init] autorelease]
	];
	/*[[table tableColumnWithIdentifier:@"PIcon"] setDataCell:
		[[[NSImageCell alloc] init] autorelease]
	];*/
}

- (void)registerWithCommander
{
	[super registerWithCommander];

	[[HDDaemonCommander sharedInstance] registerController:self forCommands:
		[NSArray arrayWithObjects:@"ADDUPLOAD",@"CHGUPLOAD",@"DELUPLOAD",nil]
	];

	// datasource needs the commander for cancelling uploads
	//[dataSource setCommander:aCommander];
}

- (void)didDisconnect:(NSNotification *)aNotification
{
	[super didDisconnect:aNotification];
	[dataSource disconnected];
}

/*- (void)gui_update:(BOOL)activeView
{
	//if (activeView) [table reloadData];
}*/

- (IBAction)cancelUpload:(id)sender
{
	[dataSource cancel:[HDDaemonCommander sharedInstance]];
}

- (IBAction)pauseUpload:(id)sender
{
	[dataSource pause:[HDDaemonCommander sharedInstance]];
}

- (IBAction)resumeUpload:(id)sender
{
	[dataSource resume:[HDDaemonCommander sharedInstance]];
}

- (IBAction)delsource:(id)sender
{
	[dataSource delsource:[HDDaemonCommander sharedInstance]];
}

- (NSString *)hostForBrowsing;
{
	return [[[[dataSource itemAtRow:[table selectedRow]] 
				objectForKey:@"PSources"]
				objectAtIndex:0]
				objectForKey:@"user"];
}

- (IBAction)browseUploadHost:(id)sender
{
	[[NSNotificationCenter defaultCenter] postNotificationName:@"PUpBrowseHost" object:self userInfo:nil];
}


- (IBAction)cleanUpUploads:(id)sender
{
	[dataSource cleanUp:[HDDaemonCommander sharedInstance]];
}

- (int)uploadSpeed
{
	return [dataSource transferSpeed];
}

- (NSMenu *)tableView:(NSTableView *)_table menuForTableColumn:(NSTableColumn *)_column row:(int)_row
{
	[[[NSApplication sharedApplication] mainWindow] makeFirstResponder:_table];
	if (![_table isRowSelected:_row]) [_table selectRow:_row byExtendingSelection:NO];
	return contextualMenu;
}

- (BOOL)validateToolbarItem:(NSToolbarItem *)theItem
{
	NSString *ident = [theItem itemIdentifier];
	if ([ident isEqualToString:@"upCancel"]) {
		return [dataSource validateCancel];
	}
	else if ([ident isEqualToString:@"upPause"]) {
		return [dataSource validatePause];
	}
	else if ([ident isEqualToString:@"upResume"]) {
		return [dataSource validateResume];
	}
	else if ([ident isEqualToString:@"upDelSource"]) {
		return [dataSource validateDelSource];
	}
	else if ([ident isEqualToString:@"browseUploadHost"]) {
		if (([table numberOfSelectedRows]==1)) return YES;
		else return NO;
	}
	else return YES;
}

- (BOOL)validateMenuItem:(NSMenuItem *)menuItem
{
	BOOL isValidated = YES;
	SEL action = [menuItem action];

	if (action == @selector(cancelUpload:)) {
		isValidated = [dataSource validateCancel];
	}
	else if (action == @selector(browseUploadHost:)) {
		if ([table numberOfSelectedRows]==1) {
			isValidated = YES;
		}
		else {
			isValidated = NO;
		}
	}
	
	return isValidated;
}

#pragma mark -
#pragma mark Download Commands

- (oneway void)ADDUPLOAD:(in NSArray *)data 
{
	if (data) {
		[[HDDaemonCommander sharedInstance] removeTicket:[data objectAtIndex:1]];
		[dataSource ADDUPLOAD:data];
	}
}

- (oneway void)CHGUPLOAD:(in NSArray *)data
{
	if (data) {
		[dataSource CHGUPLOAD:data];
		[[NSApp delegate] validateToolbar];
	}
}

- (oneway void)DELUPLOAD:(in NSArray *)data
{
	if (data) {
		[[HDDaemonCommander sharedInstance] freeTicket:[data objectAtIndex:1]];
		[dataSource DELUPLOAD:data];
		[[NSApp delegate] validateToolbar];
	}
}

- (void)ADDSOURCE:(NSArray *)data
{
	if (data) {
		[dataSource ADDSOURCE:data];
	}
}

- (void)DELSOURCE:(NSArray *)data
{
	if (data) {
		[dataSource DELSOURCE:data];
	}
}

@end
