//
// HDMainController.h
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import <Cocoa/Cocoa.h>
#import "HDHederaCommands.h"
#import "HDPreferencesConfigurationEditorOpenFT.h"

enum {
	PZDownloadStatusZero	= 0,
	PZDownloadStatusOne		= 1,
	
	//PZDownloadStatusCancelled		= 2,	// [AM] my creation; indicates cancellation
};

/*enum {
	PZCompletedDaemonShutdown = 0,
	PZCancelDaemonShutdown = 1,
};*/

enum {
	PZAspectPaneNone		= -1,

	PZAspectPaneDaemon	= 0,
	PZAspectPaneSearch		= 1,
	PZAspectPaneDownloads	= 2,
	PZAspectPaneUploads		= 3,
};

enum {
	HDMainControllerDaemonStatusZero	= 0,
	HDMainControllerDaemonStatusOne	= 1,
};

extern NSString *HDDaemonName;

@class HDAspectControllerDaemon;
@class HDAspectControllerSearch;
@class HDAspectControllerDownloads;
@class HDAspectControllerUploads;

@class HDPreferencesWindowController;

@class HDSearchFilterController;
//@class SUUpdater;


@interface HDMainController : NSObject <HDHederaCommands>
{
	BOOL programHasFinishedLaunching;

	NSString *daemonWindowTitle;
	NSString *searchWindowTitle;
	NSString *downloadsWindowTitle;
	NSString *uploadsWindowTitle;

	IBOutlet NSSegmentedControl *modeControl;
	//IBOutlet NSView *modeControlSuperview;

	HDAspectControllerDaemon *daemonController;
	HDAspectControllerSearch *searchController;
	HDAspectControllerDownloads *downloadsController;
	HDAspectControllerUploads *uploadsController;

	IBOutlet NSView *mainView;
	IBOutlet NSTabView *contentView;    
	IBOutlet NSDrawer *drawer;
	
	//IBOutlet NSView *navigation;
	//IBOutlet NSImageView *navigationImage;
	//IBOutlet NSButton *drawerButton;
	
	IBOutlet NSTextField *transferField;
	
	//IBOutlet NSView *globalSplit;	// this is the drawer view
	
	// -----------------------------------------------------------
	// this one is needed for the correct background in the drawer
	// - brushed -> no background
	// - aqua    -> draw background
	//IBOutlet NSButton *dummyBackground;
	// -----------------------------------------------------------
	
	NSMutableArray *toolbarAllowedItems;
	NSMutableArray *toolbarDefaultItems;
	//NSMutableArray *toolbarSelectableItems;
		
	//NSToolbar *toolbar;
	IBOutlet NSWindow *mainWindow;
	
	IBOutlet HDPreferencesWindowController *preferencesWindowController;
		
	int nextUpdate;
	int currentView;
	
	//NSMutableDictionary *navImages;
	NSArray *currentNav;
	
	IBOutlet NSPanel *progressWindow;
	IBOutlet NSTextField *progressTitle;
	IBOutlet NSTextField *progressTitle2;
	IBOutlet NSProgressIndicator *progressBar;
	
	
	
	// begin [AM] additions
	IBOutlet NSTextField *statsTextField;
	IBOutlet NSImageView *statusImageView;
	IBOutlet NSMenuItem *connectToLocalDaemonMenuItem;
	IBOutlet NSMenuItem *connectToRemoteDaemonMenuItem;
	//IBOutlet NSMenuItem *localConnectMenuItem;
	//IBOutlet NSMenuItem *remoteConnectMenuItem;
	
	NSArray *updatesData;
	
	IBOutlet NSPanel *daemonShutdownPanel;
	IBOutlet NSProgressIndicator *daemonShutdownProgressBar;
	BOOL AM_canceledDaemonShutdown;
	
	BOOL AM_didCancelDaemonUpdate;
	
	//IBOutlet SUUpdater *sparkleUpdater;
	// end [AM] additions
}

- (void)applicationDidFinishLaunching:(NSNotification *)notification;
- (void)completeApplicationLaunch:(BOOL)firstRun;

//- (void)fatalAlertSheetDidEnd:(NSAlert *)alert returnCode:(int)returnCode contextInfo:(void *)contextInfo;

- (void)setupDrawer;

- (int)stopDaemon;
- (void)cancelStopDaemonPanel:(id)sender;
//- (void)stopDaemonSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
- (void)shutdownDaemon;
- (BOOL)canceledDaemonShutdown;
- (void)setCanceledDaemonShutdown:(BOOL)flag;

- (int)currentView;
- (void)setCurrentView:(int)anInt;

- (void)activate:(id)sender;
- (void)didDisconnect:(NSNotification *)aNotification;

//- (IBAction)switchAppearance:(id)sender;
- (IBAction)drawerAction:(id)sender;
- (IBAction)newSearch:(id)sender;

- (void)noteCommandToBrowseDownloadHost:(NSNotification *)aNotification;
- (void)noteCommandToBrowseUploadHost:(NSNotification *)aNotification;

- (IBAction)giFTViewer:(id)sender;

//- (void)prepareWindow;
- (void)prepareMainTabView;
//- (void)setSavedDrawerSize;
- (IBAction)resetProgram:(id)sender;

#pragma mark -
#pragma mark Web Links

- (IBAction)visitHederaWebsite:(id)sender;
- (IBAction)visitPoisonedWebsite:(id)sender;
- (IBAction)visitGiFTWebsite:(id)sender;


#pragma mark -
#pragma mark Connection Methods

- (oneway void)ADDSOURCE:(in NSArray *)data;
- (oneway void)DELSOURCE:(in NSArray *)data;
- (oneway void)POISON_GUI_UPDATE:(in id)sender;
- (oneway void)REMOVE_UNKNOWN_COMMAND_TICKET:(in NSArray *)data;

@end

#pragma mark -

@interface HDMainController (CoverAccessors)

- (NSTextField *)statsTextField;
- (NSImageView *)statusImageView;

- (NSMenuItem *)localConnectMenuItem;
- (NSMenuItem *)remoteConnectMenuItem;

@end

@interface HDMainController (AspectMethods)

- (void)switchPane:(id)sender;

- (IBAction)switchToDaemon:(id)sender;
- (IBAction)switchToSearch:(id)sender;
- (IBAction)switchToDownloads:(id)sender;
- (IBAction)switchToUploads:(id)sender;

@end

@interface HDMainController (DaemonUpdater)

- (void)updateGiFT;
- (void)updateGiFTAlertSheetDidEnd:(NSAlert *)alert returnCode:(int)returnCode contextInfo:(void *)contextInfo;

- (IBAction)giftupdateCheck:(id)sender;

- (void)cancelDaemonUpdate:(id)sender;
- (BOOL)didCancelDaemonUpdate;
- (void)setDidCancelDaemonUpdate:(BOOL)flag;

- (void)startDownloadingURL:(id)theURL withFilePath:(id)filePath;
- (void)download:(NSURLDownload *)download didFailWithError:(NSError *)error;
- (void)downloadDidFailAlertSheetDidEnd:(NSAlert *)alert returnCode:(int)returnCode contextInfo:(void *)contextInfo;

- (void)downloadDidFinish:(NSURLDownload *)download;
- (void)download:(NSURLDownload *)download didCreateDestination:(NSString *)path;
- (void)download:(NSURLDownload *)download didReceiveResponse:(NSURLResponse *)response;
- (void)download:(NSURLDownload *)download didReceiveDataOfLength:(unsigned)length;
- (void)startUpdate:(int) status;
//- (void)giFTUpdatePanelDidEnd:(NSSavePanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;

@end
