//
// HDMainController.m
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import <SystemConfiguration/SystemConfiguration.h>

#import "HDMainController.h"
//#import "Sparkle/SUConstants.h"
#import "HDPreferencesWindowController.h"

#import "HDAspectControllerDaemon.h"
#import "HDAspectControllerSearch.h"
#import "HDAspectControllerDownloads.h"
#import "HDAspectControllerUploads.h"

#import "HDMainToolbarCategory.h"
#import "FileHandle.h"

#import "HDDaemonCommander.h"

#import "HDAlertLocalizedStringKeys.h"
#import "HDAdditions.h"

#import "HDProcessLister.h"


NSString *HDHederaURL = @"http://hedera.sourceforge.net/";
NSString *HDPoisonedURL = @"http://gottsilla.net/";
NSString *HDGiFTURL = @"http://gift.sourceforge.net/";	// the URL "http://giftproject.org" no longer works! :(

NSString *HDDaemonName = @"giftd";	// also used in HDAspectControllerDaemon, so declared as "extern" in header

static NSString *PZNetworkUpdateDisplayNameKey = @"displayName";
static NSString *PZNetworkUpdateNetworkNameKey = @"networkName";
static NSString *PZNetworkUpdateDownloadURLKey = @"downloadURL";
static NSString *PZNetworkUpdateLocalPathKey = @"localPath";
//static NSString *PZNetworkUpdateCheckedKey = @"checked";

static NSString *PZKCEasyHostName = @"update.kceasy.com";


@implementation HDMainController

- (id)init {
	if (nil != (self = [super init])) {
		// prepare window titles
		{
			NSBundle *mainBundle = [NSBundle mainBundle];
			NSString *programName = [mainBundle objectForInfoDictionaryKey:(NSString *)kCFBundleNameKey];	// will return a localized version, if available
			
			NSString *daemonString = NSLocalizedString(@"Daemon", @"Daemon");
			NSString *searchString = NSLocalizedString(@"Search", @"Search");
			NSString *downloadsString = NSLocalizedString(@"Downloads", @"Downloads");
			NSString *uploadsString = NSLocalizedString(@"Uploads", @"Uploads");
			
			daemonWindowTitle = [[NSString alloc] initWithFormat:@"%@ - %@", programName, daemonString];
			searchWindowTitle = [[NSString alloc] initWithFormat:@"%@ - %@", programName, searchString];
			downloadsWindowTitle = [[NSString alloc] initWithFormat:@"%@ - %@", programName, downloadsString];
			uploadsWindowTitle = [[NSString alloc] initWithFormat:@"%@ - %@", programName, uploadsString];
		}
		
		daemonController = [[HDAspectControllerDaemon alloc] init];
		searchController = [[HDAspectControllerSearch alloc] init];
		downloadsController = [[HDAspectControllerDownloads alloc] init];
		uploadsController = [[HDAspectControllerUploads alloc] init];
		
		// load network data for giFT updates
		{
			NSBundle *bundle = [NSBundle mainBundle];
			NSString *path = [bundle pathForResource:@"Updates" ofType:@"plist"];
			updatesData = [[NSArray alloc] initWithContentsOfFile:path];
		}
		
		//NSDictionary *checkForUpdatesDefault = [NSDictionary dictionaryWithObject:[NSNumber numberWithBool:YES] forKey:SUCheckAtStartupKey];
		//[[NSUserDefaultsController sharedUserDefaultsController] setInitialValues:checkForUpdatesDefault];
	}
	return self;
}

- (void)dealloc
{
	[daemonWindowTitle release];
	[searchWindowTitle release];
	[downloadsWindowTitle release];
	[uploadsWindowTitle release];
	
	daemonWindowTitle = nil;
	searchWindowTitle = nil;
	downloadsWindowTitle = nil;
	uploadsWindowTitle = nil;

	[daemonController release];		// [AM] added 2007-08-19
	[searchController release];		// [AM] added 2007-08-19
	[downloadsController release];	// [AM] added 2007-08-19
	[uploadsController release];	// [AM] added 2007-08-19

	daemonController = nil;		// [AM] added 2007-08-19
	searchController = nil;		// [AM] added 2007-08-19
	downloadsController = nil;	// [AM] added 2007-08-19
	uploadsController = nil;	// [AM] added 2007-08-19
	

	[[NSNotificationCenter defaultCenter] removeObserver:self];
	//[mainWindow release];
	//[navImages release];
	//[drawerButton release];
		
	// HDMainToolbarCategory
	[toolbarAllowedItems release];
	[toolbarDefaultItems release];
	//[toolbarSelectableItems release];
	
	toolbarAllowedItems = nil;
	toolbarDefaultItems = nil;
	//toolbarSelectableItems = nil;
	
	[super dealloc];
}

- (void)awakeFromNib	// too long!!!
{
	[modeControl retain];	// needed for toolbar item

	// the following is necessary under 10.4 due to an image-centering bug in IB
	{
		int i, count = [modeControl segmentCount];
		for (i = 0; i < count; ++i) {
			[modeControl setLabel:nil forSegment:i];
		}
	}
	//[drawerButton retain];
	
	[connectToLocalDaemonMenuItem setTarget:daemonController];
	[connectToLocalDaemonMenuItem setAction:@selector(connectToLocalDaemon:)];
	
	[connectToRemoteDaemonMenuItem setTarget:daemonController];
	[connectToRemoteDaemonMenuItem setAction:@selector(connectToRemoteDaemon:)];	
	
	// Notifications
	// ---------------------------------------------------------------
	// all objects have had awakeFromNib called
	//[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(activate:) name:NSApplicationDidFinishLaunchingNotification object:NULL];
	
	// disconnected notification...
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didDisconnect:) name:@"DaemonConnectionClosed" object:nil];	// object:[HDDaemonCommander sharedInstance]
	
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(noteCommandToBrowseDownloadHost:) name:@"PDownBrowseHost" object:downloadsController];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(noteCommandToBrowseUploadHost:) name:@"PUpBrowseHost" object:uploadsController];
		
	// ---------------------------------------------------------------

	// setting up the window
	// ---------------------------------------------------------------
	[self prepareMainTabView];
	//[drawer setContentView:globalSplit];
	//[drawer setDelegate:self];

	[self setCurrentView:PZAspectPaneNone];
	//[self prepareWindow];
	{
		[mainWindow setExcludedFromWindowsMenu:YES];
		[self prepareToolbar];
		[mainWindow setContentView:mainView];
		[drawer setContentView:[searchController drawerView]];
		[drawer setParentWindow:mainWindow];
	}
		
	// switch through all tab views, possible fix for strange brushed metal bug (moving background)
	// that's the reason why currentView was set to -1
	// so this isn't visible
	//[self switchToUploads:self];
	//[self switchToDownloads:self];
	//[self switchToSearch:self];
	[self setCurrentView:PZAspectPaneNone];
	
	[self switchToDaemon:self];

	//[self switchToolbarConfigurationTo:[self currentView]];
	
	//[mainWindow makeKeyAndOrderFront:self];	// window setup finished -> make it visible
	//[self setSavedDrawerSize];
	// ---------------------------------------------------------------
	
	// right now the timer just always fires, but really it should only start firing
	// after a HederaConnectedToCore notification and disable itself after DaemonConnectionClosed
	// i can fix that in a later release - jjt
	[[NSTimer scheduledTimerWithTimeInterval:1.0 target:self selector:@selector(updateTransferFieldTimer:) userInfo:NULL repeats:YES] retain];
		
	//[drawer open];
}

- (void)applicationDidFinishLaunching:(NSNotification *)notification {
	programHasFinishedLaunching = NO;

	[self setupDrawer];	// load the drawer's settings

	NSString *version;
	NSString *currentVersion;

	NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];

	/* first time launching Hedera? */
	BOOL firstRun = ([[userDefaults objectForKey:@"HDFirstRun"] boolValue] ? YES : NO);
		
	/*if (firstRun) {
		[self registerInitialDefaults];
	}*/
	
	/* version specific stuff */
	version = [userDefaults stringForKey:@"PVersion"];
	currentVersion = [[NSBundle bundleForClass:[self class]] objectForInfoDictionaryKey:(NSString *)kCFBundleVersionKey];
	
	if (!version) {
		version = currentVersion;
	}
							
	// check for existence of Application support
						
	/*
		This code handles all new installs or updates where it finds a Hedera folder. Since we decided not to track these
		folders by version we need to recommend to the user to delete them whenever there is a new release or if Hedera can
		not determine if Poisoned was used before and still sees a Poisoned Directory which can happen and causes problems
	*/
	
	/*
		seems like we can now make this a pre 0.515 issue. this may be needed for future versions of poisoned but for now we
		should only need to reset pre 0.515 systems
	*/
	
	NSComparisonResult versionOrder = [currentVersion compare:version options:NSNumericSearch];
	
	NSAlert *alert;
	if ((versionOrder == NSOrderedDescending) || firstRun) {		
		[userDefaults setObject:currentVersion forKey:@"PVersion"];

		if ([HDProcessLister isProcessRunning:HDDaemonName]) {
			alert = [NSAlert alertWithMessageText:HDLocalizedAlert(HDMainControllerGiFTDaemonIsRunningMessageTextKey)
				defaultButton:HDLocalizedOKButton
				alternateButton:nil
				otherButton:nil
				informativeTextWithFormat:HDLocalizedAlert(HDMainControllerGiFTDaemonMustBeShutDownBeforeUpgradingInformativeTextKey)];
				
			[alert setAlertStyle:NSCriticalAlertStyle];
			
			int returnCode = [alert runModal];
			
			if (returnCode == NSOKButton) {
				[self stopDaemon];
			}
			/*[alert beginSheetModalForWindow:mainWindow
				modalDelegate:self
				didEndSelector:@selector(prepareToStopDaemonAlertSheetDidEnd:returnCode:contextInfo:)
				contextInfo:NULL];*/
				
			[self completeApplicationLaunch:firstRun];
		}
		else {
			[self updateGiFT];
		}
	}
	else {
		[self completeApplicationLaunch:firstRun];
	}
}

- (void)completeApplicationLaunch:(BOOL)firstRun {
	//NSArray *paths = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory, NSUserDomainMask, YES);
	//NSString *basePath = ([paths count] > 0) ? [paths objectAtIndex:0] : NSTemporaryDirectory();
	//NSString *applicationSupportDirectory = [basePath stringByAppendingPathComponent:@"Hedera"];

	//if (version<=0.514f && [FileHandle fileExists : applicationSupportDirectory])
	//			if ([FileHandle moveFileToUserTrash : applicationSupportDirectory]) firstRun=YES;
	
	// checkfornodelist and plugin update on net
	if (firstRun) {
		[[NSUserDefaults standardUserDefaults] setBool:NO forKey:@"HDFirstRun"];
		[preferencesWindowController showWindow:self];
		//[self registerInitialDefaults];
		[daemonController checkConfFiles];
	}
	
	[self activate:self];
	
	programHasFinishedLaunching = YES;
}

/*- (void)fatalAlertSheetDidEnd:(NSAlert *)alert returnCode:(int)returnCode contextInfo:(void *)contextInfo {
	[[alert window] orderOut:self];
	[[NSApplication sharedApplication] terminate:self];
}*/

/*- (void)prepareToStopDaemonAlertSheetDidEnd:(NSAlert *)alert returnCode:(int)returnCode contextInfo:(void *)contextInfo {
	[[alert window] orderOut:self];
	int returnCode = [self stopDaemon];
	if (returnCode == NSOKButton) {
		[self stopDaemon];
	}
}*/

- (void)setupDrawer {
	NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];

	float drawerWidth = [userDefaults floatForKey:@"HDDrawerWidth"];
	if (drawerWidth > 0) {
		NSSize drawerSize = [drawer contentSize];
		drawerSize.width = drawerWidth;
		[drawer setContentSize:drawerSize];
	}

	BOOL drawerIsOpen = [userDefaults boolForKey:@"HDDrawerIsOpen"];
	if (drawerIsOpen) {
		[drawer open];
	}
}

- (int)stopDaemon {
	[self setCanceledDaemonShutdown:NO];

	[daemonShutdownProgressBar startAnimation:self];

	[NSThread detachNewThreadSelector:@selector(shutdownDaemon)
		toTarget:self withObject:nil];

	int returnCode = [NSApp runModalForWindow:daemonShutdownPanel];

	return returnCode;

	/*[[NSApplication sharedApplication] beginSheet:daemonShutdownPanel
		modalForWindow:mainWindow
		modalDelegate:self
		didEndSelector:@selector(stopDaemonSheetDidEnd:returnCode:contextInfo:)
		contextInfo:(void *)sender];*/
}

- (void)cancelStopDaemonPanel:(id)sender {	
	/*int returnCode;
	
	if (![HDProcessLister isProcessRunning:HDDaemonName]) {
		returnCode = HDMainControllerDaemonStatusZero;
	} else {
		returnCode = HDMainControllerDaemonStatusOne;
	}*/
	
	[self setCanceledDaemonShutdown:YES];
	[NSApp stopModal];

	//id sharedApp = [NSApplication sharedApplication];
	//id sheet = [[sharedApp mainWindow] attachedSheet];
	//[sharedApp endSheet:sheet];
	[daemonShutdownPanel orderOut:self];
}

- (void)finishedDaemonShutdown {
	[self setCanceledDaemonShutdown:NO];
	[daemonShutdownProgressBar stopAnimation:self];

	NSAlert *alert;
	
	if ([HDProcessLister isProcessRunning:HDDaemonName]) {
		alert = [NSAlert alertWithMessageText:HDLocalizedAlert(HDMainControllerSorryMustReinstallPoisonedMessageTextKey)
			defaultButton:HDLocalizedOKButton
			alternateButton:nil
			otherButton:nil
			informativeTextWithFormat:HDLocalizedAlert(HDMainControllerSorryMustReinstallPoisonedInformativeTextKey)];
	} else {
		alert = [NSAlert alertWithMessageText:HDLocalizedAlert(HDMainControllerReinstallPoisonedMessageTextKey)
			defaultButton:HDLocalizedOKButton
			alternateButton:nil
			otherButton:nil
			informativeTextWithFormat:HDLocalizedAlert(HDMainControllerReinstallPoisonedInformativeTextKey)];
	}
	
	[alert setAlertStyle:NSCriticalAlertStyle];
	
	(void)[alert runModal];
	
	[NSApp terminate:self];
	/*[alert beginSheetModalForWindow:mainWindow
		modalDelegate:self
		didEndSelector:@selector(fatalAlertSheetDidEnd:returnCode:contextInfo:)
		contextInfo:NULL];*/
}

- (void)shutdownDaemon {
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

	while ([HDProcessLister isProcessRunning:HDDaemonName] && ![self canceledDaemonShutdown]) {
		[FileHandle forceShutdown:HDDaemonName];
	}
	
	[NSApp performSelectorOnMainThread:@selector(abortModal) withObject:nil waitUntilDone:NO];
	[self performSelectorOnMainThread:@selector(cancelStopDaemonPanel:) withObject:nil waitUntilDone:NO];
	
	[pool release];
}

- (BOOL)canceledDaemonShutdown {
	@synchronized(self) {
		return AM_canceledDaemonShutdown;
	}
	return NO;
}

- (void)setCanceledDaemonShutdown:(BOOL)flag {
	@synchronized(self) {
		AM_canceledDaemonShutdown = flag;
	}
}

- (int)currentView
{
	return currentView;
}

- (void)setCurrentView:(int)anInt {
	currentView = anInt;
}

// DO is setup, so it's time to register some commands
// --------------------------------------------------------------------------------------------------
- (void)activate:(id)sender
{	
	[[HDDaemonCommander sharedInstance] registerController:self forCommands:
		[NSArray arrayWithObjects:
			@"POISON_GUI_UPDATE",
			@"REMOVE_UNKNOWN_COMMAND_TICKET",
			@"ADDSOURCE",
			@"DELSOURCE",
		nil]
	];
	[downloadsController registerWithCommander];
	[uploadsController registerWithCommander];
	[searchController registerWithCommander];
	[daemonController registerWithCommander];
	
	// currently being downloaded files (empty)
	[searchController setDownloadingHashes:[downloadsController hashes]];
}
// --------------------------------------------------------------------------------------------------


- (void)didDisconnect:(NSNotification *)aNotification
{
	[[NSApp delegate] validateToolbar];
	[transferField setStringValue:@"DL 0 kB/s   UL 0 kB/s"];
}

// --------------------------------------------------------------------------------------------------

- (void)updateTransferFieldTimer:(NSTimer *)inTimer
{
	if ([[HDDaemonCommander sharedInstance] isConnected])
	{
		int dl = [downloadsController downloadSpeed];
		int ul = [uploadsController uploadSpeed];
		[transferField setStringValue:[NSString stringWithFormat:@"DL %i kB/s   UL %i kB/s",dl,ul]];
	}
}

// Init window with content...
// --------------------------------------------------------------------------------------------------
/*- (void)prepareWindow
{
	/ *NSRect frame = [[NSScreen mainScreen] frame];
	//[drawer setParentWindow:nil];
	mainWindow = [[NSWindow alloc] 
		initWithContentRect:NSMakeRect(frame.origin.x+260,frame.size.height-560,735,500)
		styleMask:AQUAFIED
		backing:NSBackingStoreBuffered
		defer:YES];
	[mainWindow setMinSize:NSMakeSize(500,350)];
	[mainWindow setDelegate:self];
	[mainWindow setFrameUsingName:@"theMainWindow"];
	[mainWindow setReleasedWhenClosed:NO];* /
	

	[mainWindow setExcludedFromWindowsMenu:YES];

	[self prepareToolbar];

	[mainWindow setContentView:mainView];

	//if (currentView>-1) [mainWindow makeKeyAndOrderFront:self];
	//[drawer setParentWindow:mainWindow];
	[drawer setContentView:[searchController drawerView]];
	//[self setSavedDrawerSize];
}*/

- (void)prepareMainTabView
{
	NSTabViewItem *daemonTabViewItem = [[NSTabViewItem alloc] initWithIdentifier:@"daemon"];
	NSTabViewItem *searchTabViewItem = [[NSTabViewItem alloc] initWithIdentifier:@"search"];
	NSTabViewItem *downloadsTabViewItem = [[NSTabViewItem alloc] initWithIdentifier:@"download"];
	NSTabViewItem *uploadsTabViewItem = [[NSTabViewItem alloc] initWithIdentifier:@"upload"];
	
	[daemonTabViewItem setView:[daemonController view]];
	[searchTabViewItem setView:[searchController view]];
	[downloadsTabViewItem setView:[downloadsController view]];
	[uploadsTabViewItem setView:[uploadsController view]];
	
	[contentView addTabViewItem:daemonTabViewItem];
	[contentView addTabViewItem:searchTabViewItem];
	[contentView addTabViewItem:downloadsTabViewItem];
	[contentView addTabViewItem:uploadsTabViewItem];
	
	[contentView setDrawsBackground:NO];
	
	[daemonTabViewItem release];
	[searchTabViewItem release];
	[downloadsTabViewItem release];
	[uploadsTabViewItem release];
}

/*- (void)setSavedDrawerSize
{
	NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];

	int drawer_width = [userDefaults integerForKey:@"PDrawerWidth"];
	int drawer_state = [userDefaults integerForKey:@"PDrawerState"];
	/ *if (drawer_state) {
		NSSize d_size = [drawer contentSize];
		d_size.width=drawer_width;
		[drawer setContentSize:d_size];
		
		drawer_state--; // this is the actual value for the state since we increased it by 1 when saving
		[drawer toggle:self];
	}* /
}*/

// --------------------------------------------------------------------------------------------------

// check for host being accessable NOTE: this is what caused the beachball going on forever
// at startup for some users.
//
// For some reason this broke in 10.3.5 I am sure there will be an update to xcode to fix but
// for now it must be disabled

- (BOOL)isHostReachable:(NSString *)host
{
	BOOL isValid, result = NO; // change to NO when this is fixed
	SCNetworkConnectionFlags status;
	
	const char *nodename = [host cStringUsingEncoding:NSASCIIStringEncoding];	// [AM] my addition
	
	isValid = SCNetworkCheckReachabilityByName(nodename, &status);
	if (isValid && ((status & kSCNetworkFlagsReachable) && !(status & kSCNetworkFlagsConnectionRequired))) 
	{
		result = YES;
	}
	return result;
}

- (IBAction)drawerAction:(id)sender
{
	[drawer toggle:self];
}

/*- (IBAction)switchAppearance:(id)sender
{
	NSString *tab;
	NSString *winTitle;
	int state = [drawer state];
	if ([mainWindow attachedSheet])
		[[NSApplication sharedApplication] endSheet:[mainWindow attachedSheet]];
	//[drawer setParentWindow:nil];
	tab = [[contentView selectedTabViewItem] identifier];
	[contentView selectFirstTabViewItem:self];
	winTitle = [mainWindow title];
	[mainWindow saveFrameUsingName:@"theMainWindow"];
	[mainWindow orderOut:self];
	[mainWindow release];
	mainWindow = nil;
	[toolbar release];
	[self prepareWindow];
	[mainWindow setTitle:winTitle];
	[contentView selectTabViewItemWithIdentifier:tab];
	[[drawer contentView] display];
	[drawer setContentView:globalSplit];
	if (state == NSDrawerOpenState) [drawer open];

}*/

- (IBAction)newSearch:(id)sender
{
	if ([self currentView] != PZAspectPaneSearch) {
		[self switchToSearch:self];
	}
	
	[[searchController toolbarSearchField] selectText:sender];
}

- (void)noteCommandToBrowseDownloadHost:(NSNotification *)aNotification {
	[self switchToSearch:self];
	[searchController browseHost:[downloadsController hostForBrowsing]];
}

- (void)noteCommandToBrowseUploadHost:(NSNotification *)aNotification {
	[self switchToSearch:self];
	[searchController browseHost:[uploadsController hostForBrowsing]];
}

// ------------------------------------------------------------------------------
// Handling the mainWindow, delegates...
// ------------------------------------------------------------------------------

- (IBAction)giFTViewer:(id)sender; // "giFT Viewer cmd-N"
{
	[mainWindow makeKeyAndOrderFront:self];
}

/*- (BOOL)windowShouldClose:(id)sender
{
	[mainWindow saveFrameUsingName:@"theMainWindow"];
	return YES;
}*/

/*- (void)windowDidBecomeKey:(NSNotification *)aNotification
{
	// this is a workaround for the drawer becoming white when we open the window...
	if ([drawer state]==NSDrawerOpenState) {
		[drawer close];
		[drawer open];
	}
}*/

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
{
	NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
	if ([userDefaults boolForKey:@"HDDaemonStopGiFT"]&&[[HDDaemonCommander sharedInstance] isConnected]) {
		[[HDDaemonCommander sharedInstance] executeCommand:@"QUIT"];
	}
	
	// save the searches
	[searchController saveSearches];
	
	// saving the drawer's size and state
	[userDefaults setFloat:[drawer contentSize].width forKey:@"HDDrawerWidth"];
	BOOL drawerIsOpen = (NSDrawerOpenState == [drawer state]);
	[userDefaults setBool:drawerIsOpen forKey:@"HDDrawerIsOpen"];
	
	[NSApp setApplicationIconImage:[NSImage imageNamed: @"NSApplicationIcon"]];
	
	//[mainWindow saveFrameUsingName:@"theMainWindow"];
	[self saveToolbarConfiguration];
	return NSTerminateNow;
}

/*- (BOOL)applicationShouldHandleReopen:(NSApplication *)theApplication hasVisibleWindows:(BOOL)flag
{
	[mainWindow makeKeyAndOrderFront:self];
	return YES;
}*/

- (BOOL)validateToolbarItem:(NSToolbarItem *)theItem
{
	switch ([self currentView]) {
		case 2: return [downloadsController validateToolbarItem:theItem];
		case 3: return [uploadsController validateToolbarItem:theItem];
		default: return YES;
	}
}

/*- (void)drawerDidOpen:(NSNotification *)notification
{
	[drawerButton setState:NSOnState];
}


- (void)drawerDidClose:(NSNotification *)notification
{
	[drawerButton setState:NSOffState];
}*/

//- (void)registerInitialDefaults
//{	[drawer setContentSize:NSMakeSize(138,138)];
	
	//[userDefaults setBool:NO forKey:@"HDFirstRun"];

	//[userDefaults setBool:YES forKey:@"HDDaemonStopGiFT"];	// Automatically stop gift -> no
	//[userDefaults setObject:@"127.0.0.1" forKey:@"HDDaemonAddressUnused"];	// this isn't really used
	//[userDefaults setInteger:1213 forKey:@"HDDaemonPort"];
	//[userDefaults setObject:@"/usr/local/bin/giFT" forKey:@"HDDaemonGiFTPath"];
	//[userDefaults setInteger:3 forKey:@"HDDaemonConnectTimeout"];
	//[userDefaults setBool:YES forKey:@"HDDaemonAutoConnect"];
	//[userDefaults setBool:YES forKey:@"HDDaemonAutoLaunch"];
	//[userDefaults setBool:YES forKey:@"HDDaemonRelaunchAfterCrash"];

	//[userDefaults setBool:NO forKey:@"HDDaemonUseCustom"];

	//[userDefaults setBool:NO forKey:@"HDDownloadsRemoveCompleted"];
	//[userDefaults setBool:NO forKey:@"HDDownloadsRemoveCanceled"];
	//[userDefaults setBool:NO forKey:@"HDUnusedDeleteFile"];
	//[userDefaults setBool:NO forKey:@"HDDownloadsSwitchView"];

	//[userDefaults setBool:NO forKey:@"HDUploadsRemoveCompleted"];
	//[userDefaults setBool:NO forKey:@"HDUploadsRemoveCanceled"];
	
	//[userDefaults setBool:YES forKey: @"HDIntegrationAddSong"];
	//[userDefaults setBool:YES forKey: @"HDIntegrationAddSongToPlaylist"];
	//[userDefaults setObject:@"Hedera" forKey:@"HDIntegrationImportPlaylistName"];
//}

- (IBAction)resetProgram:(id)sender {
   [daemonController am_resetProgram];
}

#pragma mark -
#pragma mark Web Links


- (IBAction)visitHederaWebsite:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:HDHederaURL]];
}

- (IBAction)visitPoisonedWebsite:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:HDPoisonedURL]];
}

- (IBAction)visitGiFTWebsite:(id)sender {
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:HDGiFTURL]];
}

#pragma mark -
#pragma mark Connection Methods

- (oneway void)ADDSOURCE:(in NSArray *)data
{
	[downloadsController ADDSOURCE:data];
	[uploadsController ADDSOURCE:data];
}

- (oneway void)DELSOURCE:(in NSArray *)data
{
	[downloadsController DELSOURCE:data];
	[uploadsController DELSOURCE:data];
}

- (oneway void)POISON_GUI_UPDATE:(in id)sender
{
	// this is a very bad function, it was slowing down the entire app
	// by refreshing every item in the gui whenver any packet arrived
	// i have disabled it in an attempt to only update the parts of the
	// gui that need updating after a packet arrives - jjt
	/*
	int dl = [downloadsController downloadSpeed];
	int ul = [uploadsController uploadSpeed];
	[transferField setStringValue:[NSString stringWithFormat:@"DL %i kB/s   UL %i kB/s",dl,ul]];
	[daemonController gui_update:(currentView==0)];
	[searchController gui_update:(currentView==1)];
	[downloadsController gui_update:(currentView==2)];
	[uploadsController gui_update:(currentView==3)];
	*/
}

// unknown command with an id -> "mark" the id as used
// --------------------------------------------------------------------------------------------------
- (oneway void)REMOVE_UNKNOWN_COMMAND_TICKET:(in NSArray *)data
{
	if (data) {
		NSString *string = [data objectAtIndex:1];
		[[HDDaemonCommander sharedInstance] removeTicket:string];	// no need to test for string vs. nil, as removeTicket does that
	}
}

@end

#pragma mark -

@implementation HDMainController (CoverAccessors)

- (NSTextField *)statsTextField {
	return [[statsTextField retain] autorelease];
}

- (NSImageView *)statusImageView {
	return [[statusImageView retain] autorelease];
}

- (NSMenuItem *)localConnectMenuItem {
	return [[connectToLocalDaemonMenuItem retain] autorelease];
}

- (NSMenuItem *)remoteConnectMenuItem {
	return [[connectToRemoteDaemonMenuItem retain] autorelease];
}

@end

@implementation HDMainController (AspectMethods)

- (void)switchPane:(id)sender {
	int segment = [sender selectedSegment];	// modeControl
	
	switch(segment) {
		case 0: [self switchToDaemon:sender]; break;
		case 1: [self switchToSearch:sender]; break;
		case 2: [self switchToDownloads:sender]; break;
		case 3: [self switchToUploads:sender]; break;
		default: NSLog(@"Unknown navigation segment selected!"); break;
	}
}

- (IBAction)switchToDaemon:(id)sender
{
	if ([self currentView]!=PZAspectPaneDaemon) {
		//[self setCurrentView:PZAspectPaneDaemon];
		[self switchToolbarConfigurationTo:PZAspectPaneDaemon];
		//[contentView selectFirstTabViewItem:self];
		[contentView selectTabViewItemWithIdentifier:@"daemon"];
		[mainWindow setTitle:daemonWindowTitle];
		
		[modeControl setSelectedSegment:[self currentView]];
	}
}

- (IBAction)switchToSearch:(id)sender
{
	if ([self currentView]!=PZAspectPaneSearch) {
		//[self setCurrentView:PZAspectPaneSearch];
		[searchController gui_update:YES];
		[self switchToolbarConfigurationTo:PZAspectPaneSearch];
		//[contentView selectFirstTabViewItem:self];
		[contentView selectTabViewItemWithIdentifier:@"search"];
		[mainWindow setTitle:searchWindowTitle];
		
		[modeControl setSelectedSegment:[self currentView]];
	}
}

- (IBAction)switchToDownloads:(id)sender
{
	if ([self currentView]!=PZAspectPaneDownloads) {
		//[self setCurrentView:PZAspectPaneDownloads];
		[self switchToolbarConfigurationTo:PZAspectPaneDownloads];
		//[contentView selectFirstTabViewItem:self];
		[contentView selectTabViewItemWithIdentifier:@"download"];
		[mainWindow setTitle:downloadsWindowTitle];

		[modeControl setSelectedSegment:[self currentView]];
	}
}

- (IBAction)switchToUploads:(id)sender
{
	if ([self currentView]!=PZAspectPaneUploads) {
		//[self setCurrentView:PZAspectPaneUploads];
		[self switchToolbarConfigurationTo:PZAspectPaneUploads];
		//[contentView selectFirstTabViewItem:self];
		[contentView selectTabViewItemWithIdentifier:@"upload"];
		[mainWindow setTitle:uploadsWindowTitle];

		[modeControl setSelectedSegment:[self currentView]];
	}
}

@end

@implementation HDMainController (DaemonUpdater)

- (void)updateGiFT
{
	//check for the version of gift on server
	//check for version of gift on local
	
	NSAlert *alert = [NSAlert alertWithMessageText:HDLocalizedAlert(HDMainControllerWouldYouLikeToUpdateMessageTextKey)
		defaultButton:HDLocalizedYesButton
		alternateButton:HDLocalizedNoButton
		otherButton:nil
		informativeTextWithFormat:HDLocalizedAlert(HDMainControllerWouldYouLikeToUpdateInformativeTextKey)];
	
	[alert beginSheetModalForWindow:mainWindow
		modalDelegate:self
		didEndSelector:@selector(updateGiFTAlertSheetDidEnd:returnCode:contextInfo:)
		contextInfo:NULL];
}

- (void)updateGiFTAlertSheetDidEnd:(NSAlert *)alert returnCode:(int)returnCode contextInfo:(void *)contextInfo {
	[[alert window] orderOut:self];

	if (returnCode==NSOKButton)
	{			
		if ([HDProcessLister isProcessRunning:HDDaemonName])
		{
			NSAlert *alert = [NSAlert alertWithMessageText:HDLocalizedAlert(HDMainControllerGiFTDaemonStillRunningMessageTextKey)
				defaultButton:HDLocalizedOKButton
				alternateButton:nil
				otherButton:nil
				informativeTextWithFormat:HDLocalizedAlert(HDMainControllerMustStopGiFTToUpdateInformativeTextKey)];
				
			[alert setAlertStyle:NSCriticalAlertStyle];
			
			[alert beginSheetModalForWindow:mainWindow modalDelegate:nil didEndSelector:NULL contextInfo:NULL];

		} else {
			[self startUpdate:PZDownloadStatusZero];
		}
	} else {
		if (!programHasFinishedLaunching) {
			[self completeApplicationLaunch:YES];
		}
	}
}

- (IBAction)giftupdateCheck:(id)sender
{
	[self updateGiFT];
}

- (void)cancelDaemonUpdate:(id)sender {
	[self setDidCancelDaemonUpdate:YES];
}

- (BOOL)didCancelDaemonUpdate {
	@synchronized(self) {
		return AM_didCancelDaemonUpdate;
	}
	return NO;
}

- (void)setDidCancelDaemonUpdate:(BOOL)flag {
	@synchronized(self) {
		AM_didCancelDaemonUpdate = flag;
	}
}

- (void)startDownloadingURL:(id)theURL withFilePath:(id)filePath
{
	NSURLDownload *theDownload;
	
	NSURLRequest *theRequest=[NSURLRequest requestWithURL:[NSURL URLWithString:theURL] 
				cachePolicy:NSURLRequestUseProtocolCachePolicy timeoutInterval:60.0];
	
	theDownload=[[NSURLDownload alloc] initWithRequest:theRequest delegate:self];
	//[theDownload retain];	// [AM] why should this be needed? we *alloc and init* in the above line!
	if (theDownload) {
		[theDownload setDestination:filePath allowOverwrite:YES];
		NSLog(@"URL downloaded %@", theURL);
	} else {
		NSLog(@"File could not be downloaded");
	}
}

- (void)download:(NSURLDownload *)aDownload didFailWithError:(NSError *)error
{
	[aDownload release];

	[NSApp endSheet:progressWindow];
	[progressWindow orderOut:self];
	
	NSAlert *alert = [NSAlert alertWithMessageText:HDLocalizedAlert(HDMainControllerErrorDownloadingMessageTextKey)
		defaultButton:HDLocalizedOKButton
		alternateButton:nil
		otherButton:nil
		informativeTextWithFormat:[error localizedDescription], [[error userInfo] objectForKey:NSErrorFailingURLStringKey]];
	
	[alert setAlertStyle:NSCriticalAlertStyle];

	[alert beginSheetModalForWindow:mainWindow
		modalDelegate:self
		didEndSelector:@selector(downloadDidFailAlertSheetDidEnd:returnCode:contextInfo:)
		contextInfo:NULL];
}

- (void)downloadDidFailAlertSheetDidEnd:(NSAlert *)alert returnCode:(int)returnCode contextInfo:(void *)contextInfo {
	[[alert window] orderOut:self];
	if (![self didCancelDaemonUpdate]) {
		[self startUpdate:PZDownloadStatusOne];
	}
}

- (void)downloadDidFinish:(NSURLDownload *)aDownload
{
	NSLog(@"downloadDidFinish");
	
	[aDownload release];
	[progressBar stopAnimation:nil];
	
	//[NSApp endSheet:progressWindow];	// temp, for testing
	//[progressWindow orderOut:self];		// temp, for testing
	if (![self didCancelDaemonUpdate]) {
		[self startUpdate:PZDownloadStatusOne];	// temp disabled, for testing
	}
}

- (void)download:(NSURLDownload *)aDownload didCreateDestination:(NSString *)path
{
	NSLog(@"Final file destination: %@",path);
}

- (void)download:(NSURLDownload *)aDownload didReceiveResponse:(NSURLResponse *)response 
{
	[progressBar setMaxValue:[response expectedContentLength]]; 
} 

- (void)download:(NSURLDownload *)aDownload didReceiveDataOfLength:(unsigned)length 
{
	[progressBar incrementBy:length];
	[progressTitle2 setStringValue:[NSString stringWithFormat:@"Downloaded (%.0f kB)...", [progressBar doubleValue]/(float)HD_KILOBYTE]];
	
	if ([self didCancelDaemonUpdate]) {
		[aDownload cancel];
		[self startUpdate:PZDownloadStatusOne];	//PZDownloadStatusCancelled
	}
	
}

- (void)startUpdate:(int) status
{
	if ([self isHostReachable:PZKCEasyHostName]) {
		NSLog(@"startUpdate");
		if (status==PZDownloadStatusOne) {
			nextUpdate++;
		}
		else if (status==PZDownloadStatusZero) {
			[self setDidCancelDaemonUpdate:NO];
			nextUpdate=0;
		}
		
		NSString *filePathToUpdate;
		
		if ((nextUpdate < [updatesData count]) && ![self didCancelDaemonUpdate]) {
			NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
			NSDictionary *fileData = [updatesData objectAtIndex:nextUpdate];

			NSString *displayName = [fileData objectForKey:PZNetworkUpdateDisplayNameKey];
			
			[progressTitle setStringValue:[NSString stringWithFormat:@"Downloading %@ for %@...", displayName, [fileData objectForKey:PZNetworkUpdateNetworkNameKey]]];
			[progressTitle2 setStringValue:@"Downloaded (0 kB)..."];
			
			[progressBar setNeedsDisplay:YES];
			[progressBar startAnimation:nil];
			[progressBar setIndeterminate:NO];
			[progressBar setDoubleValue:0.0];
			
			if (status==PZDownloadStatusZero) {
				[NSApp beginSheet:progressWindow modalForWindow:mainWindow
					modalDelegate:self didEndSelector:NULL contextInfo:nil];	//@selector(giFTUpdatePanelDidEnd:returnCode:contextInfo:)
			}
				
			NSString *giftPath;
			
			if ([userDefaults boolForKey:@"HDDaemonUseCustom"]) {
				giftPath = [userDefaults stringForKey:@"HDDaemonGiFTPath"];
			}
			else {
				giftPath = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:@"giFT"];
			}
			
			if ([displayName isEqualToString:@"giFT"]) {	
				// [TODO] is this branch ever used? [AM]
				filePathToUpdate = [giftPath stringByAppendingPathComponent:[fileData objectForKey:PZNetworkUpdateLocalPathKey]];
			}
			else {
				NSString *basePath = HDApplicationSupportDirectory();
				filePathToUpdate = [basePath stringByAppendingPathComponent:[fileData objectForKey:PZNetworkUpdateLocalPathKey]];
			}
			NSString *downloadURL = [fileData objectForKey:PZNetworkUpdateDownloadURLKey];
			NSLog(@"Updating %@ to %@...", downloadURL, filePathToUpdate);
			
			if ([downloadURL isEqualToString:@"cgi"]) {
				// [TODO] is this branch ever used? [AM]
				NSBundle *bundle = [NSBundle mainBundle];
				NSDictionary *infoDict = [bundle infoDictionary];
				NSString *programName = [infoDict objectForKey:(NSString *)kCFBundleNameKey];		// we don't use objectForInfoDictionaryKey since we do not want localization
				NSString *versionString = [infoDict objectForKey:(NSString *)kCFBundleVersionKey];	// we don't use objectForInfoDictionaryKey since we do not want localization
				downloadURL = [NSString stringWithFormat:@"%@&app=%@&version=%@", downloadURL, [programName lowercaseString], versionString];
				[self startDownloadingURL:downloadURL withFilePath:filePathToUpdate];
			}
			else {
				[self startDownloadingURL:downloadURL withFilePath:filePathToUpdate];
			}
			
		} 
		else {
			[NSApp endSheet:progressWindow];
			[progressWindow orderOut:self];	// [progressWindow close];
	
			if (!programHasFinishedLaunching) {
				[self completeApplicationLaunch:YES];
			}
		}
	}
	else {
		NSAlert *alert = [NSAlert alertWithMessageText:HDLocalizedAlert(HDDaemonUpdateHostUnreachableMessageTextKey)
			defaultButton:HDLocalizedYesButton
			alternateButton:HDLocalizedNoButton
			otherButton:nil
			informativeTextWithFormat:HDLocalizedAlert(HDDaemonUpdateHostUnreachableInformativeTextKey), PZKCEasyHostName];
		
		[alert beginSheetModalForWindow:mainWindow
			modalDelegate:self
			didEndSelector:@selector(updateGiFTAlertSheetDidEnd:returnCode:contextInfo:)
			contextInfo:NULL];
		
		NSLog(@"Host \"%@\" unreachable.", PZKCEasyHostName);
	}
}

@end