// HDMusicIntegrationController.m
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------


#import "HDMusicIntegrationController.h"
#import "HDAppleScriptSongPlayer.h"

#import "AMShellWrapper.h"

NSString *PZiTunesControllerOSAScriptPath = @"/usr/bin/osascript";

NSString *PZiTunesControllerScript1Filename = @"add-song-to-library";
NSString *PZiTunesControllerScript1FileExtension = @"applescript";

NSString *PZiTunesControllerScript2Filename = @"add-song-to-playlist";
NSString *PZiTunesControllerScript2FileExtension = @"applescript";



@implementation HDMusicIntegrationController

- (id)init {
	if (nil != (self = [super init])) {
		[self setShellWrapper:nil];
		[self setFilePath:nil];
		
		[self setAppleScriptSongPlayer:[[[HDAppleScriptSongPlayer alloc] init] autorelease]];
	}
	return self;
}

- (void)dealloc {
	[self setShellWrapper:nil];
	[self setFilePath:nil];
	[self setAppleScriptSongPlayer:nil];
	
	[super dealloc];
}

- (void)handleCompletedFile:(NSString *)filePath {
    if ([[NSUserDefaults standardUserDefaults] boolForKey:@"HDIntegrationAddSong"]) {
		[self importSong:[filePath stringByExpandingTildeInPath]];
	}
}

- (void)importSong:(NSString *)filePath {
	if ([[NSFileManager defaultManager] fileExistsAtPath:filePath]) {
		NSString *scriptFile;
		NSString *scriptExt;
		NSString *playlistName;
		NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
		
		if ([userDefaults boolForKey:@"HDIntegrationAddSongToPlaylist"]) {
			scriptFile = PZiTunesControllerScript2Filename;
			scriptExt = PZiTunesControllerScript2FileExtension;
			playlistName = [userDefaults stringForKey:@"HDIntegrationImportPlaylistName"];
		}
		else {
			scriptFile = PZiTunesControllerScript1Filename;
			scriptExt = PZiTunesControllerScript1FileExtension;
			playlistName = @"";
		}

		NSString *scriptPath = [[NSBundle mainBundle] pathForResource:scriptFile ofType:scriptExt];

		NSArray *args = [NSArray arrayWithObjects:
			PZiTunesControllerOSAScriptPath,
			scriptPath,
			filePath,
			playlistName,
			nil];

		AMShellWrapper *shellWrapper = [[AMShellWrapper alloc] initWithController:self
			inputPipe:nil
			outputPipe:nil
			errorPipe:nil
			workingDirectory:@"."
			environment:nil
			arguments:args];
		
		[self setShellWrapper:shellWrapper];
		[shellWrapper release];

		@try {
			if (shellWrapper) {
				[shellWrapper setOutputStringEncoding:NSUTF8StringEncoding];
				[shellWrapper startProcess];
			}/* else {
				NSLog(@"darn, something went wrong!");
				//[self write:@"Ops! Something went wrong.\r Was not able to execute command.\r"];
			}*/
		}
		@catch (NSException *e) {
			NSLog(@"Caught %@: %@", [e name], [e reason]);
			//[self processLaunchException:e];
		}
	}
}

#pragma mark -
#pragma mark Accessors

- (AMShellWrapper *)shellWrapper {
	return [[AM_shellWrapper retain] autorelease];
}

- (void)setShellWrapper:(AMShellWrapper *)aShellWrapper {
	if (aShellWrapper != AM_shellWrapper) {
		[AM_shellWrapper release];
		AM_shellWrapper = [aShellWrapper retain];
	}
}

- (NSString *)filePath {
	return [[AM_filePath retain] autorelease];
}

- (void)setFilePath:(NSString *)aString {
	if (aString != AM_filePath) {
		[AM_filePath release];
		AM_filePath = [aString copy];
	}
}

- (id)appleScriptSongPlayer {
	return [[AM_appleScriptSongPlayer retain] autorelease];
}

- (void)setAppleScriptSongPlayer:(id)anObject {
	if (anObject != AM_appleScriptSongPlayer) {
		[AM_appleScriptSongPlayer release];
		AM_appleScriptSongPlayer = [anObject retain];
	}
}

#pragma mark -
#pragma mark AMShellWrapperController Protocol

// implement this protocol to control your AMShellWrapper object:

- (void)appendOutput:(NSString *)output {
	// do nothing
}

// output from stdout

- (void)appendError:(NSString *)error {
	NSLog(@"An error occurred: %@", error);
}

// output from stderr

- (void)processStarted:(id)sender {
	// do nothing
}
// This method is a callback which your controller can use to do other initialization
// when a process is launched.

- (void)processFinished:(id)sender withTerminationStatus:(int)resultCode {
	id player = [self appleScriptSongPlayer];

	int playMode = [[NSUserDefaults standardUserDefaults] boolForKey:@"HDIntegrationPlaySongAfterImport"];
	switch (playMode) {
		case 0:
			break;
			
		case 1:
			[player setFilePath:[self filePath]];
			[player playSong:YES];
			break;
			
		case 2:
			[player setFilePath:[self filePath]];
			[player playSong:NO];
			break;
	}
	
	[self setShellWrapper:nil];
}
// This method is a callback which your controller can use to do other cleanup
// when a process is halted.

// AMShellWrapper posts a AMShellWrapperProcessFinishedNotification when a process finished.
// The userInfo of the notification contains the corresponding NSTask ((NSTask *), key @"task")
// and the result code ((NSNumber *), key @"resultCode")
// ! notification removed since it prevented the task from getting deallocated

//#pragma mark -

/*- (void)processLaunchException:(NSException *)exception
{
	NSLog(@"-processLaunchException: called...");
}*/

@end
