//
// HDSearchFilterController.m
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import "HDSearchFilterController.h"
#import "SMDoubleSlider.h"
#import "HDFilterResultsSplitView.h"
#import "HDFilterProtocolListEntry.h"
#import "HDAdditions.h"

#import "HDValueTransformers.h"

@implementation HDSearchFilterController

+ (void)initialize {
	NSValueTransformer *vt;
	
	vt = [[PZSearchFilterStateValueTransformer alloc] init];
	[NSValueTransformer setValueTransformer:vt forName:@"SearchFilterStateValueTransformer"];
	[vt release];
}

- (id)init {
	if (nil != (self = [super init])) {
		[self setAvailableProtocols:nil];
		[self setSearchDataSource:nil];
		[self setKeywordFieldToolTip:NSLocalizedString(@"Keyword filter", @"Keyword filter")];
		[self setFilterButtonToolTip:NSLocalizedString(@"Enable/disable filtering", @"Enable/disable filtering")];
		
		/*[[NSNotificationCenter defaultCenter] addObserver:self
			selector:@selector(setupSplitView:)
			name:NSApplicationDidFinishLaunchingNotification
			object:nil];*/
	}
	return self;
}

- (void)dealloc {
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[self setAvailableProtocols:nil];
	[self setSearchDataSource:nil];
	
	[super dealloc];
}

- (void)awakeFromNib {
	{	
		[_sm_horzSlider setMinValue:0.0];
		[_sm_horzSlider setMaxValue:25.0];

		[_sm_horzSlider setDoubleLoValue:0.0];
		[_sm_horzSlider setDoubleHiValue:25.0];
		
		[_sm_horzSlider setTarget:self];
		[_sm_horzSlider setAction:@selector(changeFilterSlider:)];
	}
	
	minimumHeight = [view frame].size.height;		// [AM]
	maximumHeight = [filterView frame].size.height;	// [AM]	

	[filterSplitView setSplitterPosition:0 animate:YES];

	[keywordField setDelegate:self];
	
	[protoTable setIntercellSpacing:NSMakeSize(0,0)];	// do we need this?
	//[self setupSplitView];	

	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(noteAvailableProtocols:) name:@"PStatsProtocolsAvailable" object:nil];

	/*NSButtonCell *check = [[[NSButtonCell alloc] init] autorelease];
	[check setButtonType:NSSwitchButton];
	[check setControlSize:NSSmallControlSize];
	//[protoTable setTarget:self];
	//[protoTable setAction:@selector(changeFilterProtocols:)];
	[check setFont:[NSFont systemFontOfSize:10]];
	[check setContinuous:YES];
	[[protoTable tableColumnWithIdentifier:@"proto"] setDataCell:check];*/
	//[keywordField setToolTip:@"Keyword filter"];
}

/*- (void)setupSplitView:(NSNotification *)aNotification {
	//[filterSplitView setSplitterPosition:0 animate:YES];	//[filterSplitView setSplitterPosition:0 animate:YES];
	[filterSplitView setSplitterPosition:minimumHeight animate:YES];	//[filterSplitView setSplitterPosition:0 animate:YES];
}*/


- (void)disconnected {
	if ([filterDisclosureTriangle state]==NSOnState) {
		[filterDisclosureTriangle setState:NSOffState];
		[self toggleFilterDisclosureTriangle:nil];
	}
}


- (HDSearchResultSource *)searchDataSource {
	return [[datasource retain] autorelease];
}

- (void)setSearchDataSource:(HDSearchResultSource *)aResultSource
{
	if (aResultSource != datasource) {
		[datasource release];
		datasource = [aResultSource retain];
	
		[self prepareValues];
		[protoTable reloadData];
	}
}

- (void)prepareValues {
	if ([self searchDataSource]==nil) {
		//[de_activate setState:NSOffState];
		[keywordField setEnabled:NO];
		[_sm_horzSlider setEnabled:NO]; 
		//[info setStringValue:NSLocalizedString(@"no search selected", @"no search selected")];
		
		[keywordField setStringValue:@""];
		[_sm_horzSlider setDoubleLoValue:[_sm_horzSlider minValue]]; // dub
		[_sm_horzSlider setDoubleHiValue:[_sm_horzSlider maxValue]]; // dub
		[minText setTextColor:[NSColor grayColor]];
		[maxText setTextColor:[NSColor grayColor]];
	}
	else {
		if ([[self searchDataSource] filterProtocols]==nil) [[self searchDataSource] setFilterProtocols:[self availableProtocols]];
		[keywordField setEnabled:YES];
		[_sm_horzSlider setEnabled:YES];
		[minText setTextColor:[NSColor blackColor]]; // dub
		[maxText setTextColor:[NSColor blackColor]]; // dub
		[_sm_horzSlider setDoubleLoValue:[[self searchDataSource] minSizeFilter]];
		[_sm_horzSlider setDoubleHiValue:[[self searchDataSource] maxSizeFilter]];
		NSString *keyword = [[self searchDataSource] keywordFilter];
		if (!keyword) {
			keyword = @"";
		}
		[keywordField setStringValue:keyword];
	}
}

- (void)noteAvailableProtocols:(NSNotification *)aNotification
{
	id newProtocols = [[aNotification userInfo] objectForKey:@"protos"];
	[self setAvailableProtocols:newProtocols];

	{
		[filterProtocolListArrayController removeAllArrangedObjects:self];

		NSEnumerator *e = [[self availableProtocols] objectEnumerator];
		id proto;
		
		while (proto = [e nextObject]) {
			HDFilterProtocolListEntry *entry = [[HDFilterProtocolListEntry alloc] init];
			[entry setTitleString:proto];
			[filterProtocolListArrayController addObject:entry];
			[entry release];
		}
	}

	[filterSplitView setSplitterPosition:minimumHeight animate:YES];	//[filterSplitView setSplitterPosition:0 animate:YES];

	//[self changeSplitterPositionToMinimum];
}

- (IBAction)toggleFilterDisclosureTriangle:(id)sender {
	float newSplitterPosition = 0;

	if ([filterDisclosureTriangle state] == NSOffState) {
		newSplitterPosition = minimumHeight;
	}
	else if ([filterDisclosureTriangle state] == NSOnState) {
		newSplitterPosition = maximumHeight;
	}

	[filterSplitView setSplitterPosition:newSplitterPosition animate:YES];
}

- (void)activate
{
	//[de_activate setState:NSOnState];
	id searchDataSource = [self searchDataSource];
	if (![searchDataSource isFiltered]) {
		[searchDataSource setFiltered:![searchDataSource isFiltered]];

		[self changeFilterState:nil];
	}
}

- (IBAction)changeFilterState:(id)sender
{
	if (nil!=[self searchDataSource]) {
		[[NSNotificationCenter defaultCenter] postNotificationName:@"PFilterDeactivated" object:self userInfo:nil];
	}
}

- (IBAction)changeFilterKeyword:(id)sender
{
	[self activate];
	[[self searchDataSource] setKeywordFilter:[keywordField stringValue]];
	[[NSNotificationCenter defaultCenter] postNotificationName:@"PFilterDeactivated" object:self userInfo:nil];
}

- (IBAction)changeFilterSlider:(id)sender
{
	[self activate];
	[[self searchDataSource] setMinSizeFilter:[sender doubleLoValue]];
	[[self searchDataSource] setMaxSizeFilter:[sender doubleHiValue]];
	[[NSNotificationCenter defaultCenter] postNotificationName:@"PFilterDeactivated" object:self userInfo:nil];
}

- (IBAction)changeFilterProtocols:(id)sender
{
	if ([self searchDataSource]) {
		[self activate];
		int row = [sender clickedRow];
		NSMutableArray *_protos = [[self searchDataSource] filterProtocols];
		NSString *_item = [[self availableProtocols] objectAtIndex:row];
		if ([_protos containsObject:_item]) [_protos removeObject:_item];
		else [_protos addObject:_item];
		[protoTable reloadData];
		[[self searchDataSource] filterProtos];
		[[NSNotificationCenter defaultCenter] postNotificationName:@"PFilterDeactivated" object:self userInfo:nil];
	}
}


// ------------------------------------------------------------
// DataSource for the protocol table...
/*- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
	return [protos count];
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
	return [protos objectAtIndex:rowIndex];
}*/

// delegate method to ensure that rows do not remain selected once checkbox-button is clicked
- (void)tableViewSelectionDidChange:(NSNotification *)aNotification {
	if ([aNotification object] == protoTable) {
		(void)[filterProtocolListArrayController setSelectedObjects:nil];
	}
}

/*
// this doesn't work if we want checkbox to activate, so we use the above method instead
// Delegate for the protocol table...
- (BOOL)tableView:(NSTableView *)aTableView shouldSelectRow:(int)rowIndex
{
	return YES;
}*/

- (void)tableView:(NSTableView *)aTableView willDisplayCell:(id)aCell forTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
	NSString *identifier = [aTableColumn identifier];

	if ([identifier isEqualToString:@"protocolEnabled"]) {
		if ([self searchDataSource]) {
			[aCell setEnabled:YES];
			if ([[[self searchDataSource] filterProtocols] containsObject:[[self availableProtocols] objectAtIndex:rowIndex]]) {
				[aCell setState:NSOnState];
			}
			else {
				[aCell setState:NSOffState];
			}
		}
		else {
			[aCell setEnabled:NO];
			[aCell setState:NSOnState];
		}
		//[aCell setHighlighted:NO];
	}
}

// ------------------------------------------------------------


// ------------------------------------------------------------
// keywordField delegate
- (void)controlTextDidChange:(NSNotification *)aNotification
{
	[self changeFilterKeyword:nil];
}
// ------------------------------------------------------------

#pragma mark -
#pragma mark Split View Methods

- (float)splitView:(NSSplitView *)sender constrainMinCoordinate:(float)proposedMin ofSubviewAt:(int)offset {
	float coordinate = proposedMin;
	if (offset == 0) {
		coordinate = minimumHeight;
	}
	return coordinate;
}

- (float)splitView:(NSSplitView *)sender constrainMaxCoordinate:(float)proposedMax ofSubviewAt:(int)offset {
	float coordinate = proposedMax;
	if (offset == 0) {
		coordinate = maximumHeight;
	}
	return coordinate;
}

- (float)splitView:(NSSplitView *)sender constrainSplitPosition:(float)proposedPosition ofSubviewAt:(int)offset {
	float coordinate = proposedPosition;
	if (offset == 0) {
		float halfway = (minimumHeight + maximumHeight) / 2.0;
		if (proposedPosition > halfway) {
			coordinate = maximumHeight;
		} else if (proposedPosition < halfway) {
			coordinate = minimumHeight;
		} else {
			coordinate = maximumHeight;	// could also be minimumHeight
		}
	}
	return coordinate;
}

@end

#pragma mark -

@implementation HDSearchFilterController (Accessors)

- (NSString *)keywordFieldToolTip {
	return [[AM_keywordFieldToolTip retain] autorelease];
}

- (void)setKeywordFieldToolTip:(NSString *)aString {
	if (aString != AM_keywordFieldToolTip) {
		[AM_keywordFieldToolTip release];
		AM_keywordFieldToolTip = [aString copy];
	}
}

- (NSString *)filterButtonToolTip {
	return [[AM_filterButtonToolTip retain] autorelease];
}

- (void)setFilterButtonToolTip:(NSString *)aString {
	if (aString != AM_filterButtonToolTip) {
		[AM_filterButtonToolTip release];
		AM_filterButtonToolTip = [aString copy];
	}
}

- (NSMutableArray *)availableProtocols {
	return [[AM_availableProtocols retain] autorelease];
}

- (void)setAvailableProtocols:(NSArray *)anArray {
	if (anArray != AM_availableProtocols) {
		[AM_availableProtocols release];
		AM_availableProtocols = [anArray mutableCopy];
	}
}

@end
