//
//  HDDaemonCommandParser.m
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import "HDDaemonCommandParser.h"
#import "HDValueTransformers.h"
#import <string.h>
//#import "libgift/libgift.h"
//#import "GPacket.h"

@implementation HDDaemonCommandParser

+ (void)initialize {
	NSValueTransformer *vt;
	
	vt = [[PZSizeValueTransformer alloc] init];
	[NSValueTransformer setValueTransformer:vt forName:@"SizeValueTransformer"];
	[vt release];
	
	vt = [[PZBitrateValueTransformer alloc] init];
	[NSValueTransformer setValueTransformer:vt forName:@"BitrateValueTransformer"];
	[vt release];

	vt = [[PZDurationValueTransformer alloc] init];
	[NSValueTransformer setValueTransformer:vt forName:@"DurationValueTransformer"];
	[vt release];
}

- (id)init
{
	if (self=[super init])
	{
				// commented out for release
		//libgift_init("Poisoned", GLOG_STDERR, NULL);
		buffer=nil;
		dispatcher = [[HDDaemonCommandDispatcher alloc] init];
	}
	return self;
}

- (void)dealloc
{
	// commented out for release
	//libgift_finish();
		
	//[buffer release];
	[dispatcher release];
	[super dealloc];
}

// REGISTER FOR DISPATCHER...
- (void)registerController:(id)controller forCommands:(NSArray *)cmd
{
	[dispatcher registerController:controller forCommands:cmd];
}

// GETTING THE DATA
- (void)processOutput:(const char *)data
{
		// we use a custom autoreleasepool
		// to make sure data we don't need anymore gets released immediatly
		// hopefully this helps - rizzi
		pool = [[NSAutoreleasePool alloc] init];
		
		if (data)
	{
		[dispatcher processOutput:[self parse:data]];
		
				// // commented out for release
		// TEST CODE - jjt
		/*Interface *interface = interface_unserialize((char *)data, strlen(data));
		if (interface)
		{
			GPacket *pkt = [GPacket packetWithInterface:interface];
			NSLog(@"GPacket command: %@", [pkt command]);
		}
		else
			NSLog(@"interface parse error");*/
	}
		
		[pool release];
}

- (NSArray *)parse:(const char *)data
{
	NSMutableArray *result = [NSMutableArray array];
	int bufferlen=0;
	if (buffer) {
		bufferlen=[buffer lengthOfBytesUsingEncoding:NSUTF8StringEncoding];
	}
	char *parsing = NSZoneMalloc([self zone], (strlen(data)+bufferlen)*sizeof(char *));	// [AM] changed from sizeof(char *)
	if (bufferlen) {
		strcpy(parsing,[buffer UTF8String]);
		strcat(parsing,data);
	} else {
		strcpy(parsing,data);
	}
	if (buffer) {
		[buffer release];
		buffer=nil;
	}
	
	int len=strlen(parsing);
	int loc=0;
	
	parseindex=0;
	NSArray *parsed;
	globalfinished=NO;
	do {
		cmdfinished = NO;
		loc += parseindex;
		parseindex=0;
		parsed = [self parseCommand:&parsing[loc] withLength:strlen(&parsing[loc]) sub:NO];
		if (parsed) {
			[result addObject:parsed];
		}
	} while (cmdfinished && loc<len);
	if (!globalfinished && !parsed && loc<len) {
		buffer = [[NSString alloc] initWithCString:&parsing[loc]];
	}
	NSZoneFree([self zone], parsing);
	return result;
}

- (NSMutableArray *)parseCommand:(const char *)data withLength:(int)datalen sub:(BOOL)subComm
{
	int skip;
	char arg[4096];
	char key[128];
	NSString *keystring;
	NSString *argstring;
	int keyindex=0;
	NSMutableArray *sub;
	
	NSMutableArray *parsed = [[[NSMutableArray alloc] initWithObjects:
		@"",					// COMMAND
		@"",					// COMMAND ARGS
		[NSMutableDictionary dictionary],	// key->args
		[NSMutableArray array],			// SUBCOMMANDS
		nil] autorelease];
		   
	BOOL matched;
	BOOL finished=NO;
	BOOL cmd=YES;
	int test;
	while (!finished && parseindex<datalen) {
		test=parseindex;
		// skip whitespace
		arg[0] = 0;
		key[0] = 0;
		skip = strspn(&data[parseindex],"\r\n ");
		if (skip>0) parseindex+=skip;
		keyindex=parseindex; // set keyindex to the beginning of the command, in case this is a subcommand...
		
		if (data[parseindex]==';') {
				finished = YES;
				cmdfinished=YES; // set global variable to true, since finished command parsing
				globalfinished=YES;
				parseindex++;
				skip = strspn(&data[parseindex],"\r\n ");
				if (skip>0) parseindex+=skip;
				return parsed;
		}
		else if (data[parseindex]=='}') {
				finished = YES;
				parseindex++;
				skip = strspn(&data[parseindex],"\r\n ");
				if (skip>0) parseindex+=skip;
				return parsed;
		}
		skip=0;
		parseindex--;
		while(skip<=0) {
			parseindex++;
			skip = strspn(&data[parseindex],"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_0123456789");
		   // if (skip<=0) parseindex++;
			if (parseindex>=datalen) return nil;
		}
		strncpy(key,&data[parseindex],skip);
		key[skip]=0;
		parseindex+=skip;
		
		char current;
		int i;
		for (i=0;i<2;i++) {
		
			// skip whitespace
			if (subComm&&cmd) {
				skip = strspn(&data[parseindex],"\r\n {");
				if (skip>0) parseindex+=skip;
			}
			else {
				skip = strspn(&data[parseindex],"\r\n ");
				if (skip>0) parseindex+=skip;
			}
			// --------------
			
			current = 0;
			current = data[parseindex];

			matched=NO;
			
			// look for argument
			if (current=='(') {
				parseindex++;
				int ai=0;
				while (!matched && parseindex<datalen) {
					switch (data[parseindex]) {
						case '\\':	parseindex++;
										if (parseindex<datalen) arg[ai++]=data[parseindex++];
										break;
						case ')':	parseindex++;
										matched=YES;
										break;
						default:	arg[ai++]=data[parseindex++];
										break;
					}
				}
				arg[ai]=0;
			}
			// look for subcommand
			else if (current=='{') {
				parseindex=keyindex;
				sub = nil;
				sub = [self parseCommand:data withLength:datalen sub:YES];
				if (sub) [[parsed objectAtIndex:3] addObject:sub];
				else return nil;
			}
			if (parseindex>=datalen) i=10;
			// save parsed key/arg to array...
			if (cmd && key[0]) {
				cmd=NO;
				[parsed replaceObjectAtIndex:0 withObject:[NSString stringWithCString:key]];
				if (arg[0]) [parsed replaceObjectAtIndex:1 withObject:[NSString stringWithCString:arg]];
			}
			/*else if (key[0] && arg[0]) {
				[[parsed objectAtIndex:2] setObject:[NSString stringWithCString:arg] forKey:[NSString stringWithCString:key]];
			}*/
			else {
				if (key[0] && arg[0]) {
					keystring = [NSString stringWithCString:key];
					//argstring = [NSString stringWithCString:arg];
					
					// when using NSISOLatin1StringEncoding also umlauts etc. get displayed correctly in the results
					// it still could be the wrong encoding, but it seems to work best so far
					// this means no problems for me so far - rizzi
					argstring = [[[NSString alloc] 
						initWithData:[NSData dataWithBytes:arg length:strlen(arg)]
						encoding:NSISOLatin1StringEncoding
					] autorelease];
					
				   [[parsed objectAtIndex:2] setObject:argstring forKey:keystring];
					if ([keystring isEqualToString:@"size"]) {
						NSValueTransformer *vt = [NSValueTransformer valueTransformerForName:@"SizeValueTransformer"];
						[[parsed objectAtIndex:2] setObject:[vt transformedValue:argstring] forKey:@"calcsize"];
					} else if ([keystring isEqualToString:@"file"])
						[[parsed objectAtIndex:2] setObject:[argstring lastPathComponent] forKey:@"file"];
					else if ([keystring isEqualToString:@"bitrate"]) {
						NSValueTransformer *vt = [NSValueTransformer valueTransformerForName:@"BitrateValueTransformer"];
						[[parsed objectAtIndex :2] setObject:[vt transformedValue:argstring] forKey:@"bitrate"];
						/*int bitrate = [argstring intValue];
						NSNumber *number = [NSNumber numberWithInt:bitrate];// [AM]
						[[parsed objectAtIndex:2] setObject:number forKey:@"bitrate"];	// [AM]
						if (bitrate<10000)
							[[parsed objectAtIndex:2] 
								setObject:[NSString stringWithFormat:@"%i kbps",bitrate]
								forKey:@"bitrate"];
						else if (bitrate<999999)
							[[parsed objectAtIndex:2] 
								setObject:[NSString stringWithFormat:@"%i kbps",bitrate/1000]
								forKey:@"bitrate"];
						else [[parsed objectAtIndex:2] setObject:@"" forKey:@"bitrate"];*/
					}
					/* parse duration */
					else if ([keystring isEqualToString:@"duration"]) 
					{
						// we store the formatted duration as a new object in the dict.
						// "duration" itself is used for sorting
						// -rizzi
						NSValueTransformer *vt = [NSValueTransformer valueTransformerForName:@"DurationValueTransformer"];
						[[parsed objectAtIndex:2] setObject:[vt transformedValue:argstring] forKey:@"formattedDuration"];
						//int duration = [argstring intValue];
						//NSString *duration2 = [NSString stringWithFormat:[self getDecimal:duration]];
						//NSString *duration2 = [NSString stringWithFormat:@"%i",duration];
						//NSString *duration3 = [self getDecimal:duration2];
				
						//[[parsed objectAtIndex:2] setObject:[NSString stringWithFormat:@"%@",duration3]forKey:@"duration"];
						
						//NSDateFormatter *df = [[NSDateFormatter alloc] initWithDateFormat:@"%" allowNaturalLanguage:NO];
						
					}
				}
			}
			keystring=nil;
			argstring=nil;

			key[0]=0;
			arg[0]=0;
			// ---------------------------------
			
			skip = strspn(&data[parseindex],"\r\n ");
			if (skip>0) parseindex+=skip;
		}
		if (test==parseindex) { // this should not happen (but it does from time to time ;)
			parseindex++;
		}
	}
	return nil;
}

/*- (NSString *)calcSize:(NSString *)size
{
	if (!size) return @"";
	long long s;
	[[NSScanner scannerWithString:size] scanLongLong:&s];
	if (s<1024) return [NSString stringWithFormat:@"%i B",(int)s];
	else if (s<1048576) return [NSString stringWithFormat:@"%i kB",(int)(s/1024)];
	else if (s<1073741824) return [NSString stringWithFormat:@"%.2f MB",(s/1048576.0)];
	else return [NSString stringWithFormat:@"%.2f GB",(s/1073741824.0)];
}*/

/*- (NSString *)getDecimal:(NSString *)value
{
	NSString *returnValue = @"";

	if (value) {
		long long size;
		[[NSScanner scannerWithString:value] scanLongLong:&size];
		//if (s>6000) return @"0";
		if (size<60)
			returnValue = [NSString stringWithFormat:@"%i Secs",(int)size];
		else if (size<3600)
			returnValue = [NSString stringWithFormat:@"%.2f Mins",(size/60.00)];
		else
			returnValue = [NSString stringWithFormat:@"%.2f Hrs",(size/3600.00)];
		//else return [NSString stringWithFormat:@"%.2f Days",(s/14400.0)];if (s<14400)
	}
	
	return returnValue;
}*/

@end
