//
//  HDSearchResultSource.m
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import "HDSearchResultSource.h"
#import "HDAspectControllerSearch.h"
#import "HDAdditions.h"

#import "HDSearchResultSource+Sorting.h"
#import "PIconShop.h"

NSString *HDSearchResultSourceURLKey = @"url";
NSString *HDSearchResultSourceFileKey = @"file";
NSString *HDSearchResultSourceHashKey = @"hash";
NSString *HDSearchResultSourceIconKey = @"icon";
NSString *HDSearchResultSourceSizeKey = @"size";
NSString *HDSearchResultSourceUserKey = @"user";
NSString *HDSearchResultSourceArrayKey = @"array";
NSString *HDSearchResultSourceExpandableKey = @"expandable";
NSString *HDSearchResultSourceAvailabilityKey = @"availability";

NSString *HDSearchResultSourceAlbumKey = @"album";
NSString *HDSearchResultSourceTitleKey = @"title";
NSString *HDSearchResultSourceArtistKey = @"artist";
NSString *HDSearchResultSourceBitrateKey = @"bitrate";
NSString *HDSearchResultSourceDurationKey = @"duration";
NSString *HDSearchResultSourceFormattedDurationKey = @"formattedDuration";

NSString *HDSearchResultSourcePProtoKey = @"PProto";
NSString *HDSearchResultSourcePProtoIconKey = @"PProtoIcon";
NSString *HDSearchResultSourcePAvailKey = @"PAvail";	// [AM][TODO] is there some reason that we have both PAvail and PAvailability?
NSString *HDSearchResultSourcePAvailabilityKey = @"PAvailability";	// [AM][TODO] is there some reason that we have both PAvail and PAvailability?

NSString *HDSearchResultSourceParentKey = @"parent";



/// THIS NEEDS SOME OPTIMIZING... -> next release
@implementation HDSearchResultSource

- (id)initWithHashes:(NSSet *)_hashes andTreeController:(NSTreeController *)aTreeController andTable:(PSearchResultsOutlineView *)_table hidden:(BOOL)hidden
{
	if (self=[super init]) {
		[self setResultsTreeController:aTreeController];
	
		table = _table;
		downloading_hashes = _hashes;
	
		source = [[NSMutableArray alloc] init];
		filtered = [[NSMutableArray alloc] init];
		current_src = source;
		hashes = [[NSMutableDictionary alloc] init];

		sorting_selector = nil;
		selected_column = nil;
		ascending = [[NSImage imageNamed:@"NSAscendingSortIndicator"] retain];
		descending = [[NSImage imageNamed:@"NSDescendingSortIndicator"] retain];
		downloading = [[NSImage imageNamed:@"downloading.tiff"] retain];
		
		[self setFilterProtocols:nil];
		keyword = nil;
		min_size = 0;
		max_size = 625;
		d_min_size = 0.0;
		d_max_size = 25.0;
		
		isFiltered	= NO;

		[self setActive:YES];	// new search creates new HDSearchResultSource instance => serach is active.
		
		icon_shop = [[PIconShop alloc] init];
		
		// [AM] why did the Poisoned Project list "666 Users" (before I changed it to "0 Users")?
				//[[NSFontManager sharedFontManager] fontWithFamily:[[NSFont systemFontOfSize:11] familyName] traits:NSBoldFontMask weight:9 size:11.0],NSFontAttributeName,
		attr_users = [[NSMutableAttributedString alloc] initWithString:@"0 Users" attributes:
			[NSDictionary dictionaryWithObject:[NSFont boldSystemFontOfSize:11] forKey:NSFontAttributeName]
		];
		[self setHidden:hidden];

	}
	return self;
}

- (void)dealloc
{
	[self setResultsTreeController:nil];

	[self setFilterProtocols:nil];
	
	[source release];
	[filtered release];
	[hashes release];
	[ascending release];
	[descending release];
	[downloading release];
	[attr_users release];
	[icon_shop release];
	if (keyword) [keyword release];
	[super dealloc];
}

- (NSTreeController *)resultsTreeController {
	return [[AM_searchResultsTreeController retain] autorelease];
}

- (void)setResultsTreeController:(NSTreeController *)aTreeController {
	if (aTreeController != AM_searchResultsTreeController) {
		[AM_searchResultsTreeController release];
		AM_searchResultsTreeController = [aTreeController retain];
	}
}

/*- (void)downloadEvent:(id)sender
{
	NSLog(@"Not Implemented...");
}*/

// add new item to the results
// --------------------------------------------------------------------------------------------------
- (void)addItem:(NSArray *)data
{
	if (data && [self isActive]) {
		NSMutableDictionary *item	= [data objectAtIndex:2];

		NSString *user		= [item objectForKey:HDSearchResultSourceUserKey];
		NSString *hash		= [item objectForKey:HDSearchResultSourceHashKey];
		NSArray *meta		= [data objectAtIndex:3];
		NSString *avail		= [item objectForKey:HDSearchResultSourceAvailabilityKey];

		NSString *meta_str;
		if ([meta count]>0) {
			if (meta_str=[[[meta objectAtIndex:0] objectAtIndex:2] objectForKey:HDSearchResultSourceBitrateKey])
				[item setObject:meta_str forKey:HDSearchResultSourceBitrateKey];
			if (meta_str=[[[meta objectAtIndex:0] objectAtIndex:2] objectForKey:HDSearchResultSourceArtistKey])
				[item setObject:meta_str forKey:HDSearchResultSourceArtistKey];
			if (meta_str=[[[meta objectAtIndex:0] objectAtIndex:2] objectForKey:HDSearchResultSourceAlbumKey])
				[item setObject:meta_str forKey:HDSearchResultSourceAlbumKey];
			if (meta_str=[[[meta objectAtIndex:0] objectAtIndex:2] objectForKey:HDSearchResultSourceTitleKey])
				[item setObject:meta_str forKey:HDSearchResultSourceTitleKey];
			if (meta_str=[[[meta objectAtIndex:0] objectAtIndex:2] objectForKey:HDSearchResultSourceDurationKey])
				[item setObject:meta_str forKey:HDSearchResultSourceDurationKey];
			// PDuration = formatted duration -rizzi
			if (meta_str=[[[meta objectAtIndex:0] objectAtIndex:2] objectForKey:HDSearchResultSourceFormattedDurationKey])
				[item setObject:meta_str forKey:HDSearchResultSourceFormattedDurationKey];
		}
			
		
		
		NSMutableDictionary *sources = [hashes objectForKey:hash];
		if (sources && [sources objectForKey:user]) {   // already found from this user
			return;
		}
		
		NSScanner *scanner = [NSScanner scannerWithString:[item objectForKey:HDSearchResultSourceURLKey]];
		NSString *proto;
		[scanner scanUpToString:@":" intoString:&proto];

		[item setObject:proto forKey:HDSearchResultSourcePProtoKey];
		[item setObject:avail forKey:HDSearchResultSourcePAvailKey];
		BOOL matchesFilter = [self matchesFilter:item];

		if (sources) { //------------------------------------- new source for existing file
			NSMutableArray *sourcesArray = [sources objectForKey:HDSearchResultSourceArrayKey];
			int sourceCount = [sourcesArray count];
			NSMutableDictionary *parent;
			NSMutableDictionary *parent_copy;
			
			if (sourceCount==1) {				// we now have multiple sources -> needs some setup
				parent = [sourcesArray objectAtIndex:0];
				parent_copy = [[parent mutableCopy] autorelease];
				[parent removeObjectForKey:HDSearchResultSourceIconKey];
				[parent removeObjectForKey:HDSearchResultSourcePProtoIconKey];
				//[parent removeObjectForKey:HDSearchResultSourcePAvailabilityKey];
				//[parent_copy setObject:[icon_shop iconForAvail:avail] forKey:HDSearchResultSourcePAvailabilityKey];

				[parent_copy setObject:[NSString stringWithString:HDSearchResultSourceExpandableKey] forKey:HDSearchResultSourceExpandableKey];
				[sources setObject:parent_copy forKey:HDSearchResultSourceParentKey];
				[sourcesArray insertObject:parent_copy atIndex:0];
			}
			

			parent = [sourcesArray objectAtIndex:0];
			[sourcesArray addObject:item];
			[[attr_users mutableString] setString:[NSString stringWithFormat:@"%i Users",([sourcesArray count]-1)]];
			[parent setObject:[[attr_users copy] autorelease] forKey:HDSearchResultSourceUserKey];
			[item setObject:[icon_shop iconForAvail:avail] forKey:HDSearchResultSourcePAvailabilityKey];
			[sources setObject:item forKey:user];

			if (selected_column && [[selected_column identifier] isEqualToString:HDSearchResultSourceUserKey]) {
				[source removeObject:sourcesArray];
				[self insertObject:sourcesArray source:source];
				// totally forgot to check here if the filter matches, should be here, fixed now
				if (matchesFilter && isFiltered) {
					[filtered removeObject:sourcesArray];
					[self insertObject:sourcesArray source:filtered];
				}
			}
		}
		else { //--------------------------------------------  new file
			[item setObject:[icon_shop iconForProto:proto] forKey:HDSearchResultSourcePProtoIconKey];
			[item setObject:[icon_shop iconForFileType:[[item objectForKey:HDSearchResultSourceFileKey] pathExtension]] forKey:HDSearchResultSourceIconKey];
			[item setObject:[icon_shop iconForAvail:avail] forKey:HDSearchResultSourcePAvailabilityKey];
			NSMutableArray *newFileArray = [NSMutableArray arrayWithObjects:item,nil];
			NSMutableDictionary *newFile = [NSMutableDictionary dictionaryWithObjectsAndKeys:
				newFileArray,HDSearchResultSourceArrayKey,
				item,HDSearchResultSourceParentKey,
				item,user,
				nil];
			// we have to test for a hash here since there isn't always one
			// this fixes the bug where searching doesn't work anymore, after this happened
			if (hash) [hashes setObject:newFile forKey:hash];
			
			[sources setObject:item forKey:user];
			
			if (sorting_selector) {
				[self insertObject:newFileArray source:source];
				if (matchesFilter) [self insertObject:newFileArray source:filtered];
			}
			else {
				[source addObject:newFileArray];
				if (matchesFilter) [filtered addObject:newFileArray];
			}
		}
	}
}

// --------------------------------------------------------------------------------------------------


- (NSArray *)source
{
	return current_src;
}

- (NSString *)resultCount
{
	if (isFiltered) return [NSString stringWithFormat:@"%i of %i",[filtered count],[source count]];
	else return [NSString stringWithFormat:@"%i",[source count]];
}

- (NSArray *)selectedItems
{
	NSMutableArray *_items = [NSMutableArray array];
	NSEnumerator *enumerator = [table selectedRowEnumerator];
	NSDictionary *item;
	NSNumber *index;
	while (index = [enumerator nextObject]) {
		item = [table itemAtRow:[index intValue]];
		if ([table levelForItem:item]==0 && [item objectForKey:HDSearchResultSourceExpandableKey]) {
			NSArray *sources = [[hashes objectForKey:[item objectForKey:HDSearchResultSourceHashKey]] objectForKey:HDSearchResultSourceArrayKey];
			NSEnumerator *e = [sources objectEnumerator];
			id src;
			
			id firstSource = nil;
			
			// skip, for some reason, the first object
			while (nil != (src = [e nextObject])) {
				if (nil != firstSource) {
					[_items addObject:src];
				}
				else {
					firstSource = src;
				}
			}
		}
		else {
			[_items addObject:item];
		}
	}
	return _items;
}

- (void)cleanUpTableHeaders
{
	if (selected_column) {
		[table setHighlightedTableColumn:nil];
		[table setIndicatorImage:nil inTableColumn:selected_column];
	}
}

- (void)setTableHeaders
{
	if (selected_column) {
		[table setHighlightedTableColumn:selected_column];
		if (sortAscending) [table setIndicatorImage:ascending inTableColumn:selected_column];
		else [table setIndicatorImage:descending inTableColumn:selected_column];
	}
}

// Setting Filters...
// --------------------------------------------------------------------------------------------------

- (BOOL)matchesFilter:(NSDictionary *)item
{
	BOOL matched = NO;

	if (isFiltered) {
		if (![self matchesKeyword:item])
			matched = NO;
		else if (![self matchesMinSize:item])
			matched = NO;
		else if (![self matchesMaxSize:item])
			matched = NO;
		else if (![self matchesProto:item])
			matched = NO;
		else
			matched = YES;
	}
	
	return matched;
}

- (void)setFiltered:(BOOL)_filter
{
	isFiltered = _filter;
	if (isFiltered) {
		[filtered removeAllObjects];
		current_src=filtered;
		[self filter];
	}
	else {
		current_src=source;
		[table reloadData];
	}
}

- (void)setKeywordFilter:(NSString *)_key
{
	[keyword autorelease];
	keyword = [_key copy];
	[self filter];
}

- (void)setMinSizeFilter:(double)_min
{
	d_min_size=_min;
	min_size=(int)_min*(int)_min;
	[self filter];
}

- (void)setMaxSizeFilter:(double)_max
{
	d_max_size=_max;
	max_size=(int)_max*(int)_max;
	[self filter];
}

- (void)filterProtos
{
	[self filter];
}
// --------------------------------------------------------------------------------------------------


// getting filters...
// --------------------------------------------------------------------------------------------------

- (BOOL)isFiltered
{
	return isFiltered;
}

- (void)filter
{
	[filtered removeAllObjects];
	
	NSEnumerator *e = [source objectEnumerator];
	NSArray *item;
	
	while (nil != (item = [e nextObject])) {
		if ([self matchesFilter:[item objectAtIndex:0]]) [filtered addObject:item];
	}
	
	[table reloadData];
}

- (NSString *)keywordFilter
{
	return keyword;
}

- (double)minSizeFilter
{
	return d_min_size;
}

- (double)maxSizeFilter
{
	return d_max_size;
}

- (BOOL)matchesKeyword:(NSDictionary *)item
{
	BOOL matched = YES;

	if (!keyword || [keyword length]==0)
		matched = YES;
	else if ([[item objectForKey:HDSearchResultSourceFileKey] rangeOfString:keyword options:NSCaseInsensitiveSearch].location==NSNotFound)
		matched = NO;
	
	return matched;
}

- (BOOL)matchesProto:(NSDictionary *)item
{
	return [[self filterProtocols] containsObject:[item objectForKey:HDSearchResultSourcePProtoKey]];
}

- (BOOL)matchesMinSize:(NSDictionary *)item
{	
	if (min_size==0) return YES;
	return ([[item objectForKey:HDSearchResultSourceSizeKey] intValue]>=min_size*HD_MEGABYTE);
}

- (BOOL)matchesMaxSize:(NSDictionary *)item
{
	if (max_size==625) return YES;
	else if (max_size==0)return NO;
	return ([[item objectForKey:HDSearchResultSourceSizeKey] intValue]<=max_size*HD_MEGABYTE);
}

// --------------------------------------------------------------------------------------------------

// outline view datasource for the results table...
// --------------------------------------------------------------------------------------------------
- (id)outlineView:(NSOutlineView *)outlineView child:(int)index ofItem:(id)item
{
	if (item==nil) return [[current_src objectAtIndex:index] objectAtIndex:0];
	else return [[[hashes objectForKey:[item objectForKey:HDSearchResultSourceHashKey]] objectForKey:HDSearchResultSourceArrayKey] objectAtIndex:index+1];
}

- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item
{
	BOOL expandable = NO;

	if ([item objectForKey:HDSearchResultSourceExpandableKey])
		expandable = YES;
		
	return expandable;
}

- (int)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item
{
	if (item==nil) return [current_src count];
	else return [[[item objectForKey:HDSearchResultSourceUserKey] string] intValue];
}

- (id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item {
	NSString *identifier = [tableColumn identifier];
	if ([identifier isEqualToString:HDSearchResultSourceIconKey]) {
		if ([outlineView levelForItem:item]==0) {
			if ([downloading_hashes containsObject:[item objectForKey:HDSearchResultSourceHashKey]]) return downloading;
			else return [item objectForKey:identifier];
		}
		else return nil;
	}
	return [item objectForKey:identifier];
}

- (BOOL)outlineView:(NSOutlineView *)outlineView shouldSelectTableColumn:(NSTableColumn *)tableColumn
{
	NSString *identifier = [tableColumn identifier];
	if (selected_column) [outlineView setIndicatorImage:nil inTableColumn:selected_column];
	if (selected_column==tableColumn) sortAscending=!sortAscending;
	else if ([identifier isEqualToString:@"calcsize"] || [identifier isEqualToString:HDSearchResultSourceUserKey] || [identifier isEqualToString:HDSearchResultSourceBitrateKey] ) sortAscending=NO;
	else sortAscending=YES;
	selected_column = tableColumn;
	[outlineView setHighlightedTableColumn:tableColumn];
	
	NSMutableArray *source1=nil; 
	NSMutableArray *source2=nil; 
	if (isFiltered) { 		// process the displayed source first (hopefully)
		source1 = filtered;
		source2 = source;
	}
	else {
		source1 = source;
		source2 = filtered;
	}
	
	if (sortAscending) {
		[outlineView setIndicatorImage:ascending inTableColumn:tableColumn];
		/*sorting_selector = sel_getUid([
			[NSString stringWithFormat:@"%@Asc:",identifier] cString]);*/
		sorting_selector = NSSelectorFromString([NSString stringWithFormat:@"%@Asc:",identifier]);
			NSLog(@"sortingSelector1 = %@", NSStringFromSelector(sorting_selector));	// [AM]
		[source1 sortUsingSelector:sorting_selector];
		[outlineView reloadData];
		if (source2) [source2 sortUsingSelector:sorting_selector];
	}
	else {
		[outlineView setIndicatorImage:descending inTableColumn:tableColumn];
		/*sorting_selector = sel_getUid([
			[NSString stringWithFormat:@"%@Desc:",identifier] cString]);*/
		sorting_selector = NSSelectorFromString([NSString stringWithFormat:@"%@Desc:",identifier]);
			NSLog(@"sortingSelector2 = %@", NSStringFromSelector(sorting_selector)); // [AM]
		[source1 sortUsingSelector:sorting_selector];
		[outlineView reloadData];
		if (source2) [source2 sortUsingSelector:sorting_selector];
	}
	return NO;
}



// --------------------------------------------------------------------------------------------------

@end

@implementation HDSearchResultSource (Accessors)

- (BOOL)isActive {
	return AM_active;
}

- (void)setActive:(BOOL)flag {
	AM_active = flag;
}

- (BOOL)isHidden {
	return AM_hidden;
}

- (void)setHidden:(BOOL)flag {
	AM_hidden = flag;
}

- (NSMutableArray *)filterProtocols {
	return [[AM_filterProtocols retain] autorelease];
}

- (void)setFilterProtocols:(NSArray *)anArray {
	if (anArray != AM_filterProtocols) {
		[AM_filterProtocols release];
		AM_filterProtocols = [anArray mutableCopy];
	}
}



@end