// HDTransferSource.h
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import "HDTransferSource.h"
#import "HDValueTransformers.h"

#import "HDTransferSourceEntry.h"

#import "PIconShop.h"



@implementation HDTransferSource

- (id)init {
	if (nil != (self = [super init])) {
		iconShop = [[PIconShop alloc] init];
		
		[self setSourceArray:[NSArray array]];
		[self setTicketsDictionary:[NSDictionary dictionary]];
		
		//sorting_selector = NULL;
		//selected_column = nil;
		//ascending = [[NSImage imageNamed:@"NSAscendingSortIndicator"] retain];
		//descending = [[NSImage imageNamed:@"NSDescendingSortIndicator"] retain];
		
		//sortAscending = YES;
	}
	return self;
}

- (void)dealloc {
	// hacky variables
	//[ascending release];
	//[descending release];
	// end hacky variables

	[iconShop release];
	iconShop = nil;

	[self setSourceArray:nil];
	[self setTicketsDictionary:nil];
	
	[super dealloc];
}

/*- (id)commander {
	return giftCommander;
}

- (void)setCommander:(id)aCommander
{
	giftCommander = aCommander;
}*/

- (void)disconnected {
	[[self ticketsDictionary] removeAllObjects];
	[[self sourceArray] removeAllObjects];
}

- (void)cleanUp:(id)sender {
	/*NSMutableArray *sourceArrayCopy = [[self sourceArray] mutableCopy];
	NSEnumerator *e = [[self sourceArray] objectEnumerator];
	NSDictionary *tmp;
	
	while (nil != (tmp = [e nextObject])) {
		int state = [[tmp objectForKey:@"PStatus"] intValue];
		if (state==HDTransferStatusCompleted || state==HDTransferStatusCanceled) {
			[[self ticketsDictionary] removeObjectForKey:[tmp objectForKey:@"PTicket"]];
			[sourceArrayCopy removeObject:tmp];
			break;
		}
	}
	
	[self setSourceArray:sourceArrayCopy];
	[sourceArrayCopy release];*/
	
	NSMutableArray *sourceArrayCopy = [[self sourceArray] copy];
	
	NSEnumerator *e = [sourceArrayCopy objectEnumerator];
	NSDictionary *tmp;
	
	while (nil != (tmp = [e nextObject])) {
		int state = [[tmp valueForKey:@"PStatus"] intValue];
		if (state==HDTransferStatusCompleted || state==HDTransferStatusCanceled) {
			[[self ticketsDictionary] removeObjectForKey:[tmp valueForKey:@"PTicket"]];
			[[self sourceArray] removeObject:tmp];
		}
	}
	
	[sourceArrayCopy release];
	
	/*NSMutableArray *sourcesToRemove = [[NSMutableArray alloc] init];
	NSMutableArray *ticketsToRemove = [[NSMutableArray alloc] init];

	NSEnumerator *e = [[self sourceArray] objectEnumerator];
	NSDictionary *tmp;
	
	while (nil != (tmp = [e nextObject])) {
		int state = [[tmp objectForKey:@"PStatus"] intValue];
		if (state==HDTransferStatusCompleted || state==HDTransferStatusCanceled) {
			[sourcesToRemove addObject:tmp];
			[ticketsToRemove addObject:[tmp objectForKey:@"PTicket"]];
			//[[self ticketsDictionary] removeObjectForKey:[tmp objectForKey:@"PTicket"]];
			//[[self sourceArray] removeObject:tmp];
		}
	}
	
	[[self sourcesArray] removeObjects:sourcesToRemove];
	[[self ticketsDictionary] removeObjectsForKeys:ticketsToRemove];
	
	[sourcesToRemove release];
	[ticketsToRemove release];*/
}

- (int)transferSpeed {
	NSEnumerator *e = [[self sourceArray] objectEnumerator];
	id obj;

	float tmp, total=0.0;
	while (nil != (obj = [e nextObject])) {
		if (tmp=[[[obj valueForKey:@"PTransfer"] objectAtIndex:1] floatValue]) {
			total += tmp;
		}
	}
	return (int)floor(total);
}

- (NSString *)calcSize:(NSString *)size
{
	NSString *returnValue;

	NSValueTransformer *vt = [[PZSizeValueTransformer alloc] init];
	returnValue = [vt transformedValue:size];
	[vt release];

	return returnValue;

	/*if (!size) return @"";
	long long s;
	[[NSScanner scannerWithString:size] scanLongLong:&s];
	if (s==0) return @"nothing";
	else if (s<1024) return @"0 B";
	else if (s<1048576) return [NSString stringWithFormat:@"%i kB",(int)((float)s/(float)HD_KILOBYTE)];
	else if (s<1073741824) return [NSString stringWithFormat:@"%.2f MB",((float)s/(float)HD_MEGABYTE)];
	else return [NSString stringWithFormat:@"%.2f GB",((float)s/(float)HD_GIGABYTE)];*/
}

- (NSString *)stringForUsers:(int)users
{
	NSString *string;

	if (users != 1) {
		string = [NSString stringWithFormat:@"%i Users", users];
	}
	else {
		string = [NSString stringWithFormat:@"%i User", users];
	}

	return string;
}

- (NSNumber *)transmit:(NSString *)transmitString total:(NSString *)totalString
{
	NSNumber *returnValue;
	if (transmitString && totalString) {
		long long transmit, total;
		[[NSScanner scannerWithString:transmitString] scanLongLong:&transmit];
		[[NSScanner scannerWithString:totalString] scanLongLong:&total];
		returnValue = [NSNumber numberWithFloat:(float)transmit/(float)total];
	}
	else {
		returnValue = [NSNumber numberWithFloat:0.0];
	}
	return returnValue;
}

- (NSString *)throughput:(NSString *)throughputString elapsed:(NSString *)elapsedString
{
	NSString *returnValue;
	if (throughputString && elapsedString) {
		long long throughput, elapsed;
		[[NSScanner scannerWithString:throughputString] scanLongLong:&throughput];
		[[NSScanner scannerWithString:elapsedString] scanLongLong:&elapsed];
		returnValue = [NSString stringWithFormat:@"%.2f kB/s",(float)throughput*1000/(elapsed*(2 << 9))];
	}
	else {
		returnValue = @"";
	}
	
	return returnValue;
}

- (NSString *)timeStringFromInt:(int)sec
{
	int h=0,m=0;
	if (sec>=3600) {
		h = sec/3600;
		sec = sec % 3600;
	}
	if (sec>=60) {
		m = sec/60;
		sec = sec%60;
	}
	return [NSString stringWithFormat:@"%i:%02d:%02d",h,m,sec];
}

- (NSString *)remainingWithTotalSize:(NSString *)size transmit:(NSString *)transmit throughput:(NSString *)throughput elapsed:(NSString *)elapsed {
	NSString *returnValue = @"";
	if (size && transmit && throughput && elapsed) {
		long long s,tr,t,e;
		[[NSScanner scannerWithString:size] scanLongLong:&s];
		[[NSScanner scannerWithString:transmit] scanLongLong:&tr];
		[[NSScanner scannerWithString:throughput] scanLongLong:&t];
		[[NSScanner scannerWithString:elapsed] scanLongLong:&e];
		int speed = t*1000/e;
		int sec = (int)(s-tr)/speed;
		returnValue = [self timeStringFromInt:sec];
	}
	return returnValue;
}

#pragma mark -
#pragma mark Transfer Command Support

- (void)DEL:(NSArray *)data
{
	// do nothing
}

- (void)reloadTableData:(void *)contextInfo {
	// do nothing; meant to be overridden
}

//#pragma mark -
//#pragma mark Table and Outline View Hacks

/*- (void)cleanUpTableHeaders
{
	if (selected_column) {
		[table setHighlightedTableColumn:nil];
		[table setIndicatorImage:nil inTableColumn:selected_column];
	}
}*/

/*- (void)setTableHeaders
{
	if (selected_column) {
		[table setHighlightedTableColumn:selected_column];
		if (sortAscending) [table setIndicatorImage:ascending inTableColumn:selected_column];
		else [table setIndicatorImage:descending inTableColumn:selected_column];
	}
}*/

@end

#pragma mark -

@implementation HDTransferSource (Accessors)

- (NSMutableArray *)sourceArray {
	return [[AM_sourceArray retain] autorelease];
}

- (void)setSourceArray:(NSArray *)anArray {
	if (anArray != AM_sourceArray) {
		[AM_sourceArray release];
		AM_sourceArray = [anArray mutableCopy];
	}
}

- (NSMutableDictionary *)ticketsDictionary {
	return [[AM_ticketsDictionary retain] autorelease];
}

- (void)setTicketsDictionary:(NSDictionary *)aDictionary {
	if (aDictionary != AM_ticketsDictionary) {
		[AM_ticketsDictionary release];
		AM_ticketsDictionary = [aDictionary mutableCopy];
	}
}

@end
