//
//  HDTransferSourceDownloads.m
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import "HDTransferSourceDownloads.h"
#import "HDPreferencesConfigurationEditorGiFT.h"
#import "HDMusicIntegrationController.h"
#import "HDDaemonCommander.h"
#import "HDMediaTypeDetector.h"

#import "PDownloadTitleDescriptionCell.h"
#import "PIconShop.h"
#import "HDActionsController.h"

#import "HDDownloadEntry.h"
#import "PDownloadsOutlineView.h"
#import "HDAlertLocalizedStringKeys.h"

//#include <unistd.h>	// [AM] why was this included?!


@implementation HDTransferSourceDownloads

- (id)initWithTable:(PDownloadsOutlineView *)_table andTreeController:(NSTreeController *)aTreeController
{
	if (self = [super init])
	{
		iTunesController = [[HDMusicIntegrationController alloc] init];
	
		table = _table;
						
		menu = [[NSMenu alloc] init];
		//timers = [[NSMutableDictionary alloc] init];
		
		downloadsTreeController = aTreeController;
	}
	
	return self;
}

- (void)dealloc
{
	[iTunesController release];
	iTunesController = nil;

	[menu release];
	//[timers release];
	
	[super dealloc];
}

- (void)setHashes:(NSMutableSet *)_hashes
{
	hashes = _hashes;
}

- (BOOL)validateFindMoreSources
{
	return [self validateCancel];
}

- (BOOL)validatePreview
{
	return [self validateCancel];
}

- (BOOL)validateDelSource
{
	BOOL isValidated = NO;

	if ([table numberOfSelectedRows]>0) {
		isValidated = YES;
		NSEnumerator *enumerator = [table selectedRowEnumerator];
		NSNumber *num;
		HDDownloadEntry *selItem;
		while (num=[enumerator nextObject]) {
			selItem = [table itemAtRow:[num intValue]];
			if ([[selItem valueForKey:HDTransferSourceDownloadsPExpandableKey] boolValue]) {
				isValidated = NO;
				break;
			}
			else if (![[[selItem valueForKey:HDTransferSourcePProgressKey] objectAtIndex:1] boolValue] && [[[selItem valueForKey:HDTransferSourcePProgressKey] objectAtIndex:2] isEqualToString:@""]) {
				isValidated = NO;
				break;
			}
		}
	}
	
	return isValidated;
}

- (BOOL)validateCancel
{
	BOOL isValidated = NO;

	if ([table numberOfSelectedRows]>0) {
		isValidated = YES;
		NSEnumerator *enumerator = [table selectedRowEnumerator];
		NSNumber *num;
		HDDownloadEntry *tmp;
		while (num=[enumerator nextObject]) {
			tmp = [table itemAtRow:[num intValue]];
			if (![[tmp valueForKey:HDTransferSourceDownloadsPExpandableKey] boolValue] || ([[tmp valueForKey:HDTransferSourcePStatusKey] intValue]<=1) ) {
				isValidated = NO;
				break;
			}
		}
	}
	
	return isValidated;
}

- (BOOL)validatePause
{
	BOOL isValidated = NO;

	if ([table numberOfSelectedRows]>0) {
		isValidated = YES;
		NSEnumerator *enumerator = [table selectedRowEnumerator];
		NSNumber *num;
		HDDownloadEntry *tmp;
		while (num=[enumerator nextObject]) {
			tmp = [table itemAtRow:[num intValue]];
			if (![[tmp valueForKey:HDTransferSourceDownloadsPExpandableKey] boolValue] || [[tmp valueForKey:HDTransferSourcePStatusKey] intValue] != HDTransferStatusActive) {
				isValidated = NO;
				break;
			}
		}
	}
	
	return isValidated;
}

- (BOOL)validateResume
{
	BOOL isValidated = NO;

	if ([table numberOfSelectedRows]>0) {
		isValidated = YES;
		NSEnumerator *enumerator = [table selectedRowEnumerator];
		NSNumber *num;
		HDDownloadEntry *tmp;
		while (num=[enumerator nextObject]) {
			tmp = [table itemAtRow:[num intValue]];
			if (![[tmp valueForKey:HDTransferSourceDownloadsPExpandableKey] boolValue] || [[tmp valueForKey:HDTransferSourcePStatusKey] intValue] != HDTransferStatusPaused) {
				isValidated = NO;
				break;
			}
		}
	}
	
	return isValidated;
}

- (NSString *)hashForTicket:(NSString *)ticket
{
	return [[[self ticketsDictionary] objectForKey:ticket] valueForKey:HDTransferSourceDownloadsHashKey];
}

- (void)deleteEvent:(id)sender
{
	//if ([item objectForKey:HDTransferSourceDownloadsPExpandableKey])
	if([[NSUserDefaults standardUserDefaults] boolForKey:@"HDDownloadsConfirmCancellation"]) {
		NSAlert *alert = [NSAlert alertWithMessageText:HDLocalizedAlert(HDTransferSourceDownloadsRemoveSelectedDownloadsMessageTextKey)
		defaultButton:HDLocalizedYesButton
		alternateButton:HDLocalizedNoButton
		otherButton:nil
		informativeTextWithFormat:HDLocalizedAlert(HDTransferSourceDownloadsRemoveSelectedDownloadsInformativeTextKey)];

		[alert beginSheetModalForWindow:[[NSApplication sharedApplication] mainWindow]
			modalDelegate:self
			didEndSelector:@selector(deleteSheetDidEnd:returnCode:contextInfo:)
			contextInfo:NULL];
		
	} else {
		[self doDelete];
	}
}

- (void)deleteSheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
	if(returnCode == NSAlertDefaultReturn)
	{
		[self doDelete];
	}
}

- (void)doDelete
{
	// this array stores the tickets of the deleted downloads
	// so we don't have to delete their sources, if they are selected too
	NSMutableArray *deletedDownloads = [[NSMutableArray alloc] init];
		
	// we store all commands in one string, and send it at once
	NSMutableString *tmpcmd = [[NSMutableString alloc] initWithString:@""];
	
	NSEnumerator *enumerator = [table selectedRowEnumerator];
	NSNumber *num;
	HDDownloadEntry *item;
	while (num=[enumerator nextObject]) {
		item = [table itemAtRow:[num intValue]];
		
		// if it's a download -> cancel the download and remove it
		if ([[item valueForKey:HDTransferSourceDownloadsPExpandableKey] boolValue]) {
			[deletedDownloads addObject:[item valueForKey:HDTransferSourcePTicketKey]];
			if ([[item valueForKey:HDTransferSourcePStatusKey] intValue]>1) {
				[deletedDownloads addObject:[item valueForKey:HDTransferSourcePTicketKey]];
				[tmpcmd appendString:[NSString stringWithFormat:@";\nTRANSFER(%@) action(cancel)",[item valueForKey:HDTransferSourcePTicketKey]]];
				
				
				//// [MY][MAJOR][BUG][FOUND]
				// we have to remove the hash from the hashes set.
				// if we wait for gift to send a DELDOWNLOAD: it could happen that
				// the cancelled download starts again if the user is doing 'find more sources'
				// and the hash is still in the hashes set...
				if ([item valueForKey:HDTransferSourceDownloadsHashKey]) [hashes removeObject:[item valueForKey:HDTransferSourceDownloadsHashKey]];
			}
		}
		// a source is selected...
		else if (![deletedDownloads containsObject:[item valueForKey:HDTransferSourcePTicketKey]]) {
			[tmpcmd appendString:[NSString stringWithFormat:@";\nDELSOURCE(%@) url(%@)",
				[item valueForKey:HDTransferSourcePTicketKey],
				[[HDDaemonCommander sharedInstance] prepare:[item valueForKey:HDTransferSourceDownloadsURLKey]]]];
		}
	}
	
	// now we just have to remove the deleted downloads from the table
	// the source will be removed by DELSOURCE: as soon as gift removes the source
	NSEnumerator *e = [deletedDownloads objectEnumerator];
	NSString *ticket;
	while (nil != (ticket = [e nextObject])) {
		[[self sourceArray] removeObject:[[self ticketsDictionary] objectForKey:ticket]];
		[[self ticketsDictionary] removeObjectForKey:ticket];	// ticket, not tickets, right? must have been a typo...
	}
	[table reloadData];
	if ([tmpcmd length]>2) [[HDDaemonCommander sharedInstance] performSelector:@selector(executeCommand:) withObject:[tmpcmd substringFromIndex:2]];
	
	[deletedDownloads release];
	[tmpcmd release];
}

- (void)cancel:(id)commander
{
	NSEnumerator *enumerator = [table selectedRowEnumerator];
	NSNumber *num;
	HDDownloadEntry *item;
	while (num=[enumerator nextObject]) {
		item = [table itemAtRow:[num intValue]];
		[commander performSelector:@selector(executeCommand:) withObject:[NSString stringWithFormat:@"TRANSFER(%@) action(cancel)",[item valueForKey:HDTransferSourcePTicketKey]]];
		[item setValue:[NSNumber numberWithInt:HDTransferStatusCanceled] forKey:HDTransferSourcePStatusKey];
		[item setValue:[NSMutableArray arrayWithObjects:
			[NSNumber numberWithBool:YES],
			[NSNumber numberWithBool:NO],
			NSLocalizedString(@"Cancelling...", @"Cancelling..."),
			nil]
		forKey:HDTransferSourcePProgressKey];
		[table reloadItem:item reloadChildren:YES];
	}
}

- (void)pause:(id)commander
{
	NSEnumerator *enumerator = [table selectedRowEnumerator];
	NSNumber *num;
	HDDownloadEntry *item;
	while (num=[enumerator nextObject]) {
		item = [table itemAtRow:[num intValue]];
		[commander performSelector:@selector(executeCommand:) withObject:[NSString stringWithFormat:@"TRANSFER(%@) action(pause)",[item valueForKey:HDTransferSourcePTicketKey]]];
		[item setValue:[NSNumber numberWithInt:HDTransferStatusPausing] forKey:HDTransferSourcePStatusKey];
		[item setValue:[NSMutableArray arrayWithObjects:
			[NSNumber numberWithBool:YES],
			[NSNumber numberWithBool:NO],
			NSLocalizedString(@"Pausing...", @"Pausing"),
			nil]
		forKey:HDTransferSourcePProgressKey];
		[table reloadItem:item reloadChildren:YES];
	}
}

- (void)resume:(id)commander
{
	NSEnumerator *enumerator = [table selectedRowEnumerator];
	NSNumber *num;
	HDDownloadEntry *item;
	while (num=[enumerator nextObject]) {
		item = [table itemAtRow:[num intValue]];
		[commander performSelector:@selector(executeCommand:) withObject:[NSString stringWithFormat:@"TRANSFER(%@) action(unpause)",[item valueForKey:HDTransferSourcePTicketKey]]];
		[item setValue:[NSNumber numberWithInt:HDTransferStatusResuming] forKey:HDTransferSourcePStatusKey];
		[item setValue:[NSMutableArray arrayWithObjects:
			[NSNumber numberWithBool:YES],
			[NSNumber numberWithBool:NO],
			NSLocalizedString(@"Resuming...", @"Resuming..."),
			nil]
		forKey:HDTransferSourcePProgressKey];
		[table reloadItem:item reloadChildren:YES];
	}
}

- (void)delsource:(id)commander
{
	NSEnumerator *enumerator = [table selectedRowEnumerator];
	NSNumber *num;
	HDDownloadEntry *tmp;
	NSString *tmpcmd=@"";
	while (num=[enumerator nextObject]) {
		tmp = [table itemAtRow:[num intValue]];
		if (![[tmp valueForKey:HDTransferSourceDownloadsPExpandableKey] boolValue])
			// never forget to prepare a string, if it's possible that it contains characters that must
			// be escaped ->  \ { } [ ] ( )
			tmpcmd = [tmpcmd stringByAppendingString:[NSString stringWithFormat:@";\nDELSOURCE(%@) url(%@)",[tmp valueForKey:HDTransferSourcePTicketKey],[commander prepare:[tmp valueForKey:HDTransferSourceDownloadsURLKey]]]];
	}
	[commander performSelector:@selector(executeCommand:) withObject:[tmpcmd substringFromIndex:2]];
}

/*- (void)browseHost:(id)commander
{
}*/

- (void)expandAll
{
	// [NOTE][MAJOR] to expand all sublevels (even though we don't have more than one) we must start at the beginning
	//NSEnumerator *e = [[self sourceArray] objectEnumerator];
	/*NSEnumerator *e = [[self sourceArray] reverseObjectEnumerator];	// but we only have one sublevel
	id obj;
	
	while (nil != (obj = [e nextObject])) {
		[table expandItem:obj];
	}*/
	
	unsigned len = [table numberOfRows];
	int n;
	//for (n = 0; n < len; ++n)
	for (n = len - 1; n >= 0; --n) {
		id item = [table itemAtRow:n];
		[table expandItem:item];
	}
}

- (void)collapseAll
{
	// to collapse all sublevels (even though we don't have more than one) we must start at the end
	//NSEnumerator *e = [[self sourceArray] reverseObjectEnumerator];
	/*NSEnumerator *e = [[self sourceArray] objectEnumerator];	// but we only have one sublevel
	id obj;

	while (nil != (obj = [e nextObject])) {
		[table collapseItem:obj];
	}*/
	
	unsigned len = [table numberOfRows];
	int n;
	//for  (n = len - 1; n >= 0; ++n)
	for (n = 0; n < len; ++n) {
		id item = [table itemAtRow:n];
		[table collapseItem:item];
	}	
}

/*- (int)numberOfDownloads
{
	return [source count];
}*/

/*
- (void)findMoreSourcesTimer:(NSTimer *)timer
{
	//NSLog(@"fms: fired");
	NSString *ticket = [[timer userInfo] objectForKey:@"ticket"];
	
	// only find more sources if the download is active
	if ([[[tickets objectForKey:ticket] objectForKey:HDTransferSourcePStatusKey] intValue] != HDTransferStatusActive) return;
	
	[self findMoreSources:[[tickets objectForKey:ticket] objectForKey:HDTransferSourceDownloadsHashKey]];
}

- (void)findMoreSources:(NSString *)hash
{
	//NSLog(@"fms: send notification > %@",hash);
	[[NSNotificationCenter defaultCenter] postNotificationName:@"PFindMoreSources" object:self
		userInfo:[NSDictionary dictionaryWithObjectsAndKeys:
			hash,HDTransferSourceDownloadsHashKey,nil]];
}
*/

- (void)moveFile:(id)thePath toFolder:(id)folderPath
{
	id destPath = [folderPath stringByAppendingPathComponent:[thePath lastPathComponent]];
	[[NSFileManager defaultManager] movePath:thePath toPath:destPath handler:nil];
}

- (void)createDockBadgeIcon
{
	NSImage *appImage, *image;
	NSDictionary *attrs;
	NSString *str;
	NSSize iconSize, strSize;
	NSPoint p;
	bool showApp = YES;
	/* get the app's icon */
	appImage = [NSImage imageNamed:@"NSApplicationIcon"];
	image = [[NSImage alloc] initWithSize:[appImage size]];
	NSImage *badge = [NSImage imageNamed: @"badge.tiff"];
	/* get size */
	iconSize = [appImage size];
	/* set up font attributes based on size of icon */
	attrs = [NSDictionary dictionaryWithObjectsAndKeys:
		[NSFont boldSystemFontOfSize:22], NSFontAttributeName,
		[NSColor whiteColor], NSForegroundColorAttributeName, nil];
	str = @"";
	/* define point to draw string based on string bounding box */
	strSize = [str sizeWithAttributes:attrs];
	p = NSMakePoint(99,20);
	/* create the dock image */
	[image setFlipped:YES];
	[image lockFocus];
	/* display original application image if desired */
	if ( showApp == YES )
		[appImage compositeToPoint:NSMakePoint(0, [appImage size].height)
						 operation:NSCompositeSourceOver];
	/* this is the red check */
	[badge compositeToPoint:NSMakePoint(70,68) operation:NSCompositeSourceOver];
	/* this will draw the count into the badge */
	[str drawAtPoint:p withAttributes:attrs];  
	[image unlockFocus];
	/* set the new dock image */
	[NSApp setApplicationIconImage:image];
	[image release];
	[appImage release];
	[badge release];
	
}

/* end sorting code */

#pragma mark -
#pragma mark Outline View Methods

- (id)outlineView:(NSOutlineView *)outlineView child:(int)index ofItem:(id)item
{
	NSArray *array;

	if (!item) {
		array = [self sourceArray];
	}
	else {
		array = [item valueForKey:HDTransferSourceDownloadsPSourcesKey];
	}

	return [array objectAtIndex:index];
}

- (BOOL)outlineView:(NSOutlineView *)outlineView isItemExpandable:(id)item
{
	return [[item valueForKey:HDTransferSourceDownloadsPExpandableKey] boolValue];
}

- (int)outlineView:(NSOutlineView *)outlineView numberOfChildrenOfItem:(id)item
{
	NSArray *array;

	if (!item) {
		array = [self sourceArray];
	}
	else {
		array = [item valueForKey:HDTransferSourceDownloadsPSourcesKey];
	}
	
	return [array count];
}

- (id)outlineView:(NSOutlineView *)outlineView objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{  
	return [item valueForKey:[tableColumn identifier]];
}

// delegate method
- (void)outlineView:(NSOutlineView *)outlineView
	willDisplayCell:(id)cell
	forTableColumn:(NSTableColumn *)tableColumn
	item:(id)item 
{
	NSString *identifier = [tableColumn identifier];
	
	if ([identifier isEqualToString:PZDownloadEntryFileInfoArray]) {
		int rowIndex = [outlineView rowForItem:item];		
		BOOL shouldHighlight = ([[outlineView selectedRowIndexes] containsIndex:rowIndex] && [[NSApp mainWindow] firstResponder] == outlineView);
		NSAttributedString *attributedString = [cell generatedAttributedStringWithHighlight:shouldHighlight];
		[cell setAttributedStringValue:attributedString];
	}
	else if ([identifier isEqualToString:HDTransferSourceDownloadsPTransferKey] || [identifier isEqualToString:PZDownloadEntryFileSizeString]) {
		int rowIndex = [outlineView rowForItem:item];
		BOOL shouldHighlight = ([[outlineView selectedRowIndexes] containsIndex:rowIndex] && [[NSApp mainWindow] firstResponder] == outlineView);
		NSAttributedString *attributedString = [cell generatedAttributedStringWithHighlight:shouldHighlight];
		[cell setAttributedStringValue:attributedString];
	}
}

// delegate method in 10.4
- (float)outlineView:(NSOutlineView *)outlineView heightOfRowByItem:(id)item {
	float rowHeight = [outlineView rowHeight];
	
	if ([outlineView levelForItem:item] == 1) {
		rowHeight = [(PDownloadsOutlineView *)outlineView smallRowHeight];
	}
	
	return rowHeight;
}


/*- (BOOL)outlineView:(NSOutlineView *)outlineView shouldSelectTableColumn:(NSTableColumn *)tableColumn
{
	NSString *ident = [tableColumn identifier];
	if (selected_column) [outlineView setIndicatorImage:nil inTableColumn:selected_column];
	if (selected_column==tableColumn) sortAscending=!sortAscending;
	else sortAscending=YES;
	selected_column = tableColumn;
	[outlineView setHighlightedTableColumn:tableColumn];

	//NSMutableArray *source1=nil;
	//NSMutableArray *source2=nil;

	   //source1 = source;
		//source2 = source;

	if (sortAscending) {
		[outlineView setIndicatorImage:ascending inTableColumn:tableColumn];
		sorting_selector = sel_getUid([
			[NSString stringWithFormat:@"%@Asc:",ident] cString]);
		[source sortUsingSelector:sorting_selector];
		[outlineView reloadData];
		//if (source2) [source2 sortUsingSelector:sorting_selector];
	}
	else {
		[outlineView setIndicatorImage:descending inTableColumn:tableColumn];
		sorting_selector = sel_getUid([
			[NSString stringWithFormat:@"%@Desc:",ident] cString]);
		[source sortUsingSelector:sorting_selector];
		[outlineView reloadData];
		//if (source2) [source2 sortUsingSelector:sorting_selector];
	}
	return NO;
}*/

#pragma mark -
#pragma mark Download Commands

// ADDDOWNLOAD, CHGDOWNLOAD, etc.
// is quite messed up right now, sorry ;)
// i changed it so that it doesn't rely anymore on duplicated data from CHGDOWNLOAD (size,start,total...). 
// this is only temporarily to ensure that poisoned works with a future version of gift,
// which doesn't include duplicated data anymore in CHGDOWNLOAD.
// - rizzi
- (void)ADDDOWNLOAD:(NSArray *)data
{
	NSString *ticket = [data objectAtIndex:1];
	if ([ticket intValue] > 0) {	// [CHANGED] != 0?
		NSMutableDictionary *dict = [data objectAtIndex:2];
			
		[dict setObject:[NSNumber numberWithBool:YES] forKey:HDTransferSourceDownloadsPExpandableKey];
		[dict setObject:ticket forKey:HDTransferSourcePTicketKey];
		[dict setObject:[iconShop iconForFileType:[[dict objectForKey:HDTransferSourceDownloadsFileKey] pathExtension]] forKey:HDDownloadEntryPIconKey];
		[dict setObject:[NSMutableArray arrayWithObjects:
			[NSNumber numberWithBool:YES],
			[dict objectForKey:HDTransferSourceDownloadsFileKey],
			@"",
			nil]
			forKey:PZDownloadEntryFileInfoArray
		];
		[dict setObject:[NSMutableArray arrayWithObjects:
			[NSNumber numberWithBool:YES],
			@"",
			[NSString stringWithFormat:@"of %@",[self calcSize:[dict objectForKey:HDTransferSourceDownloadsSizeKey]]],
			nil]
			forKey:PZDownloadEntryFileSizeString];
			
		if ([[dict objectForKey:HDTransferSourceDownloadsStateKey] isEqualToString:@"Paused"])
			[dict setObject:[NSNumber numberWithInt:HDTransferStatusPaused] forKey:HDTransferSourcePStatusKey];
		else
			[dict setObject:[NSNumber numberWithInt:HDTransferStatusActive] forKey:HDTransferSourcePStatusKey];
		[dict setObject:[NSMutableArray array] forKey:HDTransferSourceDownloadsPSourcesKey];
		[dict setObject:[NSMutableDictionary dictionary] forKey:HDTransferSourceDownloadsPSourcesDictKey];
		
		{
			HDDownloadEntry *entry = [[HDDownloadEntry alloc] init];
			[entry setEntryDictionary:dict];
			
			[[self sourceArray] addObject:entry];
			[[self ticketsDictionary] setObject:entry forKey:ticket];
			
			[entry release];
		}
		
		[table reloadData];
		[self CHGDOWNLOAD:data];

		/* 
		if([userDefaults boolForKey:@"PAutoFMS"])
		{
		NSMutableArray *sources	= [data objectAtIndex:3];
			NSString * tmpstring;
			NSScanner *scanner = [NSScanner scannerWithString:[[[sources objectAtIndex:0] objectAtIndex:2] objectForKey:HDTransferSourceDownloadsURLKey]];
			[scanner scanUpToString:@":" intoString:&tmpstring];
			if([tmpstring isEqual:@"FastTrack"])
			{    
			NSLog(tmpstring);
		
			NSString *hash;
				if (hash=[dict objectForKey:HDTransferSourceDownloadsHashKey]) 
				{
				// if the download is active we do a find more sources
				// this seems nice to have when you start up gift and old downloads get added
					if ([[dict objectForKey:HDTransferSourcePStatusKey] intValue]==HDTransferStatusActive) 
					{
						NSLog(@"adddownload: find more sources");
						[self findMoreSources:hash];
					}
					
				NSLog(@"fms: create timer");
				// create a timer for 'auto find more sources
				NSTimer *fms_timer = [NSTimer
					scheduledTimerWithTimeInterval:80	// 10 min
					target:self
					selector:@selector(findMoreSourcesTimer:)
					userInfo:[NSDictionary dictionaryWithObjectsAndKeys:ticket,@"ticket",nil]
					repeats:YES
				];
				[timers setObject:fms_timer forKey:hash];
				}
			}
		}
		*/
	}
}


- (void)CHGDOWNLOAD:(NSArray *)data
{
	NSString *ticket = [data objectAtIndex:1];
	if (!ticket) return;
	
	HDDownloadEntry *item	= [[self ticketsDictionary] objectForKey:ticket];			// already saved item...
	NSNumber *_stateobj = [item valueForKey:HDTransferSourcePStatusKey];
	int _state = 0;
	if (_stateobj) _state=[_stateobj intValue];

	NSMutableDictionary *new	= [data objectAtIndex:2];
	NSMutableArray *sources	= [data objectAtIndex:3];
	//NSLog(@"%i",[sources count]);				// new sources...
	int i,sourcescount		= [sources count];
	
	NSString *state		= [new objectForKey:HDTransferSourceDownloadsStateKey];

	
	NSString *tmpstring;
	
	if ([[item valueForKey:HDTransferSourcePStatusKey] intValue] <= HDTransferStatusCanceled) return;	// HDTransferStatusCanceled OR HDTransferStatusCompleted
	
	//NSMutableArray *tmpsrc = [NSMutableArray array];
	//NSLog(@"%i",[tmpsrc count]);
	HDDownloadEntry *storedSource;
	HDDownloadEntry *updatedSource;
	NSMutableDictionary *sourcesDict = [item valueForKey:HDTransferSourceDownloadsPSourcesDictKey];
	for (i=0;i<sourcescount;i++) {
		updatedSource = [[sources objectAtIndex:i] objectAtIndex:2];
		storedSource  = [sourcesDict objectForKey:[updatedSource valueForKey:HDTransferSourceDownloadsURLKey]];
		if ([updatedSource valueForKey:HDTransferSourceDownloadsURLKey] && !storedSource) {
			[self ADDSOURCE:[NSArray arrayWithObjects:
				@"ADDSOURCE",
				ticket,
				updatedSource,
				nil]
			];
		}
		if ([updatedSource valueForKey:HDTransferSourceDownloadsTotalKey]) {
			[storedSource setValue:[updatedSource valueForKey:HDTransferSourceDownloadsTotalKey] forKey:HDTransferSourceDownloadsTotalKey];
		}
	}

	if (sourcescount>0) {
		NSScanner *scanner = [NSScanner scannerWithString:[[[sources objectAtIndex:0] objectAtIndex:2] objectForKey:HDTransferSourceDownloadsURLKey]];
		[scanner scanUpToString:@":" intoString:&tmpstring];
		tmpstring = [tmpstring stringByAppendingString:@" - "];
	}
	else tmpstring=@"";
	[[item valueForKey:PZDownloadEntryFileInfoArray] replaceObjectAtIndex:2 withObject:[NSString stringWithFormat:@"%@%@",tmpstring,[self stringForUsers:sourcescount]]];
	
	if ([state isEqualToString:@"Completed"])
	{ // clean up completed downloads...
		[item setValue:[NSArray arrayWithObjects:[NSNumber numberWithBool:YES],[NSNumber numberWithBool:NO],@"Completed",nil] forKey:HDTransferSourcePProgressKey];

		[self createDockBadgeIcon];
		
		[item setValue:[NSArray arrayWithObjects:[NSNumber numberWithBool:YES],@"",@"",nil] forKey:HDTransferSourceDownloadsPTransferKey];
		[[item valueForKey:PZDownloadEntryFileSizeString] replaceObjectAtIndex:1 withObject:[self calcSize:[item valueForKey:HDTransferSourceDownloadsSizeKey]]];
		[item setValue:[NSNumber numberWithInt:HDTransferStatusCompleted] forKey:HDTransferSourcePStatusKey];

		NSEnumerator *e = [sources objectEnumerator];
		id source;
		
		while (nil != (source = [e nextObject])) {
			updatedSource = [source objectAtIndex:2];
			storedSource  = [sourcesDict valueForKey:[updatedSource valueForKey:HDTransferSourceDownloadsURLKey]];
			[storedSource setValue:[NSArray arrayWithObjects:[NSNumber numberWithBool:NO],[NSNumber numberWithBool:NO],@"",nil]
				forKey:HDTransferSourcePProgressKey];
			[storedSource setValue:[NSArray arrayWithObjects:[NSNumber numberWithBool:NO],@"",nil]
				forKey:HDTransferSourceDownloadsPTransferKey];
			[storedSource setValue:[NSArray arrayWithObjects:[NSNumber numberWithBool:NO],@"",nil]
				forKey:PZDownloadEntryFileSizeString];
		}
		[table reloadItem:item reloadChildren:YES];
		
		HDPreferencesConfigurationEditorGiFT *gift_conf = [HDPreferencesConfigurationEditorGiFT singleton];
		[gift_conf read];
		
		NSString *path = [gift_conf optionValueForKey:@"completed"];
		NSString *fileWithPath = [[path stringByAppendingPathComponent:[[item valueForKey:PZDownloadEntryFileInfoArray] objectAtIndex:1]] stringByStandardizingPath];
		
		if ([HDMediaTypeDetector mediaTypeIsMusic:[[fileWithPath pathExtension] lowercaseString]])
			[iTunesController handleCompletedFile:fileWithPath];
		else
			[HDActionsController handleCompletedFile:fileWithPath];
		
		if ([[NSUserDefaults standardUserDefaults] boolForKey:@"HDIntegrationDeleteAfterImport"])
			[[NSWorkspace sharedWorkspace] noteFileSystemChanged:[gift_conf optionValueForKey:@"completed"]];

		return;
	}
	else if ([state isEqualToString:@"Paused"]) {
		if (_state!=HDTransferStatusResuming) { // if resuming... => don't display status as paused!!!
			[item setValue:[NSNumber numberWithInt:HDTransferStatusPaused] forKey:HDTransferSourcePStatusKey];
			[item setValue:[NSArray arrayWithObjects:[NSNumber numberWithBool:YES],[NSNumber numberWithBool:NO],@"Paused",nil] forKey:HDTransferSourcePProgressKey];
		}
		if (tmpstring=[new objectForKey:HDTransferSourceDownloadsTransmitKey])
			[[item valueForKey:PZDownloadEntryFileSizeString] replaceObjectAtIndex:1 withObject:[self calcSize:tmpstring]];
		[item setValue:[NSArray arrayWithObjects:[NSNumber numberWithBool:YES],@"",@"",nil] forKey:HDTransferSourceDownloadsPTransferKey];
		
		NSEnumerator *e = [sources objectEnumerator];
		id source;
		
		while (nil != (source = [e nextObject])) {
			updatedSource = [source objectAtIndex:2];
			storedSource  = [sourcesDict valueForKey:[updatedSource valueForKey:HDTransferSourceDownloadsURLKey]];
			[storedSource setValue:[NSArray arrayWithObjects:[NSNumber numberWithBool:NO],[NSNumber numberWithBool:NO],@"",nil]
				forKey:HDTransferSourcePProgressKey];
			[storedSource setValue:[NSArray arrayWithObjects:[NSNumber numberWithBool:NO],@"",nil]
				forKey:HDTransferSourceDownloadsPTransferKey];
			[storedSource setValue:[NSArray arrayWithObjects:[NSNumber numberWithBool:NO],@"",nil]
				forKey:PZDownloadEntryFileSizeString];
		}
		[table reloadItem:item reloadChildren:YES];
		return;
	}
	else {
		if (_state!=HDTransferStatusPausing) [item setValue:[NSNumber numberWithInt:HDTransferStatusActive] forKey:HDTransferSourcePStatusKey];
	}
	
	BOOL downloading=NO;
	for (i=0;i<sourcescount;i++) {
		updatedSource = [[sources objectAtIndex:i] objectAtIndex:2];
		storedSource  = [sourcesDict valueForKey:[updatedSource valueForKey:HDTransferSourceDownloadsURLKey]];
		tmpstring = [updatedSource valueForKey:HDTransferSourceDownloadsStatusKey];	//[AM][MAJOR][NOTE] change to HDTransferSourceDownloadsStateKey?
		if ([tmpstring isEqualToString:@"Active"]) {
			downloading=YES;
			[storedSource setValue:[NSArray arrayWithObjects:
					[NSNumber numberWithBool:NO],
					[NSNumber numberWithBool:YES],
					[self transmit:[updatedSource valueForKey:HDTransferSourceDownloadsTransmitKey] total:[storedSource valueForKey:HDTransferSourceDownloadsTotalKey]],nil]
				forKey:HDTransferSourcePProgressKey];
		}
		else {
			[storedSource setValue:[NSArray arrayWithObjects:
					[NSNumber numberWithBool:NO],
					[NSNumber numberWithBool:NO],
					tmpstring,nil]
				forKey:HDTransferSourcePProgressKey];
		}
		
		if (tmpstring=[updatedSource valueForKey:HDTransferSourceDownloadsTransmitKey]) {
			[storedSource setValue:[NSArray arrayWithObjects:[NSNumber numberWithBool:NO],
					[NSString stringWithFormat:@"%@ of %@",[self calcSize:tmpstring],[self calcSize:[storedSource valueForKey:HDTransferSourceDownloadsTotalKey]]],nil]
				forKey:PZDownloadEntryFileSizeString];
		}
		else {
			[storedSource setValue:[NSArray arrayWithObjects:[NSNumber numberWithBool:NO],@"",nil] forKey:PZDownloadEntryFileSizeString];
		}
		[storedSource setValue:[NSArray arrayWithObjects:[NSNumber numberWithBool:NO],@"",nil] forKey:HDTransferSourceDownloadsPTransferKey];
	}
	NSString *transmit		= [new objectForKey:HDTransferSourceDownloadsTransmitKey];
	[[item valueForKey:PZDownloadEntryFileSizeString] replaceObjectAtIndex:1 withObject:[self calcSize:transmit]];
		
	if (_state==HDTransferStatusPausing) ;
	else if (downloading) {
		NSString *size		= [item valueForKey:HDTransferSourceDownloadsSizeKey];
		NSString *transmit	= [new objectForKey:HDTransferSourceDownloadsTransmitKey];
		NSString *throughput	= [new objectForKey:@"throughput"];
		NSString *elapsed	= [new objectForKey:@"elapsed"];
		
		[item setValue:[NSArray arrayWithObjects:[NSNumber numberWithBool:YES],[NSNumber numberWithBool:YES],[self transmit:transmit total:size],nil] forKey:HDTransferSourcePProgressKey];
		[item setValue:[NSArray arrayWithObjects:[NSNumber numberWithBool:YES],[self throughput:throughput elapsed:elapsed],[self remainingWithTotalSize:size transmit:transmit throughput:throughput elapsed:elapsed],nil]
			forKey:HDTransferSourceDownloadsPTransferKey];
	}
	else {
		if ([[new objectForKey:HDTransferSourceDownloadsStateKey] isEqualToString:@"Active"]) {
			[item setValue:[NSArray arrayWithObjects:[NSNumber numberWithBool:YES],[NSNumber numberWithBool:NO],@"Waiting",nil] forKey:HDTransferSourcePProgressKey];
		}
		else {
			[item setValue:[NSArray arrayWithObjects:[NSNumber numberWithBool:YES],[NSNumber numberWithBool:NO],[new objectForKey:HDTransferSourceDownloadsStateKey],nil] forKey:HDTransferSourcePProgressKey];
		}
		[item setValue:[NSArray arrayWithObjects:[NSNumber numberWithBool:YES],@"",@"",nil]
			forKey:HDTransferSourceDownloadsPTransferKey];
	}
	if (transmit) {
		[[item valueForKey:PZDownloadEntryFileSizeString] replaceObjectAtIndex:1 withObject:[self calcSize:transmit]];
	}
	[table reloadItem:item reloadChildren:YES];
}


- (void)DELDOWNLOAD:(NSArray *)data
{
	NSString *ticket = [data objectAtIndex:1];
	if (ticket) {
		HDDownloadEntry *item = [[self ticketsDictionary] objectForKey:ticket];
		if (item) {
			NSString *hash;
			if (hash=[item valueForKey:HDTransferSourceDownloadsHashKey]) {
				//NSLog(@"fms: invalidate timer");
				// remove the timer from the timers dictionary
				//[[timers objectForKey:hash] invalidate];
				//[timers removeObjectForKey:hash];
			}
			
			if ([[item valueForKey:HDTransferSourcePStatusKey] intValue] == HDTransferStatusCompleted) {
				if ([[NSUserDefaults standardUserDefaults] boolForKey:@"HDDownloadsRemoveCompleted"]) {
					[[self sourceArray] removeObject:[[self ticketsDictionary] objectForKey:ticket]];
					[[self ticketsDictionary] removeObjectForKey:ticket];	// originally "tickets"... must have been a typo
					[table reloadData];
					[NSApp setApplicationIconImage:[NSImage imageNamed: @"NSApplicationIcon"]];
				}
			}
			else {
				if ([[NSUserDefaults standardUserDefaults] boolForKey:@"HDDownloadsRemoveCanceled"]) {
					[[self sourceArray] removeObject:[[self ticketsDictionary] objectForKey:ticket]];
					[[self ticketsDictionary] removeObjectForKey:ticket];	// ticket, not tickets, right? must have been a typo...
					[table reloadData];
				}
				else {
					[self DEL:data];
				}
			}
		}
	}
}

- (void)ADDSOURCE:(NSArray *)data
{
	NSString *ticket 		= [data objectAtIndex:1];
	HDDownloadEntry *new    = [data objectAtIndex:2];
	if (ticket && new) {
		// getting the download where the source needs to be added
		HDDownloadEntry *item = [[self ticketsDictionary] objectForKey:ticket];
		if (item) {
			// the sources of the download
			NSMutableDictionary *sourcesDict = [item valueForKey:HDTransferSourceDownloadsPSourcesDictKey];
			NSMutableArray *sourcesArray = [item valueForKey:HDTransferSourceDownloadsPSourcesKey];

			// make sure that the source is not already there
			if (![sourcesDict objectForKey:[new valueForKey:HDTransferSourceDownloadsURLKey]]) {
				[new setValue:[NSArray arrayWithObjects:[NSNumber numberWithBool:NO],[new valueForKey:HDTransferSourceDownloadsUserKey],nil] forKey:PZDownloadEntryFileInfoArray];
				[new setValue:[item valueForKey:HDTransferSourcePTicketKey] forKey:HDTransferSourcePTicketKey];
				
				[new setValue:[NSNumber numberWithBool:NO] forKey:HDTransferSourceDownloadsPExpandableKey];	// [TODO] is this the right place? [AM]
				
				[sourcesDict setObject:new forKey:[new valueForKey:HDTransferSourceDownloadsURLKey]];
				[sourcesArray addObject:new];
				
				// users count
				NSString *tmpstring;
				int count = [sourcesArray count];
				if (count>0) {
					NSScanner *scanner = [NSScanner scannerWithString:[[sourcesArray objectAtIndex:0] objectForKey:HDTransferSourceDownloadsURLKey]];
					[scanner scanUpToString:@":" intoString:&tmpstring];
					tmpstring = [tmpstring stringByAppendingString:@" - "];
				}
				else tmpstring=@"";
				[[[[self ticketsDictionary] objectForKey:ticket] valueForKey:PZDownloadEntryFileInfoArray] replaceObjectAtIndex:2 withObject:[NSString stringWithFormat:@"%@%@",tmpstring,[self stringForUsers:count]]];

				[table reloadItem:item reloadChildren:YES];
			}
		}
	}
}

- (void)DELSOURCE:(NSArray *)data
{
	NSString *ticket = [data objectAtIndex:1];
	NSString *url = [[data objectAtIndex:2] objectForKey:HDTransferSourceDownloadsURLKey];
	if (ticket && url) {
		HDDownloadEntry *dictionaryForTicket = [[self ticketsDictionary] objectForKey:ticket];
		if (dictionaryForTicket) {
			NSMutableArray *sourcesArray = [dictionaryForTicket valueForKey:HDTransferSourceDownloadsPSourcesKey];
			NSMutableDictionary *sourcesDict = [dictionaryForTicket valueForKey:HDTransferSourceDownloadsPSourcesDictKey];
			int count = [sourcesArray count];

			NSString *tmpstring;
			if (count>0) {
				NSScanner *scanner = [NSScanner scannerWithString:[[sourcesArray objectAtIndex:0] valueForKey:HDTransferSourceDownloadsURLKey]];
				[scanner scanUpToString:@":" intoString:&tmpstring];
				tmpstring = [tmpstring stringByAppendingString:@" - "];
			}
			else tmpstring=@"";

			[sourcesArray removeObject:[sourcesDict objectForKey:url]];
			[sourcesDict removeObjectForKey:url];

			count = [sourcesArray count];
			[[dictionaryForTicket valueForKey:PZDownloadEntryFileInfoArray] replaceObjectAtIndex:2 withObject:[NSString stringWithFormat:@"%@%@",tmpstring,[self stringForUsers:count]]];

			[table reloadItem:dictionaryForTicket reloadChildren:YES];
		}
	}
}

- (void)DEL:(NSArray *)data
{
	[super DEL:data];

	NSString *ticket = [data objectAtIndex:1];
	if (ticket) {
		HDTransferSourceEntry *entry = [[self ticketsDictionary] objectForKey:ticket];
		if ([[entry valueForKey:@"PStatus"] intValue] != HDTransferStatusCompleted) {
			// download was cancelled...
			
			NSNumber *yesValue = [NSNumber numberWithBool:YES];
			NSNumber *noValue = [NSNumber numberWithBool:NO];
						
			[entry setValue:[NSNumber numberWithInt:HDTransferStatusCanceled] forKey:@"PStatus"];
			[entry setValue:[NSArray arrayWithObjects:yesValue,[NSNumber numberWithBool:NO],@"Cancelled",nil] forKey:@"PProgress"];
			[entry setValue:[NSArray arrayWithObjects:yesValue,@"",@"",nil] forKey:@"PTransfer"];
			[entry setValue:[NSArray arrayWithObjects:yesValue,@"",@"",nil] forKey:@"PSize"];
			
			NSMutableArray *sources = [entry valueForKey:@"PSources"];
			NSEnumerator *e = [sources objectEnumerator];
			HDTransferSourceEntry *tmp;
			
			while (nil != (tmp = [e nextObject])) {
				[tmp setValue:[NSArray arrayWithObjects:noValue,[NSNumber numberWithBool:NO],@"",nil] forKey:@"PProgress"];
				[tmp setValue:[NSArray arrayWithObjects:noValue,@"",nil] forKey:@"PTransfer"];
				[tmp setValue:[NSArray arrayWithObjects:noValue,@"",nil] forKey:@"PSize"];
			}
			[self reloadTableData:(void *)entry];
		}
	}
}

@end

#pragma mark -

@implementation HDTransferSourceDownloads (OverriddenMethods)

- (void)disconnected
{
	// invalidate all timers
	/*NSEnumerator *keyEnumerator = [timers keyEnumerator];
	id key;
	while (key = [keyEnumerator nextObject]) {
		[[timers objectForKey:key] invalidate];
		[timers removeObjectForKey:key];
	}*/
	
	[super disconnected];
	[table reloadData];
}

- (void)cleanUp:(id)sender {
	[NSApp setApplicationIconImage:[NSImage imageNamed:@"NSApplicationIcon"]];
	[super cleanUp:sender];
	[table reloadData];
}


- (void)reloadTableData:(void *)contextInfo {
	[table reloadItem:(id)contextInfo reloadChildren:YES];
}

@end