//
//  PComparableDictionary.m
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import "PComparableDictionary.h"


@implementation NSArray (PComparableDictionary)

// SORT BY EXTENSION
- (NSComparisonResult)iconAsc:(NSArray *)dict
{
	// used by HDSearchResultEntry...
	NSString *leftExt = [[[self objectAtIndex:0] objectForKey:@"file"] pathExtension];
	NSString *rightExt = [[[dict objectAtIndex:0] objectForKey:@"file"] pathExtension];
	
	return [leftExt compare:rightExt options:NSCaseInsensitiveSearch];
}

- (NSComparisonResult)iconDesc:(NSArray *)dict
{
	return (-1*[self iconAsc:dict]);
}


// SORT BY FILE NAME
- (NSComparisonResult)fileAsc:(NSArray *)dict
{
	NSString *left = [[self objectAtIndex:0] objectForKey:@"file"];
	NSString *right = [[dict objectAtIndex:0] objectForKey:@"file"];

	return [left localizedCaseInsensitiveCompare:right];
}

- (NSComparisonResult)fileDesc:(NSArray *)dict
{
	return (-1*[self fileAsc:dict]);
}

// SORT BY ARTIST NAME
- (NSComparisonResult)artistAsc:(NSArray *)dict
{
	NSString *str1 = [[self objectAtIndex:0] objectForKey:@"artist"];
	NSString *str2 = [[dict objectAtIndex:0] objectForKey:@"artist"];
	if (str1==nil && str2==nil) return NSOrderedSame;
	
	return [str1 localizedCaseInsensitiveCompare:str2];
}

- (NSComparisonResult)artistDesc:(NSArray *)dict
{
	NSString *str1 = [[self objectAtIndex:0] objectForKey:@"artist"];
	NSString *str2 = [[dict objectAtIndex:0] objectForKey:@"artist"];
	if (str1==nil && str2==nil) return NSOrderedSame;
	
	return [str2 localizedCaseInsensitiveCompare:str1];
}

// SORT BY ALBUM NAME
- (NSComparisonResult)albumAsc:(NSArray *)dict
{
	NSString *str1 = [[self objectAtIndex:0] objectForKey:@"album"];
	NSString *str2 = [[dict objectAtIndex:0] objectForKey:@"album"];
	if (str1==nil && str2==nil) return NSOrderedSame;
	
	return [str1 localizedCaseInsensitiveCompare:str2];
}

- (NSComparisonResult)albumDesc:(NSArray *)dict
{
	NSString *str1 = [[self objectAtIndex:0] objectForKey:@"album"];
	NSString *str2 = [[dict objectAtIndex:0] objectForKey:@"album"];
	if (str1==nil && str2==nil) return NSOrderedSame;
	
	return [str2 localizedCaseInsensitiveCompare:str1];
}


// [AM][TODO] THIS IS HORRIBLE!!!  What idiot wanted to check the class of a string?!  It's an implementation detail, not an interface one.
// SORT BY USERNAME
- (NSComparisonResult)userAsc:(NSArray *)dict
{
	if ([NSStringFromClass([[[self objectAtIndex:0] objectForKey:@"user"] class]) isEqualToString:@"NSCFString"] && [NSStringFromClass([[[dict objectAtIndex:0] objectForKey:@"user"] class]) isEqualToString:@"NSCFString"]) {
		return [[[self objectAtIndex:0] objectForKey:@"user"] compare:[[dict objectAtIndex:0] objectForKey:@"user"] options:NSCaseInsensitiveSearch];
	}
	else if ([NSStringFromClass([[[self objectAtIndex:0] objectForKey:@"user"] class]) isEqualToString:@"NSCFString"]) {
		return NSOrderedAscending;
	}
	else if ([NSStringFromClass([[[dict objectAtIndex:0] objectForKey:@"user"] class]) isEqualToString:@"NSCFString"]) {
		return NSOrderedDescending;
	}
	else return [[NSNumber numberWithInt:[[[[self objectAtIndex:0] objectForKey:@"user"] string] intValue]] compare:[NSNumber numberWithInt:[[[[dict objectAtIndex:0] objectForKey:@"user"] string] intValue]]];
}

- (NSComparisonResult)userDesc:(NSArray *)dict
{
	return (-1*[self userAsc:dict]);
}


// SORT BY SIZE
- (NSComparisonResult)calcsizeAsc:(NSArray *)dict
{
	// used in Search aspect controller
	NSString *left = [[self objectAtIndex:0] objectForKey:@"size"];
	NSString *right = [[dict objectAtIndex:0] objectForKey:@"size"];
	
	return [left compare:right options:NSNumericSearch];
	
	/*int li = [left length];
	int ri = [right length];
	if (li>ri) return NSOrderedDescending;
	else if (ri>li) return NSOrderedAscending;
	else return [left compare:right];*/
}

- (NSComparisonResult)calcsizeDesc:(NSArray *)dict
{
	return (-1*[self calcsizeAsc:dict]);
}

// SORT BY BIT RATE
- (NSComparisonResult)bitrateAsc:(NSArray *)dict
{
	// used in HDSearchResultSource
	NSString *left = [[self objectAtIndex:0] objectForKey:@"bitrate"];
	NSString *right = [[dict objectAtIndex:0] objectForKey:@"bitrate"];

	return [left compare:right options:NSNumericSearch];

	/*int l = [[[self objectAtIndex:0] objectForKey:@"bitrate"] intValue];
	int r = [[[dict objectAtIndex:0] objectForKey:@"bitrate"] intValue];
	if (l==0 && r==0) return NSOrderedSame;
	else if (l==0) return NSOrderedDescending;
	else if (r==0) return NSOrderedAscending;
	else if (l==r) return NSOrderedSame;
	else if (l<r) return NSOrderedAscending;
	else return NSOrderedDescending;*/
}

- (NSComparisonResult)bitrateDesc:(NSArray *)dict
{
	// used in HDSearchResultSource
	NSString *left = [[self objectAtIndex:0] objectForKey:@"bitrate"];
	NSString *right = [[dict objectAtIndex:0] objectForKey:@"bitrate"];

	return [right compare:left options:NSNumericSearch];

	/*int l = [[[self objectAtIndex:0] objectForKey:@"bitrate"] intValue];
	int r = [[[dict objectAtIndex:0] objectForKey:@"bitrate"] intValue];
	if (l==0 && r==0) return NSOrderedSame;
	else if (l==0) return NSOrderedDescending;
	else if (r==0) return NSOrderedAscending;
	else if (l==r) return NSOrderedSame;
	else if (l<r) return NSOrderedDescending;
	else return NSOrderedAscending;*/
}

- (NSComparisonResult)PProtoIconAsc:(NSArray *)dict
{
	// used in HDSearchResultSource

	NSString *left = [[self objectAtIndex:0] objectForKey:@"PProto"];
	NSString *right = [[dict objectAtIndex:0] objectForKey:@"PProto"];

	return [left compare:right options:NSCaseInsensitiveSearch];
}

- (NSComparisonResult)PProtoIconDesc:(NSArray *)dict
{
	return (-1*[self PProtoIconAsc:dict]);
}

// SORT BY BIT AVAILABILITY
- (NSComparisonResult)PAvailabilityAsc:(NSArray *)dict
{
	// used by HDSearchResultSource
	NSString *left = [[self objectAtIndex:0] objectForKey:@"PAvail"];
	NSString *right = [[dict objectAtIndex:0] objectForKey:@"PAvail"];
	
	return [left compare:right options:NSCaseInsensitiveSearch];
}

- (NSComparisonResult)PAvailabilityDesc:(NSArray *)dict
{
	return (-1*[self PAvailabilityAsc:dict]);
}

// SORT BY SIZE
- (NSComparisonResult)PSizeAsc:(NSArray *)dict
{
	NSString *left = [[self objectAtIndex:0] objectForKey:@"size"];
	NSString *right = [[dict objectAtIndex:0] objectForKey:@"size"];
	
	return [left compare:right options:NSNumericSearch];
	
	/*int li = [left length];
	int ri = [right length];
	if (li>ri) return NSOrderedDescending;
	else if (ri>li) return NSOrderedAscending;
	else return [left compare:right];*/
}

- (NSComparisonResult)PSizeDesc:(NSArray *)dict
{
	return (-1*[self PSizeAsc:dict]);
}

/* sort by duration */
- (NSComparisonResult)formattedDurationAsc:(NSArray *)dict
{
	NSString *str1 = [[self objectAtIndex:0] objectForKey:@"duration"];
	NSString *str2 = [[dict objectAtIndex:0] objectForKey:@"duration"];
	
	return [str1 compare:str2 options:NSNumericSearch];

	/*int l = [[[self objectAtIndex:0] objectForKey:@"duration"] intValue];
	int r = [[[dict objectAtIndex:0] objectForKey:@"duration"] intValue];
	if (l==0 && r==0) return NSOrderedSame;
	else if (l==0) return NSOrderedDescending;
	else if (r==0) return NSOrderedAscending;
	else if (l==r) return NSOrderedSame;
	else if (l<r) return NSOrderedAscending;
	else return NSOrderedDescending;*/
}

- (NSComparisonResult)formattedDurationDesc:(NSArray *)dict
{
	NSString *l = [[self objectAtIndex:0] objectForKey:@"duration"];
	NSString *r = [[dict objectAtIndex:0] objectForKey:@"duration"];
	return [r compare:l options:NSNumericSearch];

	/*int l = [[[self objectAtIndex:0] objectForKey:@"duration"] intValue];
	int r = [[[dict objectAtIndex:0] objectForKey:@"duration"] intValue];
	if (l==0 && r==0) return NSOrderedSame;
	else if (l==0) return NSOrderedDescending;
	else if (r==0) return NSOrderedAscending;
	else if (l==r) return NSOrderedSame;
	else if (l<r) return NSOrderedDescending;
	else return NSOrderedAscending;*/
}

// SORT BY EXTENSION
- (NSComparisonResult)PIconAsc:(NSArray *)dict
{
	NSString *str1 = [[[self objectAtIndex:0] objectForKey:@"file"] pathExtension];
	NSString *str2 = [[[dict objectAtIndex:0] objectForKey:@"file"] pathExtension];

	return [str1 compare:str2 options:NSCaseInsensitiveSearch];
}

- (NSComparisonResult)PIconDesc:(NSArray *)dict
{
	return (-1*[self PIconAsc:dict]);
}

/*- (NSComparisonResult)PFileUserAsc:(NSArray *)dict
{

}

- (NSComparisonResult)PFileUserDesc:(NSArray *)dict
{

}*/

@end
