//
// PPrefDownloads.m
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import "HDPreferencesAspectDownloads.h"
#import "HDPreferencesConfigurationEditorGiFT.h"


@implementation HDPreferencesAspectDownloads

- (id)init {
	if (nil != (self = [super init])) {
		if (![NSBundle loadNibNamed:@"PreferencePaneDownloads" owner:self]) {
			[self release];
			self = nil;
		}
	}
	return self;
}

- (void)awakeFromNib
{
	gift_conf = [HDPreferencesConfigurationEditorGiFT singleton];    
	
	[self readConfFiles];

	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(noteConfigurationFilesChanged:) name:@"PUpdateFromConfFiles" object:nil];
}

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[super dealloc];
}

- (void)enable
{
	[incoming setEnabled:YES];
	[completed setEnabled:YES];
}

- (void)disable
{
	[incoming setEnabled:NO];
	[completed setEnabled:NO];
}

- (void)noteConfigurationFilesChanged:(NSNotification *)aNotification {
	[self readConfFiles];
}

- (void)readConfFiles
{
	if (gift_conf = [HDPreferencesConfigurationEditorGiFT singleton]) {
		[self enable];
		
		[gift_conf read];
		NSString *incomingPath = [gift_conf optionValueForKey:@"incoming"];
		NSString *completedPath = [gift_conf optionValueForKey:@"completed"];
		// [TODO] what on earth?  why are we checking _in for a _comp manipulation, and _comp for an _in manipulation...? [AM]
		if (completedPath) [completed setStringValue:completedPath];	//if (incomingPath)
		if (incomingPath) [incoming setStringValue:incomingPath];	// if (completedPath)
	}
	else {
		[self disable];
		return;
	}
}

- (IBAction)completedChanged:(id)sender
{
	//NSDictionary *fattrs=[[NSFileManager defaultManager] fileAttributesAtPath:[sender stringValue]];

	[gift_conf setOptionValue:[[sender stringValue] stringByAbbreviatingWithTildeInPath] forKey:@"completed"];
	[gift_conf setOptionValue:[[[sender stringValue] stringByAbbreviatingWithTildeInPath] stringByAppendingPathComponent:@".incoming"] forKey:@"incoming"];
	[self readConfFiles];
	[[NSNotificationCenter defaultCenter] postNotificationName:@"PCompletedDirectoryChanged" object:self userInfo:nil];
}

- (IBAction)incomingChanged:(id)sender
{
	// removed this and incoming is now always going to be derived from the completed folder.
	// this was because some users were setting the incoming folder to their desktop. this is dangerous
	// because when the incoming folder becomes corrupted the solution is to delete that folder.
	// in any reguard a user should never need to change this. It's just too destructive.
	
	//[gift_conf setOptionValue:[[sender stringValue] stringByAbbreviatingWithTildeInPath] forKey:@"incoming"];
	//[self readConfFiles];
}

- (IBAction)browseIncoming:(id)sender
{
	/*NSOpenPanel *open = [NSOpenPanel openPanel];
	[open setCanChooseFiles:NO];
	[open setCanChooseDirectories:YES];
	int ret = [open 
		runModalForDirectory:NSHomeDirectory()
		file:nil
		types:nil
	];
	if (ret==NSOKButton) {
		[gift_conf setOptionValue:[[[open filenames] objectAtIndex:0] stringByAbbreviatingWithTildeInPath] forKey:@"incoming"];
		[self readConfFiles];
	}*/
}

- (IBAction)browseCompleted:(id)sender
{
	NSOpenPanel *open = [NSOpenPanel openPanel];
	[open setCanChooseFiles:NO];
	[open setCanChooseDirectories:YES];
	int ret = [open 
		runModalForDirectory:NSHomeDirectory()
		file:nil
		types:nil
	];
	if (ret==NSOKButton) {
		[gift_conf setOptionValue:[[[open filenames] objectAtIndex:0] stringByAbbreviatingWithTildeInPath] forKey:@"completed"];
		[gift_conf setOptionValue:[[gift_conf optionValueForKey:@"completed"] stringByAppendingPathComponent:@".incoming"] forKey:@"incoming"];
		[self readConfFiles];
		[[NSNotificationCenter defaultCenter] postNotificationName:@"PCompletedDirectoryChanged" object:self userInfo:nil];
	}
}

@end
