//
// PPrefDownloads.m
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import "HDPreferencesAspectUploads.h"
#import "HDAdditions.h"
#import "HDPreferencesConfigurationEditorGiFT.h"
#import "HDValueTransformers.h"
#import "AMTableView.h"
#import "AMTableDataSourceArrayController.h"


@implementation HDPreferencesAspectUploads

+ (void)initialize {
	NSValueTransformer *vt;
	
	vt = [[HDUploadPreferencesIconValueTransformer alloc] init];
	[NSValueTransformer setValueTransformer:vt forName:@"UploadPreferencesIconValueTransformer"];
	[vt release];
}

- (id)init {
	if (nil != (self = [super init])) {
		[self setInterfaceEnabled:NO];
	
		[self setShareSourceArray:[NSArray array]];
		
		pathSortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"path" ascending:YES];
		
		if (![NSBundle loadNibNamed:@"PreferencePaneUploads" owner:self]) {
			[self release];
			self = nil;
		}
	}
	return self;
}

- (void)dealloc
{
	[[NSNotificationCenter defaultCenter] removeObserver:self];

	[arrayController release];
	arrayController = nil;

	[pathSortDescriptor release];
	pathSortDescriptor = nil;

	[self setShareSourceArray:nil];
	
	[super dealloc];
}

- (void)awakeFromNib
{
	[shareTable setDeleteKeyRemoves:YES];

	[shareTable setSortDescriptors:[NSArray arrayWithObject:pathSortDescriptor]];
	//NSSortDescriptor *sortDescriptor = [[NSSortDescriptor alloc] initWithKey:@"path" ascending:YES];
	//[arrayController setSortDescriptors:[NSArray arrayWithObject:sortDescriptor]];
	//[sortDescriptor release];
		
	share_completed = NO;
	//NSArray *cols = [shareTable tableColumns];
	/*int i, count=[cols count];
	for (i=0;i<count;i++) [[[cols objectAtIndex:i] dataCell] setDrawsBackground:NO];*/
	//[[shareTable tableColumnWithIdentifier:@"icon"] setDataCell:[[[NSImageCell alloc] init] autorelease]];


	//userDefaults = [NSUserDefaults standardUserDefaults];
	
	[self readConfFiles];

	/*if ([userDefaults boolForKey:@"HDUploadsRemoveCompleted"]) [removeCompleted setState:NSOnState];
	else [removeCompleted setState:NSOffState];
	
	if ([userDefaults boolForKey:@"HDUploadsRemoveCanceled"]) [removeCancelled setState:NSOnState];
	else [removeCancelled setState:NSOffState];*/

	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(completedDirectoryChanged:) name:@"PCompletedDirectoryChanged" object:nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(noteConfigurationFilesUpdated:) name:@"PUpdateFromConfFiles" object:nil];



	[[NSNotificationCenter defaultCenter] addObserver:self
		selector:@selector(noteShareAdded:)
		name:AMTableDataSourceAddedObjectNotification
		object:arrayController];
		
	[[NSNotificationCenter defaultCenter] addObserver:self
		selector:@selector(noteShareRemoved:)
		name:AMTableDataSourceRemovedObjectNotification
		object:arrayController];
}

- (void)completedDirectoryChanged:(NSNotification *)notification
{
	[self readConfFiles];
}

- (void)noteConfigurationFilesUpdated:(NSNotification *)aNotification {
	[self readConfFiles];
}

- (void)readConfFiles
{
	if (gift_conf=[HDPreferencesConfigurationEditorGiFT singleton]) {
		[self setInterfaceEnabled:YES];

		[gift_conf read];
		
		//[shareSource autorelease];
		// this enumeration is a hack to convert a path dictionary to or from a format employed by gift_conf
		NSMutableArray *tmp = [[NSMutableArray alloc] init];
		NSEnumerator *e = [[gift_conf optionValueForKey:@"root"] objectEnumerator];
		id rootObj;
		
		while (rootObj = [e nextObject]) {
			[tmp addObject:[NSDictionary dictionaryWithObject:rootObj forKey:@"path"]];
		}
		
		[self setShareSourceArray:tmp];
		[tmp release];
		
		NSMutableArray *shareSourceArray = [self shareSourceArray];
		
		if ([shareSourceArray count]==1 && [(NSString *)[[shareSourceArray objectAtIndex:0] objectForKey:@"path"] length]==0) {
			[self willChangeValueForKey:@"shareSourceArray"];
			[shareSourceArray removeObjectAtIndex:0];
			[self didChangeValueForKey:@"shareSourceArray"];
		}
		
		share_completed = NO;
		if ([[gift_conf optionValueForKey:@"share_completed"] intValue]) {
			share_completed = YES;
			/*[self willChangeValueForKey:@"shareSourceArray"];
			[shareSourceArray insertObject:
				[NSDictionary dictionaryWithObject:[gift_conf optionValueForKey:@"completed"]
					forKey:@"path"]
					atIndex:0
			];
			[self didChangeValueForKey:@"shareSourceArray"];*/
			
			(void)[arrayController setSelectedObjects:nil];
		}
		
		[maxUploads setIntValue:[[gift_conf optionValueForKey:@"max_uploads"] intValue]];
		[maxPerUser setIntValue:[[gift_conf optionValueForKey:@"max_peruser_uploads"] intValue]];
		[maxUpstream setIntValue:[[gift_conf optionValueForKey:@"upstream"] intValue]/HD_KILOBYTE];
		[maxDownstream setIntValue:[[gift_conf optionValueForKey:@"downstream"] intValue]/HD_KILOBYTE];
	}
	else {
		[self setInterfaceEnabled:NO];
		[self willChangeValueForKey:@"shareSourceArray"];
		[[self shareSourceArray] removeAllObjects];
		[self didChangeValueForKey:@"shareSourceArray"];
	}
		
	[shareTable reloadData];	// is this necessary?
}

- (void)noteShareAdded:(NSNotification *)aNotification {
	NSMutableArray *shareSourceArray = [self shareSourceArray];

	// this enumeration is a hack to convert a path dictionary to or from a format employed by gift_conf
	NSMutableArray *tmp = [[NSMutableArray alloc] initWithCapacity:[shareSourceArray count]];
	NSEnumerator *e = [shareSourceArray objectEnumerator];
	id shareSource;
	
	while (shareSource = [e nextObject]) {
		[tmp addObject:[shareSource objectForKey:@"path"]]; 
	}
	
	//if (share_completed) [tmp removeObjectAtIndex:0];
	[gift_conf setOptionValue:tmp forKey:@"root"];
	[tmp release];
	[self readConfFiles];
	[[NSNotificationCenter defaultCenter] // update the shares
		postNotificationName:@"HDDaemonCommanderNotification" 
		object:self
		userInfo:[NSDictionary dictionaryWithObjectsAndKeys:@"SHARE action(sync)",@"cmd",nil]
	];
	
}

- (void)noteShareRemoved:(NSNotification *)aNotification {
	NSMutableArray *shareSourceArray = [self shareSourceArray];
	
	// this enumeration is a hack to convert a path dictionary to or from a format employed by gift_conf
		NSMutableArray *tmp = [[NSMutableArray alloc] initWithCapacity:[shareSourceArray count]];
		NSEnumerator *e = [shareSourceArray objectEnumerator];
		id shareSource;

		while (shareSource = [e nextObject]) {
			[tmp addObject:[shareSource objectForKey:@"path"]]; 
		}

	//if (share_completed) [tmp removeObjectAtIndex:0];
	[gift_conf setOptionValue:tmp forKey:@"root"];
	[tmp release];
	[self readConfFiles];
	[[NSNotificationCenter defaultCenter] // update the shares
		postNotificationName:@"HDDaemonCommanderNotification" 
		object:self
		userInfo:[NSDictionary dictionaryWithObjectsAndKeys:@"SHARE action(sync)",@"cmd",nil]
	];
}

- (IBAction)maxUploadsChanged:(id)sender
{
	[gift_conf setOptionValue:[NSNumber numberWithInt:[sender intValue]] forKey:@"max_uploads"];
	[self readConfFiles];
}

- (IBAction)maxPerUserChanged:(id)sender
{
	[gift_conf setOptionValue:[NSNumber numberWithInt:[sender intValue]] forKey:@"max_peruser_uploads"];
	[self readConfFiles];
}

- (IBAction)maxUploadSpeedChanged:(id)sender
{
	[gift_conf setOptionValue:[NSNumber numberWithInt:[sender intValue]*HD_KILOBYTE] forKey:@"upstream"];
	[self readConfFiles];
}

- (IBAction)maxDownloadSpeedChanged:(id)sender
{
	[gift_conf setOptionValue:[NSNumber numberWithInt:[sender intValue]*HD_KILOBYTE] forKey:@"downstream"];
	[self readConfFiles];
}


// ---------------------------------------------------------------

/*- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
	BOOL shouldBeEnabled = NO;

	if ([shareTable selectedRow]!=-1) {
		shouldBeEnabled = YES;
	}
	
	[removeButton setEnabled:shouldBeEnabled];
}*/

/*- (BOOL)tableView:(NSTableView *)aTableView shouldSelectRow:(int)rowIndex
{
	BOOL shouldSelectRow = YES;

	if (share_completed && rowIndex==0) {
		shouldSelectRow = NO;
	}
	
	return shouldSelectRow;
}*/

/*- (BOOL)control:(NSControl *)control isValidObject:(id)object
{
	BOOL isValidObject = NO;

	if (control==maxUploads) {
		if ([maxUploads intValue]>=-1) {
			isValidObject = YES;
		}
	}
	else {
		if ([maxPerUser intValue]>=1) {
			isValidObject = YES;
		}
	}
	
	return isValidObject;
}*/

/*- (NSArray *)pathSortDescriptors {
	return [NSArray arrayWithObject:pathSortDescriptor];
}

- (void)setPathSortDescriptors:(NSArray *)anArray {
	// do nothing
}*/

@end

@implementation HDPreferencesAspectUploads (Accessors)

- (NSMutableArray *)shareSourceArray {
	return [[AM_shareSourceArray retain] autorelease];
}

- (void)setShareSourceArray:(NSArray *)anArray {
	if (anArray != AM_shareSourceArray) {
		[AM_shareSourceArray release];
		AM_shareSourceArray = [anArray mutableCopy];
	}
}

- (BOOL)isInterfaceEnabled {
	return AM_interfaceEnabled;
}

- (void)setInterfaceEnabled:(BOOL)flag {
	AM_interfaceEnabled = flag;
}

@end
