//
//  HDPreferencesConfigurationEditor.m
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import "HDPreferencesConfigurationEditor.h"
#import "HDAdditions.h"


@implementation HDPreferencesConfigurationEditor

+ (NSString *)pathToDaemon
{
	NSString *path;

	if ([[NSUserDefaults standardUserDefaults] boolForKey:@"HDDaemonUseCustom"]) {
		path = [NSHomeDirectory() stringByAppendingPathComponent:@".giFT"];
	}
	else {
		path = HDApplicationSupportDirectory();
	}
	
	return path;
}

- (id)init {
	if (nil != (self = [super init])) {
		[self setConf:[NSDictionary dictionary]];
		[self setLines:[NSArray array]];
	}
	return self;
}

- (void)dealloc {
	[self setLines:nil];
	[self setConf:nil];
	
	[super dealloc];
}

- (NSString *)path {
	return @"";
}

- (void)read
{
	NSString *content = [[NSString alloc] initWithContentsOfFile:[self path] encoding:NSUTF8StringEncoding error:NULL];
	[self setLines:[content componentsSeparatedByString:@"\n"]];
	[content release];
	
	NSMutableArray *lines = [self lines];
	
	int i,count = [lines count];
	NSString *theLine;
	for (i=0;i<count;i++) {
		theLine = [self skipWhitespace:[lines objectAtIndex:i]];
		[lines replaceObjectAtIndex:i withObject:theLine];
		[self readConfLine:i];
	}
}

- (NSString *)skipWhitespace:(NSString *)theLine
{
	while ([theLine hasPrefix:@" "]) theLine = [theLine substringFromIndex:1];
	return theLine;
}

- (void)readConfLine:(int)index
{
	NSString *line = [[self lines] objectAtIndex:index];
	if ([line length] > 0 && ![line hasPrefix:@"#"] && ![line hasPrefix:@"["]) {
		NSArray *split = [line componentsSeparatedByString:@"="];
		NSString *key = [[split objectAtIndex:0] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
		NSString *value = [split objectAtIndex:1];
		
		id indexedObject = nil;
		
		if ([int_confs containsObject:key]) {
			indexedObject = [NSNumber numberWithInt:[value intValue]];
			/*[[self conf] setObject:[NSMutableArray arrayWithObjects:
				[NSNumber numberWithInt:index],[NSNumber numberWithInt:[value intValue]],nil]
				forKey:key];*/
		}
		else if ([string_confs containsObject:key]) {
			// [not relevant post-OS-X-10.2][AM] -- add space before trimming because of a bug in NSString's -stringByTrimmingCharactersInSet
			//NSString *spaceAddedString = [value stringByAppendingString:@" "];
			indexedObject = [value stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
			//NSMutableArray *array = [NSMutableArray arrayWithObjects:[NSNumber numberWithInt:index],trimmedString,nil];
			//[[self conf] setObject:array forKey:key];
		}
		else if ([colon_confs containsObject:key]) {
			indexedObject = [self splitString:value withDelimiter:@":"];
			/*[[self conf] setObject:[NSMutableArray arrayWithObjects:
				[NSNumber numberWithInt:index],[self splitString:value withDelimiter:@":"],nil]
				forKey:key];*/
		}
		else if ([space_confs containsObject:key]) {
			indexedObject = [self splitString:value withDelimiter:@" "];
			/*[[self conf] setObject:[NSMutableArray arrayWithObjects:
				[NSNumber numberWithInt:index],[self splitString:value withDelimiter:@" "],nil]
				forKey:key];*/
		}
		
		if (indexedObject) {
			NSMutableArray *indexedArray = [[NSMutableArray alloc] initWithObjects:
				[NSNumber numberWithInt:index],
				indexedObject,
				nil];
			[[self conf] setObject:indexedArray forKey:key];
			[indexedArray release];
		}
	}
}

- (id)optionValueForKey:(NSString *)option
{
	if (![[self conf] objectForKey:option]) return nil;
	return [[[self conf] objectForKey:option] objectAtIndex:1];
}

- (void)setOptionValue:(id)value forKey:(NSString *)key
{
	NSString *path = [self path];
	[self read];
	if ([[self conf] objectForKey:key]) {
		[[[self conf] objectForKey:key] replaceObjectAtIndex:1 withObject:value];
		[self updateArrayForKey:key];
		
		NSFileManager *fileManager = [NSFileManager defaultManager];
		
		NSDictionary *attr = [fileManager fileAttributesAtPath:path traverseLink:NO];
		[fileManager removeFileAtPath:path handler:nil];
		[fileManager createFileAtPath:path contents:nil attributes:attr];
		NSFileHandle *handle = [NSFileHandle fileHandleForWritingAtPath:path];
		[handle writeData:[[[self lines] componentsJoinedByString:@"\n"] dataUsingEncoding:NSASCIIStringEncoding]];
		[handle closeFile];
	}
}

- (void)updateArrayForKey:(NSString *)key
{
	BOOL shouldReplaceValueInLines = YES;
	
	NSString *value;
	NSArray *array = [[self conf] objectForKey:key];
	id firstObject = [array objectAtIndex:1];
	
	if ([int_confs containsObject:key]) {
		value = [firstObject stringValue];
	}
	else if ([string_confs containsObject:key]) {
		value = firstObject;
	}
	else if ([colon_confs containsObject:key]) {
		value = [firstObject componentsJoinedByString:@":"];
	}
	else if ([space_confs containsObject:key]) {
		value = [firstObject componentsJoinedByString:@" "];
	}
	else {
		shouldReplaceValueInLines = NO;
	}
	
	if (shouldReplaceValueInLines) {
		NSString *keyValString = [[NSString alloc] initWithFormat:@"%@ = %@", key, value];
		int idx = [[array objectAtIndex:0] intValue];
		[[self lines] replaceObjectAtIndex:idx withObject:keyValString];
		[keyValString release];
	}
}

/*- (NSArray *)colonToArray:(NSString *)string
{
	NSMutableArray *split = [[[string componentsSeparatedByString:@":"] mutableCopy] autorelease];
	int i, count=[split count];
	for (i=0;i<count;i++) {
		[split replaceObjectAtIndex:i withObject:
			[[[split objectAtIndex:i] stringByAppendingString:@" "] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]]];
	}
	return split;
}*/

/*- (NSArray *)spaceToArray:(NSString *)string
{
    NSMutableArray *split = [[[string componentsSeparatedByString:@" "] mutableCopy] autorelease];
    int i, count=[split count];
    for (i=0;i<count;i++) {
        [split replaceObjectAtIndex:i withObject:
            [[[split objectAtIndex:i] stringByAppendingString:@" "] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]]];
    }
    return split;
}*/

- (NSArray *)splitString:(NSString *)string withDelimiter:(NSString *)delimiter {
	NSArray *split = [string componentsSeparatedByString:delimiter];
	NSMutableArray *splitCopy = [[split mutableCopy] autorelease];
	
	NSEnumerator *e = [split objectEnumerator];
	id obj;
	
	while (nil != (obj = [e nextObject])) {
		//NSString *spaceAddedString = [obj stringByAppendingString:@" "];	// unnecessary now that OS X 10.2 is no longer supported
		NSString *trimmedString = [obj stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
		unsigned idx = [split indexOfObjectIdenticalTo:obj];
		[splitCopy replaceObjectAtIndex:idx withObject:trimmedString];
	}

	return splitCopy;
}

#pragma mark -
#pragma mark Accessors

- (NSMutableArray *)lines {
	return [[AM_lines retain] autorelease];
}

- (void)setLines:(NSArray *)anArray {
	if (anArray != AM_lines) {
		[AM_lines release];
		AM_lines = [anArray mutableCopy];
	}
}

- (NSMutableDictionary *)conf {
	return [[AM_conf retain] autorelease];
}

- (void)setConf:(NSDictionary *)aDictionary {
	if (aDictionary != AM_conf) {
		[AM_conf release];
		AM_conf = [aDictionary mutableCopy];
	}
}

@end
