//
// HDPreferencesWindowController.h
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import "HDPreferencesWindowController.h"
//#import "Sparkle/SUConstants.h"

#import "NSWindow+AMAdditions.h"
#import "HDPreferencesConfigurationEditorGiFT.h"
#import "HDPreferencesConfigurationEditorUI.h"

#import "HDPreferencesAspectGeneral.h"
#import "HDPreferencesAspectDownloads.h"
#import "HDPreferencesAspectUploads.h"
#import "HDPreferencesAspectProtocols.h"
#import "HDPreferencesAspectIntegration.h"
#import "HDPreferencesAspectDaemon.h"
#import "HDPreferencesAspectPlugIns.h"

NSString *HDAspectGeneralKey = @"general";
NSString *HDAspectDownloadsKey = @"downloads";
NSString *HDAspectUploadsKey = @"uploads";
NSString *HDAspectProtocolsKey = @"protocols";
NSString *HDAspectIntegrationKey = @"integration";
NSString *HDAspectDaemonKey = @"daemon";
NSString *HDAspectPlugInsKey = @"plugIns";


@implementation HDPreferencesWindowController


/*+ (BOOL)shouldCheckForUpdatesAtStartup {
	return [[NSUserDefaults standardUserDefaults] boolForKey:SUCheckAtStartupKey];
}*/

+ (void)initialize {
	NSBundle *mainBundle = [NSBundle mainBundle];
	NSString *path = [mainBundle pathForResource:@"Defaults" ofType:@"plist"];
	NSDictionary *defaults = [[NSDictionary alloc] initWithContentsOfFile:path];

	NSMutableDictionary *standardDefaults = [[NSMutableDictionary alloc] init];
	NSDictionary *mainDefaults = [defaults objectForKey:@"mainDefaults"];
	NSMutableDictionary *sharedDefaults = [[defaults objectForKey:@"sharedDefaults"] mutableCopy];

//NSDictionary *checkForUpdatesDefault = [NSDictionary dictionaryWithObject:[NSNumber numberWithBool:YES] forKey:SUCheckAtStartupKey];
	//NSString *programName = [mainBundle objectForInfoDictionaryKey:(NSString *)kCFBundleNameKey];	// will return a localized version, if available

	[standardDefaults addEntriesFromDictionary:mainDefaults];
	[standardDefaults addEntriesFromDictionary:sharedDefaults];
	
	//[standardDefaults addEntriesFromDictionary:checkForUpdatesDefault];
	//[sharedDefaults addEntriesFromDictionary:checkForUpdatesDefault];
	
	// now, to register and set initial values...
	[[NSUserDefaults standardUserDefaults] registerDefaults:standardDefaults];
	[[NSUserDefaultsController sharedUserDefaultsController] setInitialValues:sharedDefaults];
	
	[defaults release];
	[standardDefaults release];
	[sharedDefaults release];
}

- (id)init {
	if (nil != (self = [super initWithWindowNibName:@"Preferences"])) {
		blankView = [[NSView alloc] initWithFrame:NSZeroRect];
		aspectControllers = [[NSMutableDictionary alloc] init];
		
		HDPreferencesAspect *aspectGeneral = [[HDPreferencesAspectGeneral alloc] init];
		HDPreferencesAspect *aspectDownloads = [[HDPreferencesAspectDownloads alloc] init];
		HDPreferencesAspect *aspectUploads = [[HDPreferencesAspectUploads alloc] init];
		HDPreferencesAspect *aspectProtocols = [[HDPreferencesAspectProtocols alloc] init];
		HDPreferencesAspect *aspectIntegration = [[HDPreferencesAspectIntegration alloc] init];
		HDPreferencesAspect *aspectDaemon = [[HDPreferencesAspectDaemon alloc] init];
		HDPreferencesAspect *aspectPlugIns = [[HDPreferencesAspectPlugIns alloc] init];
				
		[aspectControllers setObject:aspectGeneral forKey:HDAspectGeneralKey];
		[aspectControllers setObject:aspectDownloads forKey:HDAspectDownloadsKey];
		[aspectControllers setObject:aspectUploads forKey:HDAspectUploadsKey];
		[aspectControllers setObject:aspectProtocols forKey:HDAspectProtocolsKey];
		[aspectControllers setObject:aspectIntegration forKey:HDAspectIntegrationKey];
		[aspectControllers setObject:aspectDaemon forKey:HDAspectDaemonKey];
		[aspectControllers setObject:aspectPlugIns forKey:HDAspectPlugInsKey];
		
		[aspectGeneral release];
		[aspectDownloads release];
		[aspectUploads release];
		[aspectProtocols release];
		[aspectIntegration release];
		[aspectDaemon release];
		[aspectPlugIns release];
	}
	return self;
}

- (void)dealloc {
	[aspectControllers release];
	aspectControllers = nil;

	[blankView release];
	blankView = nil;
	
    [super dealloc];
}

- (void)awakeFromNib
{
    NSToolbar *toolbar = [[NSToolbar alloc] initWithIdentifier:@"prefToolbar"];
    [toolbar setAllowsUserCustomization:NO];
    [toolbar setDisplayMode:NSToolbarDisplayModeIconAndLabel];
    [toolbar setDelegate:self];
    [[self window] setToolbar:toolbar];
	[[self window] center];
    
    [self switchToGeneral:self];
    
    [toolbar setSelectedItemIdentifier:@"general"];		
	[toolbar release];
}

- (NSArray *)toolbarSelectableItemIdentifiers:(NSToolbar*)toolbar
{
    return [NSArray arrayWithObjects:
        @"general",@"download",@"upload",@"integration",@"protos",@"daemon",@"plugins",
        nil];
}


- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar*)toolbar
{
    return [NSArray arrayWithObjects:
        @"general",@"download",@"upload",@"integration",@"protos",@"daemon",@"plugins",
        nil];
}

- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar*)toolbar
{
    return [NSArray arrayWithObjects:
        @"general",@"download",@"upload",@"integration",@"protos",@"daemon",@"plugins",
        nil];
}

- (NSToolbarItem *)toolbar:(NSToolbar *)toolbar itemForItemIdentifier:(NSString *)itemIdentifier willBeInsertedIntoToolbar:(BOOL)flag
{
    NSToolbarItem *item = [[NSToolbarItem alloc] initWithItemIdentifier:itemIdentifier];
    if ([itemIdentifier isEqualToString:@"general"]) {
        [item setLabel:@"General"];
        [item setPaletteLabel:@"General"];
        [item setImage:[NSImage imageNamed:@"General Preferences"]];
        [item setTarget:self];
        [item setAction:@selector(switchToGeneral:)];
    }
    else if ([itemIdentifier isEqualToString:@"daemon"]) {
        [item setLabel:@"Daemon"];
        [item setPaletteLabel:@"Daemon"];
        [item setImage:[NSImage imageNamed:@"Terminal"]];
        [item setTarget:self];
        [item setAction:@selector(switchToDaemon:)];
    }
    else if ([itemIdentifier isEqualToString:@"download"]) {
        [item setLabel:@"Downloads"];
        [item setPaletteLabel:@"Downloads"];
        [item setImage:[NSImage imageNamed:@"download"]];
        [item setTarget:self];
        [item setAction:@selector(switchToDownloadss:)];
    }
    else if ([itemIdentifier isEqualToString:@"upload"]) {
        [item setLabel:@"Uploads"];
        [item setPaletteLabel:@"Uploads"];
        [item setImage:[NSImage imageNamed:@"upload"]];
        [item setTarget:self];
        [item setAction:@selector(switchToUploads:)];
    }
    else if ([itemIdentifier isEqualToString:@"protos"]) {
        [item setLabel:@"Protocols"];
        [item setPaletteLabel:@"Protocols"];
        [item setImage:[NSImage imageNamed:@"connect"]];
        [item setTarget:self];
        [item setAction:@selector(switchToProtocols:)];
    }
    else if ([itemIdentifier isEqualToString:@"plugins"]) {
        [item setLabel:@"Plugins"];
        [item setPaletteLabel:@"Plugins"];
        [item setImage:[NSImage imageNamed:@"HUD"]];
        [item setTarget:self];
        [item setAction:@selector(switchToPlugIns:)];
    }
    else if ([itemIdentifier isEqualToString:@"integration"]) {
        [item setLabel:@"Integration"];
        [item setPaletteLabel:@"Integration"];
        [item setImage:[NSImage imageNamed:@"ipods"]];
        [item setTarget:self];
        [item setAction:@selector(switchToIntegration:)];
    }
    return [item autorelease];
}

/*- (NSRect)calcFrame:(float)height
{
    NSRect frame = [[self window] frame];
    NSRect content = [[[self window] contentView] frame];
    float dif = frame.size.height-content.size.height;
    frame.origin.y = frame.origin.y+content.size.height-height+1;
    frame.size.height = height+dif-1;
    return frame;
}*/

- (void)switchToGeneral:(id)sender {
	[self switchToAspect:HDAspectGeneralKey];
}


- (void)switchToDaemon:(id)sender {
	[self switchToAspect:HDAspectDaemonKey];
}


- (void)switchToDownloadss:(id)sender {
	[self switchToAspect:HDAspectDownloadsKey];
}


- (void)switchToUploads:(id)sender {
	[self switchToAspect:HDAspectUploadsKey];
}


- (void)switchToProtocols:(id)sender {
	[self switchToAspect:HDAspectProtocolsKey];
}

- (void)switchToPlugIns:(id)sender {
	[self switchToAspect:HDAspectPlugInsKey];
}

- (void)switchToIntegration:(id)sender {
	[self switchToAspect:HDAspectIntegrationKey];
}

- (void)switchToAspect:(NSString *)identifier {
	HDPreferencesAspect *controller = [aspectControllers objectForKey:identifier];
	if (controller) {
		NSWindow *window = [self window];
		[window setContentView:blankView];
		NSView *view = [controller view];
		[window resizeToSize:[view frame].size];
		[window setContentView:view];
	}
}

- (BOOL)validateToolbarItem:(NSToolbarItem *)theItem
{
    HDPreferencesConfigurationEditorGiFT *gift_conf = [HDPreferencesConfigurationEditorGiFT singleton];
    HDPreferencesConfigurationEditorUI *ui_conf = [HDPreferencesConfigurationEditorUI singleton];

    NSString *ident = [theItem itemIdentifier];
    if ([ident isEqualToString:@"general"] || [ident isEqualToString:@"daemon"]) return YES;
    else return (ui_conf!=nil) && (gift_conf!=nil);
}	

@end
