//
// HDPreferencesProtocolControllerOpenFT.m
// -------------------------------------------------------------------------
// Copyright (C) 2003 Poisoned Project <http://www.gottsilla.net/>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// ---------------------------------------------------------------------------
// Portions copyright (c) 2007 Andrew Merenbach <http://www.merenbach.com>
//	for Hedera <http://hedera.sourceforge.net/>.
// ---------------------------------------------------------------------------

#import "HDPreferencesProtocolControllerOpenFT.h"
#import "HDAdditions.h"


@implementation HDPreferencesProtocolControllerOpenFT

- (Class)configurationEditorClass {
	return [HDPreferencesConfigurationEditorOpenFT class];
}

- (void)awakeFromNib
{
    [self readConfFiles];
}


- (void)enable
{
        [alias setEnabled:YES];
        [port setEnabled:YES];
        [http_port setEnabled:YES];
        [nodeClass setEnabled:YES];
}

- (void)disable
{
        [alias setEnabled:NO];
        [port setEnabled:NO];
        [http_port setEnabled:NO];
        [nodeClass setEnabled:NO];
        [tabView selectTabViewItemAtIndex:0];
}

- (void)readConfFiles
{
	[super readConfFiles];
	
	id newConfigurationEditor = [self configurationEditor];
    if (newConfigurationEditor) {
		[self enable];
		
		[newConfigurationEditor read];
		
		[alias setStringValue:[newConfigurationEditor optionValueForKey:@"alias"]];
		
		[port setIntValue:[[newConfigurationEditor optionValueForKey:@"port"] intValue]];
		[http_port setIntValue:[[newConfigurationEditor optionValueForKey:@"http_port"] intValue]];

		int class = [[newConfigurationEditor optionValueForKey:@"class"] intValue];
		switch (class) {
			case 1:
				[nodeClass selectItemWithTitle:@"USER"];
				[tabView selectTabViewItemAtIndex:0];
				break;
			case 3:
				[nodeClass selectItemWithTitle:@"SEARCH"];
				[tabView selectTabViewItemAtIndex:1];
				break;
			case 5:
				[nodeClass selectItemWithTitle:@"INDEX"];
				[tabView selectTabViewItemAtIndex:0];
				break;
			case 7:
				[nodeClass selectItemWithTitle:@"SEARCH & INDEX"];
				[tabView selectTabViewItemAtIndex:1];
				break;
			default:break;
		}
		
		[maxChilds setIntValue:[[newConfigurationEditor optionValueForKey:@"children"] intValue]];
		[dbLocation setStringValue:[newConfigurationEditor optionValueForKey:@"env_path"]];

		int priv = [[newConfigurationEditor optionValueForKey:@"env_priv"] intValue];
		if (priv==0) [private_env setState:NSOffState];
		else [private_env setState:NSOnState];
		
		
		long long cache;
		[[NSScanner scannerWithString:[newConfigurationEditor optionValueForKey:@"env_cache"]] scanLongLong:&cache];
		[cacheSize setStringValue:[NSString stringWithFormat:@"%i MB",(int)(cache/HD_MEGABYTE)]];
		[totalCacheSize setIntValue:cache/HD_KILOBYTE];
	}
    else {
        [self disable];
    }
}

- (void)displayHelp:(NSString *)file title:(NSString *)title
{
    NSBundle *bundle = [NSBundle mainBundle];
    NSString *path = [bundle pathForResource:file ofType:@"rtf"];
    [helpTextView readRTFDFromFile:path];
    [helpPanel setTitle:title];
    [helpPanel makeKeyAndOrderFront:self];
}

- (IBAction)helpClasses:(id)sender
{
    [self displayHelp:@"OpenFTConfNodeClasses" title:@"OpenFT Help: Node Classes"];
}
- (IBAction)helpSEARCH:(id)sender
{
    [self displayHelp:@"OpenFTSEARCH" title:@"OpenFT Help: SEARCH Node"];
}
- (IBAction)helpPorts:(id)sender
{
    [self displayHelp:@"OpenFTPorts" title:@"OpenFT Help: Ports"];
}


// Had to change all actions for setAction: to newAction:
// more info here: http://cocoa.mamasam.com/MACOSXDEV/2002/09/2/46437.php
- (IBAction)newAlias:(id)sender
{
	// whitespace in user names causes problems
	// set to remove space, possible other whitespace characters can cause problems don't know yet
	// /t and /r are also considered whitespace but the input feilds don't accept those
	NSMutableString* nameString = [NSMutableString stringWithFormat:[alias stringValue]];
	NSRange entireString = NSMakeRange (0, [nameString length]);
	[nameString replaceOccurrencesOfString:@" " withString:@"_" options:NSLiteralSearch range:entireString];
	[[self configurationEditor] setOptionValue: nameString forKey:@"alias"];
	[self readConfFiles];
}

- (IBAction)newPort:(id)sender
{
    [[self configurationEditor] setOptionValue:[NSNumber numberWithInt:[sender intValue]] forKey:@"port"];
	[self readConfFiles];
}

- (IBAction)newHTTP_port:(id)sender
{
    [[self configurationEditor] setOptionValue:[NSNumber numberWithInt:[sender intValue]] forKey:@"http_port"];
	[self readConfFiles];
}

- (IBAction)newNodeClass:(id)sender
{
    [[self configurationEditor] setOptionValue:[NSNumber numberWithInt:([nodeClass indexOfSelectedItem]*2+1)] forKey:@"class"];
    [self readConfFiles];
}

// SEARCH node settings
- (IBAction)browseDBLocation:(id)sender
{
    NSOpenPanel *open = [NSOpenPanel openPanel];
    [open setCanChooseFiles:NO];
    [open setCanChooseDirectories:YES];
    int ret = [open 
        runModalForDirectory:NSHomeDirectory()
        file:nil
        types:nil
    ];
    if (ret==NSOKButton) {
        [[self configurationEditor] setOptionValue:[[[open filenames] objectAtIndex:0] stringByAbbreviatingWithTildeInPath] forKey:@"env_path"];
        [self readConfFiles];
    }
}

- (IBAction)changeDBLocation:(id)sender
{
        [[self configurationEditor] setOptionValue:[sender stringValue] forKey:@"env_path"];
        [self readConfFiles];
}

- (IBAction)newMaxChilds:(id)sender
{
    [[self configurationEditor] setOptionValue:[NSNumber numberWithInt:[sender intValue]] forKey:@"children"];
    [self readConfFiles];
}

- (IBAction)newPrivate:(id)sender
{
    if ([sender state]==NSOnState) [[self configurationEditor] setOptionValue:[NSNumber numberWithInt:1] forKey:@"env_priv"];
    else [[self configurationEditor] setOptionValue:[NSNumber numberWithInt:0] forKey:@"env_priv"];
    [self readConfFiles];
}

- (IBAction)newTotalCacheSize:(id)sender
{
    int cache = [sender intValue];
    [totalCacheSize setIntValue:cache];
    [cacheSize setStringValue:[NSString stringWithFormat:@"%i MB",(int)(cache/HD_KILOBYTE)]];
    if ([[[NSApplication sharedApplication] currentEvent] type] != NSLeftMouseDown) //write to conf file on mouse up
    {
        [[self configurationEditor] setOptionValue:[NSString stringWithFormat:@"%i",cache*HD_KILOBYTE] forKey:@"env_cache"];
        [self readConfFiles];
    }
}


@end
